////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////// MODULES FUNCTIONS


////// ZEUS INIT MODULE
//zeus init module, turns synched player characters into Zeus and adds synched units to its control

jac_fnc_moduleInitWargameZeus=
{
// Argument 0 is module logic.
_logic = param [0,objNull,[objNull]];
// Argument 1 is list of affected units (affected by value selected in the 'class Units' argument))
_units = param [1,[],[[]]];
// True when the module was activated, false when it is deactivated (i.e., synced triggers are no longer active)
_activated = param [2,true,[true]];
// Module specific behavior. Function can extract arguments from logic and use them.
	// Attribute values are saved in module's object space under their class names
	waitUntil { sleep 0.001; ( (CBA_missionTime > 1) && ( (!isNull player) or !(hasInterface) ) ) };	//wait until mission starts and player object is initialized
	sleep 0.5;
	
	///turn player into a wargame zeus	
	_playerArr= _units findif {(_x == player)};
	if (_playerArr != -1) then	
	{
		_forceInterface= _logic getVariable ["jac_isForcedWargameZeusInterface", true];	
		_canAddObjects= _logic getVariable ["jac_canAddObjectsVariable", true];
	
		_canEditUnitsZeus= _logic getVariable ["jac_canEditUnitsZeus", false];	
		_canPlaceUnitsZeus= _logic getVariable ["jac_canPlaceUnitsZeus", false];
	
		_canPlaceUnitsZeusResource= _logic getVariable ["jac_canPlaceUnitsZeusResource", 1000];
		_canPlaceUnitsZeusResource= 0 max _canPlaceUnitsZeusResource min 1000;	//truncate
	
		_canPlaceUnitsZeusPopulationcap= _logic getVariable ["jac_canPlaceUnitsZeusPopulationcap", 1000];
		_canPlaceUnitsZeusPopulationcap= 0 max _canPlaceUnitsZeusPopulationcap min 1000;	//truncate
		
		_allowNighvision= _logic getVariable ["jac_zeusEnableNV", true];
		_allowThermalVision= _logic getVariable ["jac_zeusEnableThermal", true];
		
		_zeusEnableScorebar= _logic getVariable ["jac_zeusEnableScorebar", true];
	
		///get camera settings	
		_cameraObj= objNull; _cameraAreaSize= 0; _cameraAreaCeiling= 0;
		if (_logic getVariable ["jac_cameraAreaActive", true]) then 
		{
			private _str= _logic getVariable ["jac_cameraFollowingObjectName", ""];
			if (_str == "") then {_cameraObj= _logic}
			else
			{_cameraObj= missionNamespace getVariable _str};
			if ((toLower _str) == "player") then {_cameraObj= player};
			
			_cameraAreaSize= _logic getVariable ["jac_cameraAreaSize", 1000];
			_cameraAreaCeiling= _logic getVariable ["jac_cameraAreaCeiling", 250];
		};
		
		_objectsToAdd= _units select {!(isPlayer _x) && !(_x in playableUnits)};
		
		[player, _canPlaceUnitsZeus, _canEditUnitsZeus, _forceInterface, _cameraObj, _cameraAreaSize, _cameraAreaCeiling, _objectsToAdd, _canPlaceUnitsZeusResource, _canPlaceUnitsZeusPopulationcap, _logic, _canAddObjects, _allowNighvision, _allowThermalVision, _zeusEnableScorebar] remoteExec ["jac_fnc_turnPlayerIntoWargameZeus", 2];
	};
true;	// Module function is executed by spawn command, so returned value is not necessary, but it is good practice.
};

jac_fnc_turnPlayerIntoWargameZeus=
{
	params ["_player", "_canPlaceUnitsZeus", "_canEditUnitsZeus", "_forceInterface", "_cameraObj", "_cameraAreaSize", "_cameraAreaCeiling", "_objectsToAdd", "_canPlaceUnitsZeusResource", "_canPlaceUnitsZeusPopulationcap", "_logic", "_canAddObjects", "_allowNighvision", "_allowThermalVision", "_zeusEnableScorebar"];

	_curatorModuleOld = getAssignedCuratorLogic _player; unassignCurator _curatorModuleOld; deleteVehicle _curatorModuleOld; 
	_curatorModuleGrp = createGroup sideLogic; _curatorModuleGrp deleteGroupWhenEmpty true;
	_curatorModule = _curatorModuleGrp createUnit ["ModuleCurator_F", [0,0,0], [], 0, "CAN_COLLIDE"];
	
	_curatorModule setVariable ['BIS_fnc_initModules_disableAutoActivation', false, true];
	_curatorModule setVariable ["Owner", (getPlayerUID _player), true];
	sleep 0.5;
	
	_player assignCurator _curatorModule;
	_curatorModule addCuratorEditableObjects [_objectsToAdd, true];
	sleep 0.5;
	
	//initialize WRG mode for the added units 
	_unitsArray= _objectsToAdd select {(_x isKindOf "CAManBase")};
	if (count _unitsArray > 0) then 
	{
		[[_unitsArray],
		{
			params ['_unitsArray'];
			{if (local _x) then {[_x, (group _x)] call jac_fnc_initWRGunit}}forEach _unitsArray;
		}] remoteExecCall ["call", 0];
	};
	
	_curatorModule setCuratorEditingAreaType true;	_curatorModule allowCuratorLogicIgnoreAreas false;
	
	/// set zeus editing parameters
	_atributeArr= [];
	
	if !(_canPlaceUnitsZeus) then {_atributeArr pushBack "Place"};
	
	if !(jac_isPVPmatch) then { {_curatorModule setCuratorCoef [_x, -1e10]}forEach _atributeArr };
	
	_curatorModule setVariable ['jac_canEditUnitsZeus', _canEditUnitsZeus, true];
	_curatorModule setVariable ['jac_canPlaceUnitsZeus', _canPlaceUnitsZeus, true];
	
	//zeus allowed vision modes
	_curatorModule setVariable ['jac_zeusEnableNV', _allowNighvision, true];
	_curatorModule setVariable ['jac_zeusEnableThermal', _allowThermalVision, true];
	
	/// zeus unit creation resource
	_curatorModule addCuratorPoints (curatorPoints _curatorModule * -1);	//reset to 0 and set new value
	sleep 0.1; _curatorModule addCuratorPoints (_canPlaceUnitsZeusResource * 0.001);
	
	//camera following routine
	if !(isNull _cameraObj) then
	{
		_varName= ("jac_cameraObj" + (str _curatorModule));
		missionNamespace setVariable [_varName, _cameraObj, true];
		_cameraObj setVariable ["jac_isCameraObj", true, true];
		_curatorModule setCuratorCameraAreaCeiling _cameraAreaCeiling;
		
		[_cameraAreaSize, _cameraObj, _curatorModule, _varName] spawn {
			params ["_cameraAreaSize", "_cameraObj", "_curatorModule", "_varName"];
			_oldPos= [0,0,-1];
			while {true} do {
				sleep 1;
				_cameraObj= (missionNamespace getVariable [_varName, objNull]);
				if !(alive _cameraObj) then {continue};
				if !((position _cameraObj) isEqualTo _oldPos) then {_oldPos= (position _cameraObj); _curatorModule addCuratorCameraArea [1, (position _cameraObj), _cameraAreaSize]};
			};
		};
	};

	//make sure that zeus interface is opened
	[_forceInterface, _logic, _canAddObjects, _canPlaceUnitsZeusPopulationcap, _zeusEnableScorebar] remoteExec ["jac_fnc_WargameZeusStartInterface", (owner _player)];
};

//open zeus interface
jac_fnc_WargameZeusStartInterface=
{
	params ["_forceInterface", "_logic", "_canAddObjects", "_canPlaceUnitsZeusPopulationcap", "_zeusEnableScorebar"];
	
	waitUntil { sleep 0.001; ( !(isNull (getAssignedCuratorLogic player)) ) };	//wait until player has a zeus module
	
	if (didJIP) then
	{
		_profileName= profileName; _clientOwner= clientOwner;
		[_profileName, _clientOwner] remoteExec ["jac_fnc_zeusReconnectGetData", 2];
		sleep 1;
	};

	waitUntil { sleep 0.001; ( (jac_fnc_moduleUnitCostWargameZeusActive) ) };	//wait until unit cost modules are finished
	
	sleep 0.5;

	//set zeus rules
	jac_forceNoDirectControl= (_logic getVariable ["jac_canRemoteControlUnits", true]);
	jac_forceAmmoUseSetting= (_logic getVariable ["jac_forceAmmoUse", false]);
	jac_canUseWargameCheatsSetting= !(_logic getVariable ["jac_blockWargameCheats", true]);
	jac_wargameZeusAny= false;	//prevent the player from using the wargame zeus anywhere to cheat
	jac_canAddObjects= _canAddObjects;	//if the ability to add friendly units and empty vehicles is on
	jac_zeusUnitsPopulationcap= _canPlaceUnitsZeusPopulationcap; //player local var to set zeus population cap; 0 means no pop limit
	
	player setVariable ["jac_forceInterface", _forceInterface];

	/// set the custom unit creation list event handler
	_curatorModule= (getAssignedCuratorLogic player);
	_curatorModule addEventHandler [
	"CuratorObjectRegistered",
	{
		if ((count jac_zeusUnitCost > 0) or !((getAssignedCuratorLogic player) getVariable ["jac_canPlaceUnitsZeus", true])) then
		{
			_classes = _this select 1; _costs = [];
			{
				_unitClass= _x;
				_id= jac_zeusUnitCost findif { if ((typeName _x) == 'STRING') then {_unitClass == _x} else {false} };
				if ((_id != -1) && ((typeName _id) != 'STRING') ) then { _costs pushBack [true, (( jac_zeusUnitCost # (_id +1) )), (( jac_zeusUnitCost # (_id +1) )) ] } else { _costs pushBack [false, 0, 0] }; //_costs pushBack [false, 0]
			} forEach _classes; // go through all classes and assign cost for each of them from the jac_zeusUnitCost list 
			_costs;
		};
	}];

	if (_forceInterface) then
	{
		sleep 0.5;
		
		//if interface was open already before zeus wargame was properly set up, close and open interface again
		if !(isNull findDisplay 312) then {findDisplay 312 closeDisplay 2; sleep 0.25};
		
		[_forceInterface, true] spawn BIS_fnc_forceCuratorInterface;
		for "_i" from 0 to 4 step +1 do {
			sleep 1; if (isNull findDisplay 312) then {openCuratorInterface} else {break}; //try to make sure that zeus interface is opened - MP fix
		};
	};
	
	/// set atributes edit box
	[_curatorModule , "object", []] call BIS_fnc_setCuratorAttributes;
	[_curatorModule , "group", ["GroupID", "Formation"]] call BIS_fnc_setCuratorAttributes;
	[_curatorModule , "waypoint", ["WaypointType", "WaypointTimeout", "Formation"]] call BIS_fnc_setCuratorAttributes;
	[_curatorModule , "marker", ["MarkerText", "MarkerColor", "Formation"]] call BIS_fnc_setCuratorAttributes;
	
	//start UI routine (show score and popcap)
	sleep 1;
	if ( (jac_isWargameZeus) && !(jac_wargameMatchGUIActive) && (_zeusEnableScorebar) ) then {[] call jac_fnc_wargameMatchGUI};
};

//when a player reconnects to a match get the unit cost data module back - must be remoteExec on the server
jac_fnc_zeusReconnectGetData=
{
	params ['_name', '_owner'];
	
	_data= missionNamespace getVariable ["jac_playerZeusReconnectData" + _name, []];
	if (count _data > 0) then 
	{
		[[_data, _name],
		{
			params ['_data', '_name'];
			missionNamespace setVariable ["jac_playerZeusReconnectData" + _name, _data];
			
			_data params ['_curatorEditableObjects', '_curatorPoints', '_side', '_zeusUnitsAddedPopList', '_zeusUnitsAddedPopCost', '_zeusUnitCostModuleCustomLoadout', '_zeusUnitCost', '_virtualAirSupportData'];
			if (_side == (side player)) then
			{
				_curator= (getAssignedCuratorLogic player);
				if !(isNull _curator) then
				{
					[_curator, [_curatorEditableObjects, true]] remoteExec ["addCuratorEditableObjects", 2];
				
					[_curator, _curatorPoints] spawn 
					{
						params ['_curator', '_curatorPoints'];
						_removeCuratorPoints= (curatorPoints _curator * -1);
						[_curator, _removeCuratorPoints] remoteExecCall ["addCuratorPoints", 2];
						sleep 0.5;
						[_curator, _curatorPoints] remoteExecCall ["addCuratorPoints", 2];
					}; //spawn
				
					jac_zeusUnitsAddedPopList= _zeusUnitsAddedPopList;
					player setVariable ["jac_zeusUnitsAddedPopListVar", jac_zeusUnitsAddedPopList, true];
				
					jac_zeusUnitsAddedPopCost= _zeusUnitsAddedPopCost;
					player setVariable ["jac_zeusUnitsAddedPopCostVar", jac_zeusUnitsAddedPopCost, true];

					//unit cost data
					jac_zeusUnitCost= _zeusUnitCost;
					player setVariable ["jac_zeusUnitCostVar" + str (side player), _zeusUnitCost, true];		
					{ if (_forEachIndex mod 2 == 0) then {_x call jac_fnc_prepareClassAddons} }foreach _zeusUnitCost;

					jac_zeusUnitCostModuleCustomLoadout= _zeusUnitCostModuleCustomLoadout;
					player setVariable ["jac_zeusUnitCostModuleCustomLoadoutVar" + str (side player), _zeusUnitCostModuleCustomLoadout, true];	//unit loadout data

					jac_fnc_moduleUnitCostWargameZeusActive= true;
					
					player setvariable ["jac_virtualAirSupportData", _virtualAirSupportData, true];
				};
			};
		}] remoteExec ["call", _owner];
	};
};

///////// SET UNIT CREATION COST MODULE
//variable not shared on the network used to regist units to the specific zeus player
jac_zeusUnitCost= [];	//["_className", 0.1, "_className", 0.1]
jac_zeusUnitCostModuleCustomLoadout= [];	//save the loadouts of the units and apply them when they are placed by zeus

jac_fnc_moduleUnitCostWargameZeusActive= true;

jac_fnc_moduleUnitCostWargameZeus=
{
// Argument 0 is module logic.
_logic = param [0,objNull,[objNull]];
// Argument 1 is list of affected units (affected by value selected in the 'class Units' argument))
_units = param [1,[],[[]]];
// True when the module was activated, false when it is deactivated (i.e., synced triggers are no longer active)
_activated = param [2,true,[true]];

jac_fnc_moduleUnitCostWargameZeusActive= false;	//the server will turn this on after it has processed all unit's cost modules, then the zeus init module will enter zeus interface
if !(isServer) then {_logic setVariable ["jac_moduleUnitCostWargameZeusInitialized", true]};

// Module specific behavior. Function can extract arguments from logic and use them.
if ((_activated) && (isServer) ) then
{
	waitUntil {sleep 0.001; (CBA_missionTime > 0)};	//wait until mission starts
	
	//{_unit= (vehicle _x); _unit hideObjectGlobal true; _unit enableSimulationGlobal false}forEach _units; //hide synched units
	
	// Attribute values are saved in module's object space under their class names
	_cost= _logic getVariable ["jac_canPlaceUnitsZeusCost", 0];  _cost= _cost * 0.001;
	_cost= 0 max _cost min 1;	//truncate
	
	_populationCost= _logic getVariable ["jac_canPlaceUnitsZeusPopulationCost", 5];
	_populationCost= 0 max _populationCost min 1000;	//truncate
	
	_unitCostModuleZeusReward= _logic getVariable ["jac_unitCostModuleZeusReward", 5];
	_unitCostModuleZeusReward= 0 max _unitCostModuleZeusReward min 1000;	//truncate
	
	_unitCostModuleVPReward= _logic getVariable ["jac_unitCostModuleVPReward", 5];
	_unitCostModuleVPReward= 0 max _unitCostModuleVPReward min 100;	//truncate
	
	_addCrew= _logic getVariable ["jac_unitCostModuleAddCrew", true];
	
	_saveLoadout = _logic getVariable ["jac_unitCostModuleSaveLoadout", true];
	
	_startInAir = _logic getVariable ["jac_unitCostModuleStartInAir", false];
	
	_unitCostModuleSyncedZeusOwner= _logic getVariable ["jac_unitCostModuleSyncedZeusOwner", false];
	_unitCostModuleSyncedZeusOwnerList= [];
	
	if (_unitCostModuleSyncedZeusOwner) then 
	{
		{
			if ((typeOf _x) == "jac_moduleInitWargameZeus") then
			{
				_curatorInit= _x;
				_unitCostModuleSyncedZeusOwnerList insert [-1, (synchronizedObjects _curatorInit select { ( (isPlayer _x) or (_x in playableUnits) ) }), true];
			};
		}forEach (synchronizedObjects _logic);
	};
	
	_sideId= _logic getVariable ["jac_setModuleOwner", 0];
	_side= ((jac_wargameSidesData # _sideId) # 0);
	
	_code= _logic getVariable ["jac_costModuleUnitInitCode", '']; //to set a custom init code for objects of the same class - use _this to reference the unit
	
	{
		_crew= (crew _x); _unit= _crew # 0; if ((count _crew) == 0) then {_unit= _x};
		if !(isNull (objectParent _unit)) then {_unit= objectParent _unit};

		_myType= (typeOf _unit);
		if (_myType == 'jac_moduleInitWargameZeus') then {continue};	//only valid units
		
		[_side, _myType, _cost, _unitCostModuleSyncedZeusOwner, _unitCostModuleSyncedZeusOwnerList] remoteExec ["jac_fnc_zeusUnitCostSetArrayData", 0, true];	//send data to all curators and JIP
		
		/// save custom unit data
		_loadoutData= []; _unitTraits= [];
		
		/// if is an infantry unit save the loadout for the class
		if (_myType isKindOf 'CAManBase') then
		{
			_loadoutData= (getUnitLoadout _unit);
			
			//save unit traits
			_unitTraits = getAllUnitTraits _unit;
		};
		
		/// if is an air unit save the pylon data for the class
		if (_myType isKindOf 'Air') then {_loadoutData= (getAllPylonsInfo _unit)};
		
		/// save vehicle inventory and appareance
		_vehicleInventoryData= []; _vehicleAppearanceData= [];
		if (_saveLoadout) then
		{
			if !(_myType isKindOf 'CAManBase') then
			{
				_vehicleInventoryData= (weaponCargo _unit);
				_vehicleInventoryData insert [-1, (magazineCargo _unit)];
				_vehicleInventoryData insert [-1, (itemCargo _unit)];
				{
					if (_x # 0 isKindOf "Bag_Base") then {_vehicleInventoryData insert [-1, [_x # 0]]};
				}forEach (everyContainer _unit);
				
				_id= format ["jac_vehicleAppearance_%1", _myType];
				_vehicleAppearanceData= [_unit, [missionNamespace, _id], [], false] call BIS_fnc_saveVehicle;
			};
		};
		
		/// get all init trait modules synched to the unit
		_unitFunction= [];
		{ if (count (_x getVariable ["jac_moduleInitTraitFunctionData", []]) > 0) then {_unitFunction pushBack (_x getVariable ["jac_moduleInitTraitFunctionData", []])} }forEach (synchronizedObjects _unit); //will get the necessary data to run a group trait init module function
		
		/// to save Zeus points or victory points reward from enemy units or when module is assigned to specific zeus
		_forceSave= ( (_unitCostModuleZeusReward > 0) or (_unitCostModuleVPReward > 0) or (_unitCostModuleSyncedZeusOwner) );
		
		_dataArray= [_myType, _loadoutData, _code, _addCrew, _saveLoadout, _populationCost, _cost, _startInAir, _unitCostModuleZeusReward, _unitCostModuleVPReward, _vehicleInventoryData,
		_vehicleAppearanceData, _unitTraits, _unitFunction, _unitCostModuleSyncedZeusOwner, _unitCostModuleSyncedZeusOwnerList];	//15
		[_side, _dataArray, _forceSave] remoteExec ["jac_fnc_zeusUnitCostModuleCustomLoadoutSetArrayData", 0, true];	//send data to all curators and JIP

	}forEach _units;
	
	//delete all synched units
	_logic setVariable ["jac_moduleUnitCostWargameZeusInitialized", true];
	
	if (allUnits findif { ( ((typeOf _x) isEqualTo "jac_moduleUnitCostWargameZeus") && !(_x getVariable ["jac_moduleUnitCostWargameZeusInitialized", false]) ) } == -1) then	//if all unit cost modules finished initializing
	{
		{deleteVehicleCrew (vehicle _x); deleteVehicle (vehicle _x)}forEach _units;
		[] remoteExecCall ["jac_fnc_moduleUnitCostWargameZeusActiveSetValue", 0, true];
	};
};
true;	// Module function is executed by spawn command, so returned value is not necessary, but it is good practice.
};

//functions for JIP compatibility
jac_fnc_zeusUnitCostSetArrayData=
{
	params ["_side", "_unitType", "_cost", "_unitCostModuleSyncedZeusOwner", "_unitCostModuleSyncedZeusOwnerList"];
	
	waitUntil { sleep 0.001; ( (!isNull player) or !(hasInterface) ) };	//wait until mission starts and player object is initialized

	if ( ( !(_unitCostModuleSyncedZeusOwner) && ((side player) == _side) ) or (player in _unitCostModuleSyncedZeusOwnerList) ) then
	{
		if ((jac_zeusUnitCost pushBackUnique _unitType) != -1) then 
		{
			jac_zeusUnitCost pushBack _cost;
			_unitType call jac_fnc_prepareClassAddons;
		};
	};
};

jac_fnc_zeusUnitCostModuleCustomLoadoutSetArrayData=
{
	params ["_side", "_dataArray", "_forceSave"];
	
	waitUntil { sleep 0.001; ( (!isNull player) or !(hasInterface) ) };	//wait until mission starts and player object is initialized
	
	_unitCostModuleSyncedZeusOwner= _dataArray # 14; _unitCostModuleSyncedZeusOwnerList= _dataArray # 15;
	if ( ( !(_unitCostModuleSyncedZeusOwner) && ((side player) == _side) ) or (_forceSave) or (player in _unitCostModuleSyncedZeusOwnerList) ) then {jac_zeusUnitCostModuleCustomLoadout pushBack _dataArray};
};

//server inform players that the unit cost module finished processing unit data
jac_fnc_moduleUnitCostWargameZeusActiveSetValue=
{
	[{(allUnits findif { ( ((typeOf _x) isEqualTo "jac_moduleUnitCostWargameZeus") && !(_x getVariable ["jac_moduleUnitCostWargameZeusInitialized", false]) ) } == -1)}, {
		jac_fnc_moduleUnitCostWargameZeusActive= true;
		
		//for reconnection; unit cost data
		player setVariable ["jac_zeusUnitCostVar" + str (side player), jac_zeusUnitCost, true];
		player setVariable ["jac_zeusUnitCostModuleCustomLoadoutVar" + str (side player), jac_zeusUnitCostModuleCustomLoadout, true];	
	}, []] call CBA_fnc_waitUntilAndExecute;
};


////////////// WARGAME RULES MODULE
//set parameters for the PVP wargame mode and runs the game routine
jac_missionPhase= 0;	//tracks the current game phase; initPhase= 0; preparationPhase= 1; combatPhase= 2; endPhase= 3;
jac_preparationPhaseTimer= 0;
jac_victoryPointsToWin= 100;
jac_customVictoryCondition= "false";
jac_doEndMatchNow= false;

jac_fnc_moduleWargameRules=
{
// Argument 0 is module logic.
_logic = param [0,objNull,[objNull]];
// Argument 1 is list of affected units (affected by value selected in the 'class Units' argument))
_units = param [1,[],[[]]];
// True when the module was activated, false when it is deactivated (i.e., synced triggers are no longer active)
_activated = param [2,true,[true]];
// Module specific behavior. Function can extract arguments from logic and use them.
if (_activated) then 
{
	civilian setFriend [east, 1]; civilian setFriend [west, 1]; civilian setFriend [independent, 1];	//for the civilian spectators to work
	
	// Attribute values are saved in module's object space under their class names
	
	//set global variables - paremeters
	jac_isPVPmatch= true;
	jac_victoryPointsToWin= (_logic getVariable ["jac_victoryPointsToWin", 100]);
	jac_preparationPhaseTimer= (_logic getVariable ["jac_preparationPhaseTimer", 30]);
	jac_matchGameTime= (_logic getVariable ["jac_matchGameTime", 1800]);
	jac_endGameWhenSideIsDeadVar= (_logic getVariable ["jac_endGameWhenSideIsDead", false]);
	jac_customVictoryCondition= (_logic getVariable ["jac_customVictoryCode", "false"]);
	
	waitUntil {sleep 0.001; (CBA_missionTime > 1)};	//wait until mission starts
	
	//execute main routine
	if (isServer) then
	{
		if (jac_preparationPhaseTimer > 0) then
		{
			[1] remoteExecCall ["jac_fnc_setMissionPhase", 0, true];	//start preparation Phase
			[["PREPARATION PHASE START!", "PLAIN"]] remoteExec ["titleText", 0];
			
			///countdown
			for "_i" from jac_preparationPhaseTimer to 0 step -1 do {
				sleep 1; _msg= format ["PREPARATION PHASE!\n    %1", _i];
				[[_msg, "PLAIN"]] remoteExec ["titleText", 0];
			};
			
			///start combat phase
			sleep 1; [2] remoteExecCall ["jac_fnc_setMissionPhase", 0, true];	//start combat phase
		}
		else	//enter preparationPhase or skip to combatPhase
		{
			sleep 1; [2] remoteExecCall ["jac_fnc_setMissionPhase", 0, true];	//start combat phase
		};
	};
	
	waitUntil {sleep 0.001; (jac_missionPhase == 2)};	//wait until combat phase starts
	titleText ["COMBAT PHASE STARTED!", "PLAIN"];
	
	//start main routine for server
	if (isServer) then {[] call jac_fnc_wargameMatchRoutine};
	
	//area to prevent all unintended editing
	_curator= (getAssignedCuratorLogic player);	
	if !(_curator getVariable ["jac_canPlaceUnitsZeus", true]) then {[_curator, [101, [0,0,0], 0]] remoteExec ["addCuratorEditingArea", 2]};	
};

true;	// Module function is executed by spawn command, so returned value is not necessary, but it is good practice
};


/////  SIDES INDEX
// to track how many victory points each side has and side status
jac_wargameSidesCtrl= []; jac_wargameMatchRoutine= true; _isDead= false;
jac_wargameSidesData= [[blufor, 0, "jac_bluforVictory", _isDead, [0.5,0.8,0.95,0.9], "colorBLUFOR", "BLUFOR"], [opfor, 0, "jac_opforVictory", _isDead, [1,0,0,0.9], "colorOPFOR", "REDFOR"], 
[independent, 0, "jac_indforVictor", _isDead, [0,1,0,0.9], "colorIndependent", "GREENFOR"], [sideEmpty, 0, "jac_drawGame", _isDead, [1,0.35,0,0.9], "ColorOrange", "NEUTRAL"]];


////draw score on top of the screen if the mouse is near it
jac_fnc_wargameMatchGUI=
{
	jac_wargameMatchGUIActive= true;
	jac_wargameSidesCtrl= [];
	_img= "\jac_arma_wargame\data\images\jac_moduleObjArea.paa";
	_posX= ((safeZoneW * 0.25) + safeZoneX); _posY= ((safeZoneH * 0.05775) + safeZoneY); _w= (safeZoneW * 0.03725); _h= (safeZoneH * 0.0228); _oldPoints= [];

	///TITLE
	_ctrlT = findDisplay 312 ctrlCreate ["ctrlButton", -1];
	_ctrlT ctrlSetPosition [_posX - (_w * 0.05) , _posY - ( _h + (_h * 0.1)), (_w * 3) + (_w * 0.05), _h + (_h * 0.1)];
	_ctrlT ctrlSetText "Victory Points";
	_ctrlT ctrlSetFont jac_ctrlFont;
	_ctrlT ctrlCommit 0;	

	//background
	_ctrlB = findDisplay 312 ctrlCreate ["ctrlButton", -1];
	_ctrlB ctrlSetPosition [_posX - (_w * 0.05) , _posY - (_h * 0.025), (_w * 3) + (_w * 0.05), _h + (_h * 0.1)];
	_ctrlB ctrlEnable false;
	_ctrlB ctrlCommit 0;	
	
	//flags with score
	for "_i" from 0 to 2 step +1 do {
		_color= ((jac_wargameSidesData # _i) # 4);	_points= ((jac_wargameSidesData # _i) # 1);
		
		_ctrl = findDisplay 312 ctrlCreate ["RscButton", -1];
		_ctrl ctrlSetPosition [(_posX + (_w * _i)) , _posY, _w * 0.9, _h];
		_ctrl ctrlSetText str _points;
		_ctrl ctrlSetBackgroundColor _color; _ctrl ctrlSetTextColor [1,1,1,1];
		_ctrl ctrlSetTooltip format ["%1 Victory Points, reach %2 points to win", ((jac_wargameSidesData # _i) # 6), jac_victoryPointsToWin];
		_ctrl ctrlCommit 0;
		jac_wargameSidesCtrl pushBack _ctrl;
		_oldPoints pushBack _points;
	};
	
	jac_wargameSidesCtrl pushBack _ctrlT;
	jac_wargameSidesCtrl pushBack _ctrlB;
	
	//zeus points number	
	_control= ((findDisplay 312) displayCtrl 16806);
	_pos = ctrlPosition _control;
	
	_ctrl = findDisplay 312 ctrlCreate ["RscButton", -1];
	_ctrl ctrlSetPosition [(_pos # 0) + (_pos # 2) - ((_pos # 2) * 0.05), (_pos # 1), ((_pos # 2) * 0.05), (_pos # 3) *0.675];
	_curatorPoints= curatorPoints (getAssignedCuratorLogic player) * 1000;
	_ctrl ctrlSetText str _curatorPoints;
	_ctrl ctrlSetTextColor [1,1,1,1];
	_ctrl ctrlSetBackgroundColor (ctrlBackgroundColor _control);
	//_ctrl ctrlEnable false;
	_ctrl ctrlCommit 0;
	jac_wargameZeuspointsCtrl= _ctrl;

	//keep track of the mouse position and update score data and visibility
	_handler= [{
		(_this # 0) params ['_oldPoints'];
		
		getMousePosition params ["_mouseX", "_mouseY"];
		if ( (_mouseY < ((safeZoneH * 0.16) + safeZoneY)) or (visibleMap) ) then
		{
			//display score data
			_countArr= ((count jac_wargameSidesCtrl) -1);
			for "_i" from 0 to _countArr do {
				_ctrl= (jac_wargameSidesCtrl # _i);
				if !(ctrlShown _ctrl) then {_ctrl ctrlShow true};
				_points= ((jac_wargameSidesData # _i) # 1);
				if (isNil '_points') then {continue};
				if (_points != (_oldPoints # _i)) then {_ctrl ctrlSetText str _points; _oldPoints set [_i, _points]; _ctrl ctrlCommit 0};
			};
		}
		else
		{
			for "_i" from 0 to (count jac_wargameSidesCtrl) -1 step +1 do {
				_ctrl= (jac_wargameSidesCtrl # _i);
				if (ctrlShown _ctrl) then {_ctrl ctrlShow false};
			};
		};
		
		//zeus points number
		_curatorPoints= curatorPoints (getAssignedCuratorLogic player) * 1000;
		jac_wargameZeuspointsCtrl ctrlSetText str _curatorPoints;
		jac_wargameZeuspointsCtrl ctrlShow (ctrlShown ((findDisplay 312) displayCtrl 16806));
		
		/// display population cap data
		// check if entity selected in create menu has a popcap cost and display it
		if (jac_zeusUnitsPopulationcap > 0) then
		{
			_ctrl= focusedCtrl (findDisplay 312);
			_myTypeOf= (_ctrl tvData (tvSelection _ctrl # 0));
			if !(isNil "_myTypeOf") then
			{
				if ((typeName _myTypeOf) == "STRING") then
				{
					_findType= jac_zeusUnitCostModuleCustomLoadout findIf {_x # 0 isEqualTo _myTypeOf};
					if (_findType != -1) then
					{
						_populationCost= (jac_zeusUnitCostModuleCustomLoadout # _findType # 5);
						if (_populationCost > 0) then 
						{
							_line1= "POPULATION CAPACITY:";
							_msg= format ["SELECTION COST: %1								  POPULATION: %2 / %3", _populationCost, jac_zeusUnitsAddedPopCost, jac_zeusUnitsPopulationcap, _curatorPoints];
						
							_unitCostModuleVPReward= (jac_zeusUnitCostModuleCustomLoadout # _findType # 9);
							_unitCostModuleZeusReward= (jac_zeusUnitCostModuleCustomLoadout # _findType # 8);
							if ( (_unitCostModuleVPReward > 0) or (_unitCostModuleZeusReward > 0) ) then
							{
								_msg= format ["%1											 REWARD: VPs: %2 / ZEUS: %3", _msg, _unitCostModuleVPReward, _unitCostModuleZeusReward];
							};
							[_line1, _msg, 3] call BIS_fnc_curatorHint;
						};
					};
				};
			};
		};
		if ( !(jac_wargameMatchRoutine) or (isNull findDisplay 312)) then {[(_this # 1)] call CBA_fnc_removePerFrameHandler};
	
	} , 0.25, [_oldPoints]] call CBA_fnc_addPerFrameHandler;
};

//// main routine
jac_fnc_wargameMatchRoutine=
{

missionNamespace setVariable ['jac_sidesDead', 0];
missionNamespace setVariable ['jac_sideLastAlive', []];

_customVictoryCondition= compile jac_customVictoryCondition;
jac_customVictoryCondition= _customVictoryCondition;

[{
	
	//keep track of winning side by points
	_sideWinner= (jac_wargameSidesData # 3); _doEndMatch= false;
	{
		_side= (_x # 0); if (_side isEqualTo sideEmpty) then {continue};
		
		if ((_x # 1) >= jac_victoryPointsToWin) then {(jac_wargameSidesData # _forEachIndex) set [1, jac_victoryPointsToWin]; _doEndMatch= true};  //this side reached the amount of VPs needed to win
		
		if ((_x # 1) > (_sideWinner # 1)) then {_sideWinner= _x};	//this side has more VPs than the current winning side
		
		if ( !(_side isEqualTo (_sideWinner # 0)) && ((_x # 1) > 0) && ((_x # 1) == (_sideWinner # 1)) ) then {_sideWinner= ["DRAW GAME", 0, "jac_drawGame"]};	//this side has the same VPs as the winning side
	}forEach jac_wargameSidesData;
	
	//check sides status - jac_victoryPointsToWin
	if ( (CBA_missionTime > (jac_preparationPhaseTimer +30)) && (jac_missionPhase == 2) ) then
	{
		if (jac_endGameWhenSideIsDeadVar) then
		{
			missionNamespace setVariable ['jac_sidesDead', 0];
			{
				_side= (_x # 0);
				if (_side isEqualTo sideEmpty) then {continue};
				_sideUnits= allUnits select {( (alive _x) && ((side _x) isEqualTo _side) && !(isPlayer _x) )};
				
				//if only the player character is left alive the side is considered dead
				if (count _sideUnits == 0) then{ (jac_wargameSidesData # _forEachIndex) set [3, true]; missionNamespace setVariable ['jac_sidesDead', ((missionNamespace getVariable ['jac_sidesDead', 0]) +1) ]; }
				else
				{(jac_wargameSidesData # _forEachIndex) set [3, true]; missionNamespace setVariable ['jac_sideLastAlive', _x];};
			}forEach jac_wargameSidesData;
			
			//if only 1 or no sides left alive, end the match
			switch (missionNamespace getVariable ['jac_sidesDead', 0]) do
			{
				case 2: {_sideWinner= (missionNamespace getVariable ['jac_sideLastAlive', []]); _doEndMatch= true};
				case 3: {_sideWinner= ["DRAW GAME", -1, "jac_drawGame"]; _doEndMatch= true};
			};
		};
		
		//check custom victory condition
		if (call jac_customVictoryCondition) then {_doEndMatch= true};
	};
	
	//mission ends in 5 minutes message
	if ( ((jac_matchGameTime - CBA_missionTime) <= 300) && (missionNamespace getVariable ['jac_matchGameTimeMessage', true]) ) then 
	{missionNamespace setVariable ['jac_matchGameTimeMessage', false]; _msg= format ["MATCH ENDS IN 5 MINUTES!", (jac_matchGameTime - CBA_missionTime)]; [[_msg, "PLAIN"]] remoteExec ["titleText", 0]};
	
	//mission countdown
	if ( ((jac_matchGameTime - CBA_missionTime) < 50) && ((jac_matchGameTime - CBA_missionTime) >= 0) )  then {_msg= format ["MATCH ENDS IN %1 SECONDS!", round(jac_matchGameTime - CBA_missionTime)]; [[_msg, "PLAIN"]] remoteExec ["titleText", 0]};
	
	//end mission
	if ( (CBA_missionTime >= jac_matchGameTime) or (_doEndMatch) or (jac_doEndMatchNow) ) then 
	{
		if (jac_missionPhase == 2) then 
		{
			_doEndMatch= false; jac_missionPhase= 3;	//go to endPhase
			[_sideWinner] spawn { params ["_sideWinner"];
				sleep 2; [["END OF THE MATCH!", "PLAIN"]] remoteExec ["titleText", 0];
				sleep 2; [_sideWinner # 2] call BIS_fnc_endMissionServer;
			};//spawn
		};
	};
	
	if !(jac_wargameMatchRoutine) then {[(_this # 1)] call CBA_fnc_removePerFrameHandler};
} , 1, []] call CBA_fnc_addPerFrameHandler;

};

/// set victory points - [_side, _vpAdd] remoteExecCall ["jac_fnc_setVictoryPoints", 0, true];
jac_fnc_setVictoryPoints=
{
	params ["_side", "_vpAdd", "_legacyCompatibility"];
	_index= ([blufor, opfor, independent] findIf {_x == _side});
	if (_index != -1) then
	{
		_vps= ((jac_wargameSidesData # _index) # 1) + _vpAdd;
		(jac_wargameSidesData # _index) set [1, _vps];
		_msg= format ["%1 GAINED %2 VICTORY POINTS", ((jac_wargameSidesData # _index) # 6), _vpAdd];
		hintSilent _msg;
		
		if ((_vps mod 25) == 0) then 
		{
			_msg2= format ["%1 HAS %2 VICTORY POINTS!", ((jac_wargameSidesData # _index) # 6), _vps];
			titleText [_msg2, "PLAIN"]; hint _msg2;
		};
	};
};



//////// OBJECTIVE AREA MODULE
//set the size of the objective area for the PVP mode - set default owner and timer

jac_fnc_moduleObjArea=
{
// Argument 0 is module logic.
_logic = param [0,objNull,[objNull]];
// Argument 1 is list of affected units (affected by value selected in the 'class Units' argument))
_units = param [1,[],[[]]];
// True when the module was activated, false when it is deactivated (i.e., synced triggers are no longer active)
_activated = param [2,true,[true]];
// Module specific behavior. Function can extract arguments from logic and use them.
if (_activated) then
{
	waitUntil {sleep 0.001; (CBA_missionTime > 1)};	//wait until mission starts
	// Attribute values are saved in module's object space under their class names
	//name
	_name= (_logic getVariable ["jac_objectiveAreaName", "Point Alpha"]);
	
	//default owner - 3 is neutral
	_sideOwner= (_logic getVariable ["jac_setObjectiveAreaOwner", 3]);
	
	//VPs reward
	_reward= (_logic getVariable ["jac_objectiveAreaRewardPoints", 100]);
	
	//curator points reward
	_curatorReward= (_logic getVariable ["jac_objectiveAreaZeusPoints", 100]);
	_curatorReward= 0 max _curatorReward min 1000;	//truncate
	
	//reward points continuously
	_continuousReward= (_logic getVariable ["jac_objectiveAreaRewardPointsContinuous", 30]);
	
	//time to capture objective
	_capTime= (_logic getVariable ["jac_objectiveAreaCaptureTime", 60]);
	
	//jac_objectiveAreaRespawnPoint
	_respawnPoint= (_logic getVariable ["jac_objectiveAreaRespawnPoint", false]);
	
	//is a deployment zone
	_deploymentZone= (_logic getVariable ["jac_objectiveAreaDeploymentZone", false]);
	
	//is a redeployment point
	_redeployPoint= (_logic getVariable ["jac_objectiveAreaRedeployPoint", false]);
	
	//if the redeployment function will use a parachute or place the unit direct on the ground
	_redeployPointParadrop= (_logic getVariable ["jac_objectiveAreaRedeployPointParadrop", false]);
	
	//is a supply point
	_supplyPoint= (_logic getVariable ["jac_objectiveAreaSupplyPoint", 0]);
	
	//code to run after objective is taken
	_objectiveAreaCode= (_logic getVariable ["jac_objectiveAreaCode", '']);	
	
	//respawn timers
	jac_strongpoint_respawnDefault= (_logic getVariable ["jac_objectiveAreaRespawnTimeInfantry", jac_strongpoint_respawnDefault]);
	jac_respawnDefaultVehicle= (_logic getVariable ["jac_objectiveAreaRespawnTimeVehicle", jac_respawnDefaultVehicle]);
	
	//area size AttributeValues 
	_area= (_logic getVariable ["objectArea", 'objectArea']); _size= [_area # 0, _area # 1, _area # 4, _area # 2]; _dir= (_area # 2);
	_areaShape= "RECTANGLE";
	_isRectangle= (_area # 3); if !(_isRectangle) then {_areaShape= "ELLIPSE"};
	
	//add 3D icon with options to see info about area
	_areaLogic = createagent ['jac_wargameLogic',[0,0,0],[],0,'CAN_COLLIDE']; sleep 0.25;
	_areaLogic setPosASL ((getPosASL _logic) vectorAdd [0,0,30]);
	_sideColor= (jac_wargameSidesData # _sideOwner) # 5; _markerSize= [(_size # 0), (_size # 1)]; _markerPos= [((getPosASL _logic) # 0), ((getPosASL _logic) # 1)];
	
	///icon and text above the objective center area 
	_image= "\jac_arma_wargame\data\images\jac_moduleObjArea.paa";
	if (_respawnPoint) then {_image= "\jac_arma_wargame\data\images\jac_HQgroup.paa"};
	
	_areaLogic setVariable ["jac_my3dIcon", _image, true];
	_areaLogic setVariable ["jac_my3dIconColor", ((jac_wargameSidesData # _sideOwner) # 4), true]; _areaLogic setVariable ["jac_my3dIconText", _name, true]; //icon color based on the side that owns it
	_areaLogic setVariable ["jac_isZeusProtected", true, true];	//to prevent griefing
	
	//draw area on the 3D scene
	[_logic, _sideOwner, _size, _areaLogic] remoteExec ["jac_fnc_updateObjectiveAreaMarker", 0, true];
	
	//draw area on the map
	jac_markerGlobal= (jac_markerGlobal + 1);
	_mapMarkerName= ("jac_objectiveMarker" + (str jac_markerGlobal));	//get an unique name
	_str= format ["|%1|%2|mil_dot|%3|%4|%7|DiagGrid|%5|0.9|%6", _mapMarkerName, _markerPos, _areaShape, _markerSize, _sideColor, _name, _dir]; 
	[_str] remoteExecCall ["BIS_fnc_stringToMarker", 0, true];
	_logic setVariable ["jac_objectiveAreaMapMarkerName", _mapMarkerName, true];
	
	//dot with objective name on the map
	jac_markerGlobal= (jac_markerGlobal + 1);
	_mapDotName= ("jac_objectiveMarker" + (str jac_markerGlobal));	//get an unique name
	_mapMarkerText= format ["%1 - VPs: %2 - Zeus: %3", _name, _reward, _curatorReward];
	_strM= format ["|%1|%2|mil_dot|ICON|[0.9,0.9]|0|Solid|ColorBlack|1|%3", _mapDotName, _markerPos, _mapMarkerText];
	[_strM] remoteExecCall ["BIS_fnc_stringToMarker", 0, true];
	
	/// Respawn base
	_areaLogic setVariable ["jac_BaseSide", ((jac_wargameSidesData # _sideOwner) # 0), true];	//set side before calling the init function
	[[_areaLogic], ((jac_wargameSidesData # _sideOwner) # 0)] remoteExec ["jac_fnc_initSetSide", 0, true];	//JIP
	
	if (_respawnPoint) then {[[_areaLogic]] remoteExec ["jac_fnc_initMainBase", 0, true]};
	
	/// Redeployment Point
	if (_redeployPoint) then {[[_areaLogic]] remoteExec ["jac_fnc_initRedeployPoint", 0, true]};
	
	/// Redeployment Point paradrop
	if (_redeployPointParadrop) then {_areaLogic setVariable ["jac_objectiveAreaRedeployPointParadrop", _redeployPointParadrop, true]};
	
	/// supply point
	if (_supplyPoint > 0) then {[[_areaLogic], _supplyPoint] remoteExec ["jac_fn_initSupplyPoint", 0, true]};

	///main routine
	sleep 1; 
	[_logic, _areaLogic, _name, _size, _capTime, _reward, _continuousReward, _respawnPoint, _curatorReward, _objectiveAreaCode, _mapMarkerName, _deploymentZone] call jac_fnc_moduleObjAreaRoutine;
	
	// add to all zeus players
	{[_x, [[_areaLogic], true]] remoteExec ["addCuratorEditableObjects", 2]}forEach allCurators;
	
	[_areaLogic] remoteExecCall ["jac_fnc_moduleObjAreaAddToZeus", 0, true];
};

true;	// Module function is executed by spawn command, so returned value is not necessary, but it is good practice.
};

//function to fix join progress problems with the modules not being added to the zeus list
jac_fnc_moduleObjAreaAddToZeus=
{
	params ["_areaLogic"];
	// the logic is added to this global variable, when a player enter zeus, all logics from this list are added to his control list
	jac_moduleObjAreaAddToZeusList pushBackUnique _areaLogic;
	jac_logicIconList pushBack _obj;
};


jac_fnc_moduleObjAreaRoutine=
{
params ["_logic", "_areaLogic", "_name", "_size", "_capTime", "_reward", "_continuousReward", "_respawnPoint", "_curatorReward", "_objectiveAreaCode", "_mapMarkerName", "_deploymentZone"];

[{((jac_missionPhase == 2) or !(jac_isPVPmatch))},	 //in pvp missions wait for combat phase before running the routine
{
	params ["_logic", "_areaLogic", "_name", "_size", "_capTime", "_reward", "_continuousReward", "_respawnPoint", "_curatorReward", "_objectiveAreaCode", "_mapMarkerName", "_deploymentZone"];
	
	_modulePos= getPosASL _logic;
	_zMin= ((_modulePos # 2) - (_size # 2)); _zMax= ((_modulePos # 2) + (_size # 2));	
	_PARAMS= _this; _PARAMS append [_modulePos, _zMin, _zMax, _continuousRewardTimer];
	
	[{
		(_this # 0) params ["_logic", "_areaLogic", "_name", "_size", "_capTime", "_reward", "_continuousReward", "_respawnPoint", "_curatorReward", "_objectiveAreaCode", "_mapMarkerName", "_deploymentZone",
		"_modulePos", "_zMin", "_zMax", "_continuousRewardTimer"];
		
		//get variables
		_sideCapture= _logic getVariable ["jac_sideCapture", [0,0,0]];
		_dominantSideIndex= _logic getVariable ["jac_dominantSideIndex", 0];
		_sideOwnerIndex= _logic getVariable ["jac_sideOwnerIndex", 0];
		_continuousRewardTimer= _logic getVariable ["jac_continuousRewardTimer", CBA_missionTime];

		_sideOwner= jac_wargameSidesData # (_logic getVariable ["jac_setObjectiveAreaOwner", 3]) # 0;
		_dominantSide= _sideOwner; _lastSideCount= 0; _dominantSideCount= 0; _sideOwnerCount= 0;
		
		//detect units inside the objective area and determine if they should be capturing
		_unitsInside = allUnits select {_pos= getPosASL _x; ( (alive _x) && ((_pos # 2) >= _zMin) && ((_pos # 2) <= _zMax) )} inAreaArray _mapMarkerName;
		
		{
			_testSide= _x; _testSideInside= _unitsInside select {((side _x) == _testSide)};
			if (_testSide == _sideOwner) then {_sideOwnerCount= count _testSideInside; _sideOwnerIndex= _forEachIndex};
			if (count _testSideInside > _lastSideCount) then {_dominantSide= _testSide; _lastSideCount= count _testSideInside; _dominantSideCount= _lastSideCount; _dominantSideIndex= _forEachIndex};
		}forEach [blufor, opfor, independent];
		
		if ( ( ([_dominantSide, _sideOwner] call BIS_fnc_sideIsEnemy) or (_sideOwner == sideEmpty) ) && (_dominantSideCount > _sideOwnerCount) ) then 
		{
			//text will display the status of the objective - "sideName CAPTURING..."; //  (_numberA / _numberB) * 100
			_add= (_sideCapture # _dominantSideIndex) +1; _sideCapture= [0,0,0]; _sideCapture set [_dominantSideIndex, _add];
			_msg= format ["%1 is capturing: %2%3", ((jac_wargameSidesData # _dominantSideIndex) # 6), (100 min round ((_add / _capTime) * 100)), "%"];
			[250, _areaLogic, _msg] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0];
			
			//update victory points, marker color and owner when captured
			if (_add > _capTime) then
			{
				_mapMarkerName= _logic getVariable ["jac_objectiveAreaMapMarkerName", ""];
				_mapMarkerName setMarkerColor ((jac_wargameSidesData # _dominantSideIndex) # 5);
				_areaLogic setVariable ["jac_my3dIconColor", ((jac_wargameSidesData # _dominantSideIndex) # 4), true];
	
				[_logic, _dominantSideIndex, _size, _areaLogic] remoteExecCall ["jac_fnc_updateObjectiveAreaMarker", 0, true];
				
				if (_reward > 0) then {[_dominantSide, _reward] remoteExecCall ["jac_fnc_setVictoryPoints", 0, true]};
				_msg= format ["%1 CAPTURED THE %2!", ((jac_wargameSidesData # _dominantSideIndex) # 6), _name]; [300, _areaLogic, _msg] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0];
				_logic setVariable ["jac_setObjectiveAreaOwner", _dominantSideIndex, true];
				_continuousRewardTimer= CBA_missionTime;
	
				_areaLogic setVariable ["jac_BaseSide", ((jac_wargameSidesData # _dominantSideIndex) # 0), true];	//set the side that owns this point for redeployment and respawning
				if (_respawnPoint) then {[((jac_wargameSidesData # _dominantSideIndex) # 0), _areaLogic] remoteExecCall ["jac_fnc_setMainBaseSide", 0, true]}; //set the side of the main base object
				
				if (_curatorReward > 0) then 
				{
					{
						_x addCuratorPoints (_curatorReward * 0.001);
					} forEach (allCurators select {(side (getAssignedCuratorUnit _x) isEqualTo  _dominantSide)});	//grant resource points to all curators that belong to the owner's side
				};
				
				// exec custom code
				if ((count _objectiveAreaCode) > 1) then
				{
					_zeusList= [];
					{_zeusList insert [-1, ([_x] call jac_fnc_objectCurators), true]} forEach _unitsInside;
					
					_code = compile _objectiveAreaCode;
					[_modulePos, _dominantSide, _sideOwner, _unitsInside, _zeusList, _logic] spawn _code;
				};
				
				// update deployment zone
				if (_deploymentZone) then
				{
					_zoneId= (_logic getVariable ["jac_objectiveEditingArea", -1]);
					if (_zoneId != -1) then
					{
						// remove from previous owner side zeus
						_curators= (allCurators select { (side (getAssignedCuratorUnit _x)) == _sideOwner});
						{ _x removeCuratorEditingArea _zoneId }forEach _curators;
						
						_logic setVariable ["jac_objectiveEditingArea", -1];
					};
				};
			};
		}
		else
		{
			if (_continuousReward > 0) then	///continuously reward the owner with victory points every _continuousReward seconds - as long as it's not being captured by another side
			{
				if (_continuousRewardTimer <= (CBA_missionTime - _continuousReward)) then
				{
					_continuousRewardTimer= CBA_missionTime;
					if (_reward > 0) then {[_sideOwner, _reward] remoteExecCall ["jac_fnc_setVictoryPoints", 0, true]};
					
					if (_curatorReward > 0) then {
						{
							_x addCuratorPoints (_curatorReward * 0.001);
						} forEach (allCurators select {(side (getAssignedCuratorUnit _x) isEqualTo _dominantSide)});	//grant resource points to all curators that belong to the owner's side
					};
				};
			};
		};
		
		// create deployment zone
		if (_deploymentZone) then
		{
			if ((_logic getVariable ["jac_objectiveEditingArea", -1]) == -1) then
			{
				_sideOwner= jac_wargameSidesData # (_logic getVariable ["jac_setObjectiveAreaOwner", 3]) # 0;
				if !(_sideOwner isEqualTo sideEmpty) then
				{
					_curators= (allCurators select { (side (getAssignedCuratorUnit _x)) == _sideOwner});
					if (count _curators > 0) then
					{
						_preparationPhaseEditArea= (missionNamespace getVariable ["jac_preparationPhaseEditArea", 1]);
						_position= (position _logic);
						[_preparationPhaseEditArea, _position, _size, _sideOwner] remoteExec ["jac_fnc_addEditingAreaJIPCompat", 0, true];
						
						_logic setVariable ["jac_objectiveEditingArea", _preparationPhaseEditArea];
						missionNamespace setVariable ["jac_preparationPhaseEditArea", (_preparationPhaseEditArea +1)];
					};
				};
			};
		};
		
		//maintain variables
		_logic setVariable ["jac_sideCapture", _sideCapture];
		_logic setVariable ["jac_dominantSideIndex", _dominantSideIndex];
		_logic setVariable ["jac_sideOwnerIndex", _sideOwnerIndex];
		_logic setVariable ["jac_continuousRewardTimer", _continuousRewardTimer];
		
	} , 1, _PARAMS] call CBA_fnc_addPerFrameHandler;
}, _this] call CBA_fnc_waitUntilAndExecute;
};

//JIP compat
jac_fnc_addEditingAreaJIPCompat=
{
	params ["_preparationPhaseEditArea", "_position", "_size", "_sideOwner"];
	if (side player == _sideOwner) then
	{
		if (isNull (getAssignedCuratorLogic player)) then {sleep 5};	//JIP wait for player to become zeus
		
		_curatorModule= (getAssignedCuratorLogic player);		
		[_curatorModule, [_preparationPhaseEditArea, _position, ((_size # 0) max (_size # 1))]] remoteExec ["addCuratorEditingArea", 2];
	};
};

jac_fnc_updateObjectiveAreaMarker=
{
	params ["_logic", "_sideId", "_size", "_areaLogic"];
	
	_sideId= (_logic getVariable ["jac_setObjectiveAreaOwner", 3]);
	
	_modulePos= getPosATL _logic; _xSize= (_size # 0);  _ySize= (_size # 1); _z= (_modulePos # 2) +5; _dir= _size # 3;	
	_rgb= ((jac_wargameSidesData # _sideId) # 4); _id= _logic getVariable ["jac_objectiveAreaMarkerID", str _logic]; _logic setVariable ["jac_objectiveAreaMarkerID", _id];
	
	/// delete last area
	[_id] call jac_fnc_delete3DShape;
	
	/// create the new area
	[_xSize, _ySize, _z, _rgb, _id, (getPosASL _logic), _dir] call jac_fnc_drawRectangle3D;
	
	if !(_areaLogic in jac_logicIconList) then {jac_logicIconList pushBack _areaLogic};
};


///////////PREPARATION PHASE MODULE

jac_fnc_modulePrepArea=
{
// Argument 0 is module logic.
_logic = param [0,objNull,[objNull]];
// Argument 1 is list of affected units (affected by value selected in the 'class Units' argument))
_units = param [1,[],[[]]];
// True when the module was activated, false when it is deactivated (i.e., synced triggers are no longer active)
_activated = param [2,true,[true]];
// Module specific behavior. Function can extract arguments from logic and use them.
if (_activated) then
{
	// Attribute values are saved in module's object space under their class names
	waitUntil {sleep 0.001; ( (CBA_missionTime > 1) && (!isNull player) or !(hasInterface) )};	//wait until mission starts
	
	// if this module is meant for the player
	_sideOwner= (jac_wargameSidesData # (_logic getVariable ["jac_setPreparationAreaOwner", 0]) # 0);	
	if ((side player) != _sideOwner) exitWith {false};	
	jac_modulePrepAreaActivated= true;
	
	_position= (position _logic);
	_area= (_logic getVariable ["objectArea", 'objectArea']); _size= ((_area # 0) max (_area # 1));
	
	_doRemove= (_logic getVariable ["jac_setPreparationAreaRemove", false]);
	
	[_logic, _position, _size, _doRemove] call jac_fnc_modulePrepAreaActivated;
};

true;	// Module function is executed by spawn command, so returned value is not necessary, but it is good practice.
};

jac_fnc_modulePrepAreaActivated=
{
	params ["_logic", "_position", "_size", "_doRemove"];	
	
	waitUntil {sleep 0.1; ( !(isNull findDisplay 312) )};	//wait until player starts the zeus interface
	
	_curator= (getAssignedCuratorLogic player);
	[[_curator, _position, _size, _doRemove],
	{
		params ['_curator', '_position', '_size', '_doRemove'];
		_curator setCuratorCoef ["Edit", 0];
		_curator setCuratorCoef ["Delete", 0.2];
		_curator setCuratorCoef ["Destroy", 0];
		if (_curator getVariable ["jac_canPlaceUnitsZeus", false]) then {_curator setCuratorCoef ["Place", -1]};		//placing if available has normal cost
		
		_preparationPhaseEditArea= (missionNamespace getVariable ["jac_preparationPhaseEditArea", 1]);
		_curator addCuratorEditingArea [_preparationPhaseEditArea, _position, _size];
		missionNamespace setVariable ["jac_preparationPhaseEditArea", (_preparationPhaseEditArea +1)];
	
		waitUntil {sleep 0.001; ((jac_missionPhase == 2) or !(jac_isPVPmatch)) };
	
		sleep 1;	//prevent errors
	
		///remove editing abilities
		_curator setCuratorCoef ["Edit", -0.1];
		_curator setCuratorCoef ["Delete", 0];
		_curator setCuratorCoef ["Destroy", 0];
		if !(_curator getVariable ["jac_canEditUnitsZeus", false]) then {_curator setCuratorCoef ["Edit", -1e10]};
		if !(_curator getVariable ["jac_canPlaceUnitsZeus", false]) then {_curator setCuratorCoef ["Place", -1e10]; removeAllCuratorEditingAreas _curator};
		
		if (_doRemove) then {_curator removeCuratorEditingArea _preparationPhaseEditArea};
	
	}] remoteExec ["spawn", 2];
};



/// remote exec to set mission phase
jac_fnc_setMissionPhase= {params ["_newPhase"]; jac_missionPhase= _newPhase};




////////// GROUP TRAIT INIT MODULES

// HQ
jac_fnc_moduleInitTraitHQ=
{
// Argument 0 is module logic.
_logic = param [0,objNull,[objNull]];
// Argument 1 is list of affected units (affected by value selected in the 'class Units' argument))
_units = param [1,[],[[]]];
// True when the module was activated, false when it is deactivated (i.e., synced triggers are no longer active)
_activated = param [2,true,[true]];
// Module specific behavior. Function can extract arguments from logic and use them.
if (_activated) then
{
	// Attribute values are saved in module's object space under their class names	
	[synchronizedObjects _logic] remoteExec ["jac_fn_initHQTeam", 0];
	
	_logic setvariable ["jac_moduleInitTraitFunctionData", [[],"jac_fn_initHQTeam"], true];
};

true;	// Module function is executed by spawn command, so returned value is not necessary, but it is good practice.	
};

// forward observer
jac_fnc_moduleInitTraitFO=
{
// Argument 0 is module logic.
_logic = param [0,objNull,[objNull]];
// Argument 1 is list of affected units (affected by value selected in the 'class Units' argument))
_units = param [1,[],[[]]];
// True when the module was activated, false when it is deactivated (i.e., synced triggers are no longer active)
_activated = param [2,true,[true]];
// Module specific behavior. Function can extract arguments from logic and use them.
if (_activated) then
{
	// Attribute values are saved in module's object space under their class names
	_isFOdefaultRange= (_logic getVariable ["jac_moduleInitTraitFO", jac_isFOdefaultRange]);
	[synchronizedObjects _logic, _isFOdefaultRange] remoteExec ["jac_fn_initFOTeam", 0];
	
	_logic setvariable ["jac_moduleInitTraitFunctionData", [[_isFOdefaultRange],"jac_fn_initFOTeam"], true];
};

true;	// Module function is executed by spawn command, so returned value is not necessary, but it is good practice.	
};

// supply point
jac_fnc_moduleInitTraitSupply=
{
// Argument 0 is module logic.
_logic = param [0,objNull,[objNull]];
// Argument 1 is list of affected units (affected by value selected in the 'class Units' argument))
_units = param [1,[],[[]]];
// True when the module was activated, false when it is deactivated (i.e., synced triggers are no longer active)
_activated = param [2,true,[true]];
// Module specific behavior. Function can extract arguments from logic and use them.
if (_activated) then
{
	// Attribute values are saved in module's object space under their class names
	_supplyPoints= (_logic getVariable ["jac_moduleInitTraitSupply", 1000]);		
	_supplyPointsLimit= (_logic getVariable ["jac_moduleInitTraitSupplyLimit", _supplyPoints]);
	_supplyPointsReplenish= (_logic getVariable ["jac_moduleInitTraitSupplyReplenish", 0]);
	_isSupplyPointReinforceGroup= (_logic getVariable ["jac_isSupplyPointReinforceGroup", true]);
	
	[synchronizedObjects _logic, _supplyPoints, _supplyPointsLimit, _supplyPointsReplenish, _isSupplyPointReinforceGroup] remoteExec ["jac_fn_initSupplyPoint", 0];
	
	_logic setvariable ["jac_moduleInitTraitFunctionData", [[_supplyPoints, _supplyPointsLimit, _supplyPointsReplenish, _isSupplyPointReinforceGroup], "jac_fn_initSupplyPoint"], true];
};

true;	// Module function is executed by spawn command, so returned value is not necessary, but it is good practice.	
};

// engineer
jac_fnc_moduleInitTraitEngineer=
{
// Argument 0 is module logic.
_logic = param [0,objNull,[objNull]];
// Argument 1 is list of affected units (affected by value selected in the 'class Units' argument))
_units = param [1,[],[[]]];
// True when the module was activated, false when it is deactivated (i.e., synced triggers are no longer active)
_activated = param [2,true,[true]];
// Module specific behavior. Function can extract arguments from logic and use them.
if (_activated) then
{
	// Attribute values are saved in module's object space under their class names
	_fortificationPoints= (_logic getVariable ["jac_moduleInitTraitEngineer", 1000]);
	[synchronizedObjects _logic, _fortificationPoints] remoteExec ["jac_fn_initEngineerTeam", 0];
	
	_logic setvariable ["jac_moduleInitTraitFunctionData", [[_fortificationPoints],"jac_fn_initEngineerTeam"], true];
};

true;	// Module function is executed by spawn command, so returned value is not necessary, but it is good practice.
};

// EOD
jac_fnc_moduleInitTraitEOD=
{
// Argument 0 is module logic.
_logic = param [0,objNull,[objNull]];
// Argument 1 is list of affected units (affected by value selected in the 'class Units' argument)
_units = param [1,[],[[]]];
// True when the module was activated, false when it is deactivated (i.e., synced triggers are no longer active)
_activated = param [2,true,[true]];
// Module specific behavior. Function can extract arguments from logic and use them.
if (_activated) then
{
	// Attribute values are saved in module's object space under their class names
	[(synchronizedObjects _logic)] call jac_fnc_moduleInitTraitEODSetTrait;
	
	_logic setvariable ["jac_moduleInitTraitFunctionData", [[],"jac_fnc_moduleInitTraitEODSetTrait"], true];
};

true;	// Module function is executed by spawn command, so returned value is not necessary, but it is good practice.	
};

jac_fnc_moduleInitTraitEODSetTrait=
{
	params ["_units"];
	{_x setUnitTrait ["explosiveSpecialist", true]}forEach _units;
	[_units] remoteExec ["jac_fn_initEODTeam", 0];
};

// set radio range
jac_fnc_moduleInitTraitRadio=
{
// Argument 0 is module logic.
_logic = param [0,objNull,[objNull]];
// Argument 1 is list of affected units (affected by value selected in the 'class Units' argument))
_units = param [1,[],[[]]];
// True when the module was activated, false when it is deactivated (i.e., synced triggers are no longer active)
_activated = param [2,true,[true]];
// Module specific behavior. Function can extract arguments from logic and use them.
if (_activated) then
{
	// Attribute values are saved in module's object space under their class names
	_RadioRange= (_logic getVariable ["jac_moduleInitTraitRadio", jac_GrpDefaultRadioRange]);
	
	[(synchronizedObjects _logic), _RadioRange] remoteExec ["jac_fnc_moduleInitTraitRadioSetRange", 0];
	
	_logic setvariable ["jac_moduleInitTraitFunctionData", [[_RadioRange],"jac_fnc_moduleInitTraitRadioSetRange"], true];
};

true;	// Module function is executed by spawn command, so returned value is not necessary, but it is good practice.
};

jac_fnc_moduleInitTraitRadioSetRange=
{
	params ["_units", "_RadioRange"];
	{
		_unit= _x;
		[{ params ['_unit', '_RadioRange']; ((leader _unit) getVariable ["jac_wrgInitFinished", false]) },
		{
			params ['_unit', '_RadioRange'];
			(group _unit) setVariable ["jac_my_radio_range", _RadioRange, true];
		}, [_unit, _RadioRange]] call CBA_fnc_waitUntilAndExecute;
	}forEach _units;
};


// FOG OF WAR parameters module 
jac_fnc_moduleFogOfWarParameters=
{
// Argument 0 is module logic.
_logic = param [0,objNull,[objNull]];
// Argument 1 is list of affected units (affected by value selected in the 'class Units' argument))
_units = param [1,[],[[]]];
// True when the module was activated, false when it is deactivated (i.e., synced triggers are no longer active)
_activated = param [2,true,[true]];
// Module specific behavior. Function can extract arguments from logic and use them.
if (_activated) then 
{
	// Attribute values are saved in module's object space under their class names
	_moduleFogOfWarParametersViewRange= 0 max (_logic getVariable ["jac_moduleFogOfWarParametersViewRange", 0]) min 4000;
	_moduleFogOfWarParametersStealthBonus= ((0 max (_logic getVariable ["jac_moduleFogOfWarParametersStealthBonus", 0]) min 100) *0.01);
	_moduleFogOfWarParametersBlacklist= _logic getVariable ["jac_moduleFogOfWarParametersBlacklist", false];
	_moduleFogOfWarParametersTargets= _logic getVariable ["jac_moduleFogOfWarParametersTargets", false];
	{
		_unit= _x;
		if (_moduleFogOfWarParametersViewRange > 0) then {_unit setVariable ["jac_baseViewRangeModule", _moduleFogOfWarParametersViewRange]};		
		if (_moduleFogOfWarParametersStealthBonus > 0) then {_unit setVariable ["jac_bonusStealth", _moduleFogOfWarParametersStealthBonus]};
		_unit setVariable ['jac_blackListFromValidUnits', _moduleFogOfWarParametersBlacklist, true];
		
		if !(_moduleFogOfWarParametersTargets) then
		{
			_typeOf= (typeOf _unit);
			if (jac_unitCustomPostInit findIf {(_x # 0) == _typeOf} == -1) then 	//if already added init event handler
			{
				jac_unitCustomPostInit pushBack [_typeOf, _moduleFogOfWarParametersViewRange, _moduleFogOfWarParametersStealthBonus, _moduleFogOfWarParametersBlacklist];
				[_typeOf, "InitPost", {
					_unit= (_this # 0); _typeOf= (typeOf _unit);
					
					_id= jac_unitCustomPostInit findIf {_x # 0 == _typeOf};
					
					if (_id != -1) then
					{
						_moduleFogOfWarParametersViewRange= jac_unitCustomPostInit # _id # 1;
						_moduleFogOfWarParametersStealthBonus= jac_unitCustomPostInit # _id # 2;
						_moduleFogOfWarParametersBlacklist= jac_unitCustomPostInit # _id # 3;
						
						if (_moduleFogOfWarParametersViewRange > 0) then {_unit setVariable ["jac_baseViewRangeModule", _moduleFogOfWarParametersViewRange]};		
						if (_moduleFogOfWarParametersStealthBonus > 0) then {_unit setVariable ["jac_bonusStealth", _moduleFogOfWarParametersStealthBonus]};
						_unit setVariable ['jac_blackListFromValidUnits', _moduleFogOfWarParametersBlacklist, true];
					};
				}, true, ["Logic"], false] call CBA_fnc_addClassEventHandler;
			};
		};
	}forEach (synchronizedObjects _logic);
};
true;	// Module function is executed by spawn command, so returned value is not necessary, but it is good practice.
};


// DYNAMIC PATROL
jac_fnc_moduleDynamicPatrol=
{
// Argument 0 is module logic.
_logic = param [0,objNull,[objNull]];
// Argument 1 is list of affected units (affected by value selected in the 'class Units' argument))
_units = param [1,[],[[]]];
// True when the module was activated, false when it is deactivated (i.e., synced triggers are no longer active)
_activated = param [2,true,[true]];
// Module specific behavior. Function can extract arguments from logic and use them.
if (_activated) then 
{
	// Attribute values are saved in module's object space under their class names	
	{
		//groups of units synched to this module will become dynamic patrols
		//will move to the position of a friendly group within radio comms range that comes under attack.
		_my_group= (group effectiveCommander (vehicle _x));
		_my_group setVariable ['jac_grpIsPatrol', true, true];
		if ((jac_patrolGroupList pushBackUnique _my_group) != -1) then {jac_patrolGroupListCount= (jac_patrolGroupListCount +1)};
	}forEach (synchronizedObjects _logic);
};
true;	// Module function is executed by spawn command, so returned value is not necessary, but it is good practice.
};

// CUSTOM ENGINEER FORTIFICATIONS
jac_fnc_moduleCustomFortifications=
{
// Argument 0 is module logic.
_logic = param [0,objNull,[objNull]];
// Argument 1 is list of affected units (affected by value selected in the 'class Units' argument))
_units = param [1,[],[[]]];
// True when the module was activated, false when it is deactivated (i.e., synced triggers are no longer active)
_activated = param [2,true,[true]];
// Module specific behavior. Function can extract arguments from logic and use them.
if (_activated) then 
{
	waitUntil { sleep 0.001; (CBA_missionTime > 0) };	//wait until mission starts
	// Attribute values are saved in module's object space under their class names
	_name= (_logic getVariable ["jac_moduleCustomFortificationsName", "custom fortification"]);
	_spacing= (_logic getVariable ["jac_moduleCustomFortificationsSpacing", 5]);
	_duration=(_logic getVariable ["jac_moduleCustomFortificationsDuration", 30]);
	_cost= (_logic getVariable ["jac_moduleCustomFortificationsCost", 25]);	
	_directionFix=(_logic getVariable ["jac_moduleCustomFortificationsDirectionFix", 0]);
	_deleteOriginal=(_logic getVariable ["jac_moduleCustomFortificationsDeleteOriginal", false]);
	
	_classNameArray= [];
	{_classNameArray pushBackUnique (typeOf _x)}forEach (synchronizedObjects _logic);
	
	[_classNameArray, _name, _spacing, _duration, _cost, _directionFix, _deleteOriginal] remoteExecCall ["jac_fnc_moduleCustomFortificationsSetArray", 0, true];	
};
true;	// Module function is executed by spawn command, so returned value is not necessary, but it is good practice.
};

jac_fnc_moduleCustomFortificationsSetArray=
{
	params ['_classNameArray', '_name', '_spacing', '_duration', '_cost', '_directionFix', '_deleteOriginal'];
	
	/// delete orginal fortifications
	if (_deleteOriginal) then
	{
		{
			_valueArray= _x;
			_id= jac_fortificationDataArray findif {_x isEqualTo _valueArray};
			if (_id != -1) then {jac_fortificationDataArray deleteAt _id};
		}forEach jac_fortificationDataArrayHardcoded;
	};
	
	// add custom fortification
	{
		jac_fortificationDataArray pushBack [_name, _spacing, _x, _duration, _cost, _directionFix];
	}forEach _classNameArray;
};


// CUSTOM CONTEXT SENSITIVE MENU OPTION
jac_fnc_moduleCustomMenu=
{
// Argument 0 is module logic.
_logic = param [0,objNull,[objNull]];
// Argument 1 is list of affected units (affected by value selected in the 'class Units' argument))
_units = param [1,[],[[]]];
// True when the module was activated, false when it is deactivated (i.e., synced triggers are no longer active)
_activated = param [2,true,[true]];
// Module specific behavior. Function can extract arguments from logic and use them.
if (_activated) then 
{
	waitUntil { sleep 0.001; (CBA_missionTime > 0) };	//wait until mission starts
	// Attribute values are saved in module's object space under their class names
	_name= (_logic getVariable ["jac_moduleCustomMenuName", "MENU OPTION"]);
	_tooltip= (_logic getVariable ["jac_moduleCustomMenuTooltip", "Click on this option to activate it."]);
	_requireSelected= (_logic getVariable ["jac_moduleCustomMenuRequireSelected", false]);
	_code= compile (_logic getVariable ["jac_moduleCustomMenuCode", ""]);

	[_units, _name, _tooltip, _requireSelected, _code] remoteExecCall ["jac_fnc_moduleCustomMenuInit", 0, true];	
};
true;	// Module function is executed by spawn command, so returned value is not necessary, but it is good practice.
};

jac_fnc_moduleCustomMenuInit=
{
	params ['_units', '_name', '_tooltip', '_requireSelected', '_code'];
	_number= if (_requireSelected) then {1} else {0};
	{
		_obj= _x;		
		_optionData= _obj getVariable ['jac_interactiveMenuArray', []];

		_varName= format ["jac_customCode%1%2%3", _obj, CBA_missionTime, (count _optionData)];
		missionNamespace setVariable [_varName, _code];
		
		_optionData pushBack [_name, _varName, _tooltip, _number];		
		_obj setVariable ['jac_interactiveMenuArray', _optionData];
	}forEach _units;
};


// ZEUS TACTICAL PING
// send message to others zeus and set the menu options
jac_fnc_wargameTacticalPingInit=
{
	params ["_logic", "_side", "_myId", "_owner", "_pos"];
	if ((side player) isEqualTo _side) then
	{
		//add icon
		_logic setVariable ["jac_my3dIconColor", [1,0,0,0.9]];
		_logic setVariable ["jac_my3dIconText", _myId];
		_icon= "\A3\ui_f\data\map\markers\military\warning_CA.paa"; //format ["\A3\ui_f\data\map\markers\military\warning_CA.paa", _sideId];
		_logic setVariable ["jac_my3dIcon", _icon]; jac_logicIconList pushBack _logic;
		
		//add menu options
		_optionData= _logic getVariable ['jac_interactiveMenuArray', []];
		_optionData pushBack ["DELETE", "jac_fnc_deleteWargameTacticalPing", 'Delete Tactical Marker', 0, _owner];				//delete
		_logic setVariable ['jac_interactiveMenuArray', _optionData, true];
		
		//play ping sound
		playSoundUI [(format ["a3\ui_f_curator\data\sound\cfgsound\ping0%1.wss", ((floor (random 7)) +1)])];
	};

	//add to zeus control list
	if (isServer) then
	{
		_curators= (allCurators select {(side (getAssignedCuratorUnit _x) isEqualTo _side)});
		{_x addCuratorEditableObjects [[_logic], true]} forEach (allCurators select {(side (getAssignedCuratorUnit _x) isEqualTo _side)});
	};
};

jac_fnc_deleteWargameTacticalPing=
{
	params ["_ctrlPos", "_ctrlOBJ", "_grpArray", "_optionalArgument"];
	if (clientOwner == _optionalArgument) then 
	{
		jac_tacticalPingArray= jac_tacticalPingArray - [_ctrlOBJ];
		jac_tacticalPingNumber= count jac_tacticalPingArray;
		deleteVehicle _ctrlOBJ;
		
		jac_tacticalPingArray= jac_tacticalPingArray - [objNull];
	};	
};

//// AIRCRAFT CONTROL MODULES
jac_fnc_virtualAirSupport=
{
// Argument 0 is module logic.
_logic = param [0,objNull,[objNull]];
// Argument 1 is list of affected units (affected by value selected in the 'class Units' argument))
_units = param [1,[],[[]]];
// True when the module was activated, false when it is deactivated (i.e., synced triggers are no longer active)
_activated = param [2,true,[true]];
// Module specific behavior. Function can extract arguments from logic and use them.
if (_activated) then 
{
	waitUntil { sleep 0.001; ( (CBA_missionTime > 1) && ( (!isNull player) or !(hasInterface) ) ) };	//wait until mission starts and player object is initialized

	// Attribute values are saved in module's object space under their class names
	_cost= _logic getVariable ["jac_virtualAirSupportCost", 0];
	_cost= 0 max _cost min 1000;	//truncate

	_uses= _logic getVariable ["jac_virtualAirSupportAvailability", 0];
	_cooldown= _logic getVariable ["jac_virtualAirSupportCooldown", 0];
	_cooldownDestroyed= _logic getVariable ["jac_virtualAirSupportCooldownDestroyed", 0];
	_virtualAirSupportExecCode= _logic getVariable ["jac_virtualAirSupportExecCode", ""];
	_virtualAirSupportCondition= _logic getVariable ["jac_virtualAirSupportCondition", "'true'"];
	_virtualAirSupportName= _logic getVariable ["jac_virtualAirSupportName", ""];
	
	_startCooldown= if (_logic getVariable ["jac_virtualAirSupportStartInCooldown", false]) then {(CBA_missionTime + _cooldown)} else {0};	//if the option wil be available right at the start

	//is synced to a player or zeus init module
	if (_units findif {private _test= _x; ( (_test == player) or ((synchronizedObjects _test) findif {(_x == player)} != -1) )} != -1) then
	{
		//save the data of aircraft units synced to the module
		_dataArray= [];
		{
			if (_x isKindOf "Air") then
			{
				//prepare unit
				_unit= _x; _myType= (typeOf _unit);
				{ _unit deleteVehicleCrew _x } forEach (crew _unit);
				_unit hideObjectGlobal true; _unit enableSimulationGlobal false;
				_unit setVariable ['jac_blackListFromValidUnits', true, true];
				_unit setPosASL [0,0,0];
				
				// save the pylon and weapon data for the aircraft - exclude countermeasures, laserdesignator and AA missiles from weapon list
				_pylons= (getAllPylonsInfo _unit); _weapons= [];
				[_unit] call jac_fnc_returnAllTurretsWeapons params ["_allTurrets", "_allWeapons"];

				{
					_addWeapons= (_x select {_weaponType= ((_x call BIS_fnc_itemType) # 1); ( !("laserdesignator" in toLower _x) && (_weaponType != "CounterMeasuresLauncher") &&
					!((_weaponType == "MissileLauncher") && ((getNumber (configFile >> "CfgAmmo" >> (getText (configFile >> "CfgMagazines" >> ((getArray (configFile >> "CfgWeapons" >> _x >> "magazines")) # 0) >> "ammo")) >> "aiAmmoUsageFlags")) == 256)) ) }); //no AA missile
					_weapons insert [-1, _addWeapons];
				}foreach _allWeapons;
				_loadoutData= [_pylons, _weapons];
		
				// save vehicle Appearance
				_id= format ["jac_vehicleAppearance_%1%2", _myType, _logic];
				_vehicleAppearanceData= [_unit, [missionNamespace, _id], [], false] call BIS_fnc_saveVehicle;
				
				//last array saves the selected ammo index from (_loadoutData # 1); number of times used; time when it will be available for use again; if the option is currently in use
				_dataArray pushBack [_myType, _loadoutData, _vehicleAppearanceData, [_cost, _uses, _cooldown, _cooldownDestroyed, _virtualAirSupportExecCode, _virtualAirSupportCondition, _virtualAirSupportName], [0, 0, _startCooldown, false]]; //4
			};
		}foreach _units;
		
		if (count _dataArray > 0) then
		{
			_virtualAirSupportData= player getvariable ["jac_virtualAirSupportData", []];
			_virtualAirSupportData insert [-1, _dataArray];
			player setvariable ["jac_virtualAirSupportData", _virtualAirSupportData, true];
		};
	};
};
true;	// Module function is executed by spawn command, so returned value is not necessary, but it is good practice.
};













