//////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////
/////////////// WP arguments


///// AREA SUPPRESSION

//left click and drag to draw a fire arc, the arc height is determined by the start position - move arc up and down with [ ]
//for each unit, get the closest los point collision - unit will only shoot if position found is near the leader's target position (to prevent shooting in wild directions)
//scan back and forth between the start and end points firing short bursts according to the weapon's config
//divide the arc in segments, each corresponding to a burst
//if the shooter moves a determined distance, recalculate los to the target positions (this enables moving and firing)

jac_fnc_suppressionCommand=
{
params ['_suppressAreaP1', '_suppressAreaP2', '_suppressUnits'];
{
	if !(local _x) then {continue};
	_unit= _x; private ["_selectedShooter", "_selectedMuzzle", "_isInfantry", "_turretPath"];

	//if a is a vehicle go through each crew member and check if their turret weapon/ammo is the one selected by the player from the weapons menu, if none is selected use default
	_isInfantry= (_unit isKindOf 'CAManBase');
	_isLauncher= false;
	
	if !(_isInfantry) then
	{
		_veh= _unit;
		_selectedMuzzle= _veh getVariable ['jac_vehSelectedMuzzle', 'default'];	//the muzzle selected from the interactive menu
		_selectedShooter= (_veh turretUnit (_veh getVariable ['jac_vehSelectedTurretPath', [0]]));
		if (_selectedMuzzle == 'default') then {_selectedShooter= (gunner _veh); _selectedMuzzle = _veh currentWeaponTurret (_veh unitTurret _selectedShooter)};	//if no weapon has been selected from the list use the gunner's
		_turretPath= (_veh unitTurret _selectedShooter);
		if (alive _selectedShooter) then { [200, _selectedShooter, "suppressive fire!"] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0] };
	}
	else
	{
		_currentWeapon= (_unit getVariable[ "jac_currentWeapon", [(currentWeapon _unit), ((getArray (configFile >> "cfgWeapons" >> (currentWeapon _unit) >> "muzzles")) # 0)] ]);
		_selectedWeapon= _currentWeapon # 0;
		_selectedMuzzle= _currentWeapon # 1;
		
		if (_selectedWeapon == (secondaryWeapon _unit)) then {_isLauncher= true};

		if (_selectedMuzzle == 'this') then {_selectedMuzzle= _selectedWeapon};
		_selectedShooter=  _unit;
		_turretPath= [];
		if ((leader _unit) == _unit) then {[200, _unit, "suppressive fire!"] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0]; if (count _suppressUnits > 1) then {_unit playActionNow "gestureAdvance";}};
	};
	
	if ((alive _selectedShooter) && !(_unit getVariable ['jac_doEndSuppressAction', false])) then 
	{
		//if executing area suppression already, cancel previous action
		if ((_unit getVariable ['jac_doSuppressAction', false]) or (_selectedShooter getVariable ['jac_doSuppressAction', false])) then {_unit setVariable ['jac_doEndSuppressAction', true, true]};
		
		//waitUntil is read for a new suppression action
		[{
			params ['_selectedShooter', '_selectedMuzzle', '_suppressAreaP1', '_suppressAreaP2', '_isInfantry', '_turretPath', '_isLauncher'];
			!((vehicle _selectedShooter) getVariable ['jac_doSuppressAction', false]);
		},{
			params ['_selectedShooter', '_selectedMuzzle', '_suppressAreaP1', '_suppressAreaP2', '_isInfantry', '_turretPath', '_isLauncher'];
			[_selectedShooter, _selectedMuzzle, _suppressAreaP1, _suppressAreaP2, _isInfantry, _turretPath, _isLauncher] spawn jac_fnc_suppressiveFire;
		}, [_selectedShooter, _selectedMuzzle, _suppressAreaP1, _suppressAreaP2, _isInfantry, _turretPath, _isLauncher]] call CBA_fnc_waitUntilAndExecute;
	};	
}forEach _suppressUnits;
};


///firing and scanning target area routine - must spawn
jac_fnc_suppressiveFire=
{
params ['_unit', '_selectedMuzzle', '_suppressAreaP1', '_suppressAreaP2', '_isInfantry', '_turretPath', '_isLauncher'];
_grp= group _unit;	
_shooter= _unit;			 //shooter is the soldier/crew that will pull the trigger
_unit= vehicle _unit;		//for vehicles _unit is the vehicle itself
_vehCommander= effectiveCommander _unit;

_unit setVariable ['jac_doEndSuppressAction', false, true]; //reset
_unit setVariable ['jac_doSuppressAction', true, true];	//set unit is suppressing
_shooter setVariable ['jac_doSuppressAction', true, true];

if (_grp getVariable ["jac_isCancelAction", false]) then {waitUntil {!(_grp getVariable ["jac_isCancelAction", false])}}; //wait if cancelling action

///get the weapon info - infantry or vehicle
_weaponState= []; _weaponClass = ''; _muzzleClass=''; _firemode=''; _magazineClass = ''; _isInfantryGrpLeader= false;
if (_isInfantry) then 
	{
		_weaponState= _unit weaponState _selectedMuzzle;							//[weapon, muzzle, firemode, magazine, ammoCount, roundReloadPhase, magazineReloadPhase
		_weaponClass = (_weaponState # 0); _muzzleClass= (_weaponState # 1); _firemode= (_weaponState # 2); _magazineClass = (_weaponState # 3);
		if ((leader _grp) == _unit) then {_isInfantryGrpLeader= true};
		_unit setVehicleAmmo 1;
	}
	else
	{
		_weaponState= weaponState [_unit, _turretPath, _selectedMuzzle];
		_weaponClass = (_weaponState # 0); _muzzleClass= (_weaponState # 1); _firemode= (_weaponState # 2); _magazineClass = (_weaponState # 3);		
		if ( !(_unit isKindOf "LandVehicle") or (speed _unit == 0)) then {_unit setEffectiveCommander _shooter}; //required for some vehicles to aim properly, but causes problems if set on  the move
	};

_modes= (getArray (configFile >> "cfgWeapons" >> _selectedMuzzle >> "modes"));
_modesId= _modes findIf {(_x == 'FullAuto') or (_x == 'FullAutoFast') or ('Burst' in str _x) or ('burst' in str _x) or ('Far' in str _x) or ('far' in str _x)};
if (_modesId != -1) then {_firemode= _modes # _modesId};

//find rate of fire
_config = configFile >> "CfgWeapons" >> _weaponClass;
if (_muzzleClass != _weaponClass) then { _config = _config >> _muzzleClass};
if (_muzzleClass != _fireMode) then { _config = _config >> _fireMode};
_fireInterval= getNumber (_config >> "reloadTime");


////create target object 
_target = createVehicle ["CBA_B_InvisibleTargetVehicle", [0,0,0], [], 0, "CAN_COLLIDE"];
_target setVariable ['jac_suppressionTargetObject', true];
_menu= _target;

//menu object with CHANGE ROF and CANCEL options - create if it's an infantry group leader or is a vehicle
if ( (!(_isInfantry) || _isInfantryGrpLeader) && !(!(_isInfantry) && _isInfantryGrpLeader) ) then	// XOR used successfully - (!_isInfantry XOR _isInfantryGrpLeader) - switch {true} do {} also works
{
	_menu = createagent ['jac_wargameLogic',[0,0,0],[],0,'CAN_COLLIDE']; //sleep 0.001;
	_menu setVariable ['jac_suppressionMenuObjectLogic', true];
	_menu setPosASL ((getPosASL _unit) vectorAdd [0,0,8]);
	_menu attachTo [_unit];
	_menu setVariable ['jac_suppressionTargetParentUnit', _unit, true];
	_menu setVariable ['jac_suppressionTargetParentUnitShooter', _shooter, true];	//uses shooter instead of vehicle to get around a problem with static turrets set up by the player
	_unit setVariable ['jac_suppressionMenuObject', _menu, true];
	[[_menu], _shooter] remoteExec ["jac_fnc_initSuppressionHandlerMenu", 0, true];
};

//disable AI features that might get in the way of suppression routine
_features= ["FSM", "TARGET", "AUTOTARGET"];	_featuresStatus= [];
{_featuresStatus set [_forEachIndex, (_shooter checkAIFeature _x)]; _shooter disableAI _x} forEach _features;
if (jac_isWargameSpottingActive) then {_shooter enableAI "CHECKVISIBLE"};

//save last combat mode before changing
_behaviour = combatBehaviour _unit;
_unit setCombatBehaviour "COMBAT";
_combatMode= combatMode _unit;
_unit setUnitCombatMode 'BLUE';

//determine the reference points the target will jump to to make the unit scan the arc of fire - call this function everytime the unit moves a certain distance to keep the target reference points (TRPs) in sight
_targetPos= [_shooter, _suppressAreaP1, _suppressAreaP2] call jac_fnc_getSuppressionPoints; 
_unitReferencePos= (getPosASL _unit);

	_itt= 1;
	_ii= floor ( random ((count _targetPos)-1) );	//each unit will start at different points
	_target setPosASL (_targetPos # _ii);
	
	// handling launcher case, to make soldier aim with launcher more reliably
	if (_isLauncher) then
	{
		_sideTarget= opfor;
		_sideEnemyList = [blufor, opfor, independent, civilian] select {[(side _shooter), _x] call BIS_fnc_sideIsEnemy};
		if (count _sideEnemyList > 0) then {_sideTarget= _sideEnemyList # 0};
		(_sideTarget) createVehicleCrew _target;
	};
	sleep 0.1;
	
	//aim at target	
	_shooter reveal [_target, 4];	_shooter lookAt _target;	_shooter doWatch _target;	_shooter doTarget _target;
	_unit setVariable ['jac_suppressionTargetObject', _target, true];

	//time to move aiming point
	_waitUntilTime= (CBA_missionTime +6);
	if (_isInfantry) then 
	{
		if !(_isInfantryGrpLeader) then {doStop _unit};
		_posDir= (_unit getdir _target); _unit setDir _posDir;
		waitUntil {sleep 0.001;
		(
		([(getDir _shooter), _posDir, 5] call jac_targetDirInFOV) or
		(CBA_missionTime >= _waitUntilTime) or !(alive _shooter) or !(alive _unit) or !(alive _target) or !(alive _menu) or (_unit getVariable ['jac_doEndSuppressAction', false]) )};
		sleep 0.5  //sleep to not shot at the ground when weapon is lowered
	}
	else
	{
		waitUntil {sleep 0.001; ( ((_unit aimedAtTarget [_target]) > 0) or (CBA_missionTime >= _waitUntilTime +6) or !(alive _shooter) or !(alive _unit) or !(alive _target) or !(alive _menu) or (_unit getVariable ['jac_doEndSuppressAction', false]) )};
	};
	
	/// moving target routine
	_target setVariable ['jac_unitReferencePos', _unitReferencePos]; _target setVariable ['jac_targetPos', _targetPos];
	_target setVariable ['jac_ii', _ii]; _target setVariable ['jac_itt', _itt];

	[{
		(_this # 0) params ['_isInfantry', '_unit', '_shooter', '_target', '_suppressAreaP1', '_suppressAreaP2'];
	
		_targetPos= _target getVariable 'jac_targetPos';
		_ii= _target getVariable 'jac_ii'; _itt= _target getVariable 'jac_itt';
	
		if (!(alive _target) or !(alive _shooter)) exitWith {[(_this # 1)] call CBA_fnc_removePerFrameHandler};
	
		//determine the reference points the target will jump to to make the unit scan the arc of fire - call this function everytime the unit moves a certain distance to keep the target reference points (TRPs) in sight
		if ( (_target getVariable 'jac_unitReferencePos') distance (getPosASL _unit) > 2 ) then
		{
			_targetPos= [_shooter, _suppressAreaP1, _suppressAreaP2] call jac_fnc_getSuppressionPoints;
			
			if ( !(_isInfantry) && (speed _unit == 0) ) then {_unit setEffectiveCommander _shooter}; //required for some vehicles to aim properly, but causes problems if set while on the move			
			_target setVariable ['jac_unitReferencePos', (getPosASL _unit)];
			_target setVariable ['jac_targetPos', _targetPos];
		};
	
		_ii= (_ii + _itt);
		if (_ii >= (count _targetPos)-1) then {_ii= (count _targetPos)-1; _itt= -1}; if (_ii <= 0) then {_ii= 0; _itt = 1};	//goes back and forth along the TRPs
		_target setVariable ['jac_ii', _ii]; _target setVariable ['jac_itt', _itt];
	
		_target setPosASL (_targetPos # _ii);
		_shooter reveal [_target, 4];	_shooter lookAt _target;	_shooter doWatch _target;	_shooter doTarget _target; //keep tracking target

	} , 0.25, [_isInfantry, _unit, _shooter, _target, _suppressAreaP1, _suppressAreaP2]] call CBA_fnc_addPerFrameHandler;

	///shooting routine
	while {(_unit getVariable ['jac_doSuppressAction', false])} do
	{
		///if shooting is interrupted by obstacle in the way
		if ((count ((lineIntersectsSurfaces [(eyePos _shooter), getPosASL _target, (vehicle _unit), _target, true, 1, 'VIEW', 'NONE', true]) )) != 0) then 
		{
			waitUntil {sleep 0.5; _colLos= ((lineIntersectsSurfaces [(eyePos _shooter), getPosASL _target, (vehicle _unit), _target, true, 1, 'VIEW', 'NONE', true]));
			( (count _colLos == 0) or !(alive _shooter) or !(alive _unit) or !(alive _target) or !(alive _menu) or (_unit getVariable ['jac_doEndSuppressAction', false]) ) };
			sleep 0.5; _shooter reveal [_target, 4];	_shooter lookAt _target;	_shooter doWatch _target;	_shooter doTarget _target; sleep 0.5;		//reacquire target
		};
		
		///fire a burst
		_target hideObject true; //hide before shooting
		for "_i" from 1 to (_unit getVariable ['jac_SuppressionburstSize', 6]) step +1 do
		{
			if (_isInfantry) then 
			{
				if ( ((_unit ammo _weaponClass) <= 0) && (jac_infinite_ammo_suppress_setting) && !(jac_forceAmmoUseSetting) ) then {_unit addMagazine _magazineClass}; //infantry units will receive a new mag when they are empty
			}
			else 
			{
				if ( (jac_infinite_ammo_suppress_setting_veh) && !(jac_forceAmmoUseSetting)) then {_unit setVehicleAmmo 1};	//if infinite ammo for vehicles is on
				if !(someAmmo _unit) then {[200, _shooter, 'OUT OF AMMO!'] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0]};	//vehicle informs  that it ran out of ammo
			};
			
			if ( (alive _menu)&& (_grp getVariable ['jac_actionMode', 'combatMode'] isEqualTo 'combatMode') ) then
			{
				_shooter forceWeaponFire [_muzzleClass, _firemode];
				sleep _fireInterval;		//firing interval based on firing rate
			};
			if (_unit getVariable ['jac_doEndSuppressAction', false]) then {break};
		};
		_target hideObject false; //unhide target
		
		///burst interval
		_waitUntilTime= CBA_missionTime;
		waitUntil {sleep 0.001; ( !(alive _shooter) or !(alive _unit) or !(alive _target) or !(alive _menu) or (_unit getVariable ['jac_doEndSuppressAction', false]) or (CBA_missionTime >= (_waitUntilTime + (_unit getVariable ['jac_SuppressionburstInterval', 1.5]))) )};

		//end suppression action, delete target, restore AI features/behaviour, reset variables
		if ( !(alive _shooter) or !(alive _unit) or !(alive _target) or !(alive _menu) or (_unit getVariable ['jac_doEndSuppressAction', false]) or (_grp getVariable ['jac_doEndSuppressAction', false]) ) exitWith
		{
			_unit setCombatBehaviour _behaviour;	_unit setBehaviourStrong _behaviour;
			if ((_combatMode == "BLUE") or (_combatMode == (_grp getVariable ['jac_setCombatMode', "YELLOW"]))) then {_unit setUnitCombatMode _combatMode} else {_unit setUnitCombatMode (_grp getVariable ['jac_setCombatMode', "YELLOW"])};
			_unit doWatch objNull;	_unit lookAt objNull;	_unit doTarget objNull;
			_unit setEffectiveCommander _vehCommander;
			
			deleteVehicle _menu; deleteVehicle _target;
			
			{if (_featuresStatus # _forEachIndex) then {_shooter enableAI _x}} forEach _features;	//reenable features
			if (jac_isWargameSpottingActive) then {_shooter disableAI "CHECKVISIBLE"};
			
			if ((leader _unit) == _unit) then {_grp setVariable ['jac_doEndSuppressAction', false, true]};		//for suppression legacy code compatibility
			_unit setVariable ['jac_doEndSuppressAction', false, true];
			_unit setVariable ['jac_doSuppressAction', false, true];
			_shooter setVariable ['jac_doSuppressAction', false, true];
		};
	};
};


////calculate and return the target reference points that the target object will jump around in to guide the aiming point
jac_fnc_getSuppressionPoints=
{
params ['_unit', '_suppressAreaP1', '_suppressAreaP2'];

//if is area or point suppression
_targetPosArray= [];
_posFinal= [];
_maxTRPDis= 100; //4000; //150;
_dropCompensateDis= (_maxTRPDis +150); //300
_unitPos= eyePos _unit;
_dis= (_suppressAreaP1 distance _suppressAreaP2);
_intervalDis= 2;	//distance between each TRP - 5
_numberToBuild= round(_dis/_intervalDis);
_isArea= false;	//is area or point target fire

if (_dis >= _intervalDis) then 
{
	_isArea= true;
	_maxTRPDis= 150; //150;
	_dropCompensateDis= 300; //300

	//find the target positions dividing the area to fire from start to end in equal number of parts then calculate the los to each from the unit
	_dir= (_suppressAreaP1 getDir _suppressAreaP2);	
	for "_i" from 0 to _numberToBuild step +1 do
	{
		_distance= (_intervalDis * _i);
		_pos = [_suppressAreaP1, _suppressAreaP2, _distance] call jac_fnc_findInterpolatedPoint;
		
		//keep TRPs near the unit
		_trpDis= (_pos distance _unitPos); _trpCompDrop= 0;	_interpolationDis= (_trpDis - 1);
		
		if (_trpDis > _maxTRPDis) then { _interpolationDis= _maxTRPDis; if (_trpDis > _dropCompensateDis) then {_trpCompDrop= 0.001} };	//if (_trpDis > 300) then {_trpCompDrop= 0.00075}
		
		_pos = [_pos, _unitPos, (_trpDis - _interpolationDis) ] call jac_fnc_findInterpolatedPoint;
		
		_collision_lineArray= (lineIntersectsSurfaces [_unitPos, _pos, _unit, vehicle (_unit), true, 1, 'VIEW', 'NONE', true]);
		if (count _collision_lineArray !=0) then {_posFinal= (_collision_lineArray # 0) # 0} else {_posFinal=_pos};
		
		if ((_pos distance _unitPos) > 1) then{ _posFinal = [_posFinal, _unitPos, 1] call jac_fnc_findInterpolatedPoint };
		
		_targetPosArray pushBack (_posFinal vectorAdd [0,0,(_trpDis*_trpCompDrop)]);
	};
}
else
{
	_posArray= [_suppressAreaP1, _suppressAreaP2];
	{
		//keep TRPs near the unit
		_pos= _x; _trpDis= (_pos distance _unitPos); _trpCompDrop= 0;	_interpolationDis= (_trpDis - 1);
		if (_trpDis > 900) then { _interpolationDis= _maxTRPDis; if (_trpDis > _dropCompensateDis) then {_trpCompDrop= 0.001} };	//if (_trpDis > 300) then {_trpCompDrop= 0.00075}
		
		_pos = [_pos, _unitPos, (_trpDis - _interpolationDis) ] call jac_fnc_findInterpolatedPoint;
		
		_collision_lineArray= (lineIntersectsSurfaces [_unitPos, _pos, _unit, vehicle (_unit), true, 1, 'VIEW', 'NONE', true]);
		if (count _collision_lineArray !=0) then {_posFinal= (_collision_lineArray # 0) # 0} else {_posFinal= _pos};
		
		if ((_pos distance _unitPos) > 1) then{ _posFinal = [_posFinal, _unitPos, 1] call jac_fnc_findInterpolatedPoint };
		
		_targetPosArray pushBack (_posFinal vectorAdd [0,0,(_trpDis*_trpCompDrop)]);
	}forEach _posArray;
};
_targetPosArray;
};



//////// throw GRENADE waypoint

jac_fnc_placeGrenadeWP=
{
	params ["_grp", "_targetPos", "_magazine"];
	
	if (!(local _grp) or (_grp getVariable["jac_isPanicked", false])) exitWith {false};
	
	_assaultOriginPos= (getPos (leader _grp));
	/// adjusts origin pos if there are waypoints	
	_wpCount= count (waypoints  _grp);
	_WPPosCurrent= getWPPos [_grp, (currentWaypoint _grp)];
	if !(_WPPosCurrent isEqualTo [0,0,0]) then {_assaultOriginPos= getWPPos [_grp, (_wpCount -1)]};
	
	////first set a move waypoint to get within grenade range of the target, then throw grenade
	if ((_targetPos distance2D _assaultOriginPos) > jac_grenadeRange) then
	{
		_heading= (_targetPos getDir _assaultOriginPos);						
		_moveToPos= _targetPos getPos [(jac_grenadeRange - 1), _heading];
		_moveToPos= AGLToASL _moveToPos;
		_wp = _grp addWaypoint [_moveToPos, -1, (count (waypoints  _grp))+1, "MOVE"];
	};
	
	[{
		params ["_grp", "_targetPos", "_magazine"];
		[_grp, _targetPos, _magazine] remoteExec ["jac_fnc_addGrenadeWP", 0];  //declared in WParguments.sqf
	}, [_grp, _targetPos, _magazine], 0.5] call CBA_fnc_waitAndExecute;
};

jac_fnc_addGrenadeWP=
{
	params ["_grp", "_targetPos", "_grenadeType"];
	if (!(local _grp) or (_grp getVariable["jac_isPanicked", false])) exitWith {false};
	
	(units _grp) doFollow (leader _grp);	// to make unit move in case it's in doStop mod
	{if (!(_grp getVariable ['jac_isDugIn', false]) && (_x getVariable ['jac_isInCover', false]) ) then {_x enableAi 'PATH'}}forEach (units _grp);

	_wp = _grp addWaypoint [_targetPos, -1, (count (waypoints  _grp))+1, _grenadeType];
	_wp setWaypointType "SCRIPTED";
	_wp setWaypointStatements [wp_AWGcond,""];
};

wp_AWGcond="
_my_leader=this;
_my_grp=(group _my_leader);
if !(local _my_grp) exitWith {false};

_wp= (currentWaypoint _my_grp);
_target_g= getWPPos [_my_grp, _wp];
_grenadeType= waypointName [_my_grp, _wp];

if ((this distance2D (_target_g) <= jac_grenadeRange) && (waypointDescription [_my_grp, _wp] != 'wp_AWGcondActivated')) then 
{
	[_my_grp, _wp] setWaypointDescription 'wp_AWGcondActivated';
	[_my_leader, _target_g, _grenadeType] spawn jac_fnc_wp_throwGrenadeSelected;
};
";

jac_fnc_wp_throwGrenadeSelected=
{
	params ['_unit', '_target_g', '_grenadeType'];	
	_myGrp= group _unit; 
	_my_leaderLoadout= getUnitLoadout _unit; _restoreMinusOne= true;
	{_x disableAI "PATH"} forEach units _myGrp;
	_unit doWatch [_target_g # 0, _target_g # 1, (_target_g # 2)+1];
	_targetMarker= createVehicle ["VR_Area_01_square_1x1_yellow_F", _target_g, [], 0, "CAN_COLLIDE"];	//target marker

	///event handler used to direct the grenade to the target
	if !(_unit getVariable ["jac_addedGrenadeWP_EH",false]) then {
		_unit addEventHandler ["Fired", {
		params ["_unit", "_weapon", "_muzzle", "_mode", "_ammo", "_magazine", "_projectile", "_gunner"];
		
		if (_unit getVariable ["is_AWGWP_throw",false]) then {
			(_unit setVariable ["is_AWGWP_throw",false]);
		
			[_unit, _weapon, _muzzle, _mode, _ammo, _magazine, _projectile, _gunner] spawn {
				params ["_unit", "_weapon", "_muzzle", "_mode", "_ammo", "_magazine", "_projectile", "_gunner"];
	
				if (_weapon != "Throw") exitWith {};
				if (isNull _projectile) then {_projectile = nearestObject [_unit, _ammo]};
				sleep 0.5;
			
				_projectile setVelocity [0,0,0];
				_projectile setPosATL ((_unit getVariable "grenade_target_g")vectorAdd [0,0,0.15]);
			};
		};
	}];
	_unit setVariable ["jac_addedGrenadeWP_EH",true];
	};
	
	[300, _unit, format ['Throwing %1!', (getText (configFile >> "CfgMagazines" >> _grenadeType >> "displayName"))]] remoteExecCall ["jac_fnc_zeusAddMSG3D", remoteExecutedOwner];
	//check if leader has the grenade type, if not check the other units in the group	 //remove item from owner and give it to the leader who will throw it
	if ( ((magazines _unit) find _grenadeType) == -1 ) then
	{
		_restoreMinusOne= false;
		{
			if ( ((magazines _x) find _grenadeType) != -1 ) exitWith {_x removeItem _grenadeType; removeAllItemsWithMagazines _unit; sleep 0.1; _unit addItem _grenadeType};
		}forEach units _myGrp;
	};
	
	sleep 5;		//wait the unit needs to properly equip the grenade to throw	
	_throwdirection=(_unit getDir [_target_g # 0, _target_g # 1, (_target_g # 2)]);
	_unit setDir _throwdirection; _myGrp setFormDir _throwdirection;
	_unit setVariable ["grenade_target_g",_target_g];
	(_unit setVariable ["is_AWGWP_throw",true]);
	
	///throw selected item
	_index = ((magazines _unit) find _grenadeType);		//get the position of the magazine in the unit inventory array
	if (_index != -1) then
	{
		_magazineDet = ((magazinesDetail _unit) # _index);
		_splitStr= _magazineDet splitString "([ ]/:)";  //["9mm","16Rnd","Mag","13","16","id","cr","10000011","0"]
		_id= parseNumber (_splitStr # ((count _splitStr) - 2)); _owner= parseNumber (_splitStr # ((count _splitStr) - 1));
		action ["UseMagazine", _unit, _unit, _owner, _id];	// throws grenade
		sleep 1;
	};
	
	//can move again
	{if (!(_myGrp getVariable ['jac_isDugIn', false]) && !(_x getVariable ['jac_isInCover', false])) then {_x enableAi "PATH"}} forEach units _myGrp;
	_unit doWatch objNull; deleteVehicle _targetMarker;
	_unit setUnitLoadout _my_leaderLoadout; sleep 0.1; if (_restoreMinusOne) then {_unit removeItem _grenadeType};
	
	deleteWaypoint [_myGrp, (currentWaypoint _myGrp)];
	_myGrp addWaypoint [getPosASL _unit, -1, -1, 'tempWP']; sleep 0.1;
	deleteWaypoint [_myGrp, (count waypoints _myGrp)-1];
};

/// SMOKESCREEN

jac_fnc_addSmokeWP=
{
	params ["_grp", "_targetPos"];	
	if (!(local _grp) or (_grp getVariable["jac_isPanicked", false])) exitWith {false};
	
	(units _grp) doFollow (leader _grp);	// to make unit move in case it's in doStop mod
	{if (!(_grp getVariable ['jac_isDugIn', false]) && (_x getVariable ['jac_isInCover', false]) ) then {_x enableAi 'PATH'}}forEach (units _grp);	//enable path if in cover, but not dug in
	
    _wp = _grp addWaypoint [_targetPos, -1, (count (waypoints  _grp))+1, "CREATE SMOKESCREEN"];
    _wp setWaypointType "SCRIPTED";
    _wp setWaypointStatements [wp_SMKcond,""];
};

wp_SMKcond="
_my_leader=this;
_my_grp=(group _my_leader);
if !(local _my_grp) exitWith {false};

_wp= (currentWaypoint _my_grp);
_target_g= getWPPos [_my_grp,(currentWaypoint _my_grp)]; 

_minD= jac_SMKgrenadeRange;
if ((this distance2D (_target_g) <= _minD) && (waypointDescription [_my_grp, _wp] != 'wp_SMKcondActivated')) then {
[_my_grp, _wp] setWaypointDescription 'wp_SMKcondActivated';

  [_my_leader, _target_g, _my_grp, _wp] spawn { 
  params ['_my_leader', '_target_g', '_my_grp', '_wp'];

  deleteWaypoint [_my_grp,_wp];
  _my_grp addWaypoint [getPosASL _my_leader, -1, -1, 'tempWP'];
  sleep 0.1;
  
  deleteWaypoint [_my_grp, (count waypoints _my_grp)-1];  
  [_my_leader] spawn jac_fnc_wp_smokescreen;
 };
};
";

//tell vehicle to use internal smoke grenade launchers
jac_fnc_wp_smokescreen=
{
	params ['_unit'];

	if !(isNull objectParent _unit) exitWith
	{
		sleep 0.25; 
		[200, _unit, "CREATING SMOKESCREEN!"] remoteExecCall ["jac_fnc_zeusAddMSG3D", remoteExecutedOwner];
		[(vehicle _unit), "SmokeLauncher"] call BIS_fnc_fire;
	};
};


//////CAPTURE ENEMY
////When an enemy is pinned by sustained fire there's a chance his morale will break
//panicked groups will surrender if they are encircled

jac_fnc_addCaptureWP=
{
	params ["_grp", "_targetPos" , '_targetGRP'];
	
	(units _grp) doFollow (leader _grp);	// to make unit move in case it's in doStop mod
	_wp = _grp addWaypoint [_targetPos, -1, (count (waypoints  _grp))+1, str (_targetGRP)];
	
	_grp setVariable ['jac_captureWpTargetGrp', _targetGRP, true];
	_wp setWaypointType "SCRIPTED";
	_wp setWaypointStatements [jac_wp_CaptureCond,""];
};

jac_wp_CaptureCond="
_my_leader=this;
_my_grp=(group _my_leader);
if !(local _my_grp) exitWith {false};

_wp= (currentWaypoint _my_grp);
_target_g= getWPPos [_my_grp,(currentWaypoint _my_grp)]; 
_wpName= (waypointName [_my_grp, _wp]);

_minD= 12.5;
if ((this distance2D (_target_g) <= _minD) && (waypointDescription [_my_grp, _wp] != 'wp_condActivated')) then {
[_my_grp, _wp] setWaypointDescription 'wp_condActivated';

	[_my_leader, _target_g, _my_grp, _wp, _wpName] spawn { params ['_my_leader', '_target_g', '_my_grp', '_wp', '_wpName'];
		deleteWaypoint [_my_grp, _wp];
		_my_grp addWaypoint [getPosASL _my_leader, -1, -1, 'tempWP'];
		sleep 0.1;
		deleteWaypoint [_my_grp, (count waypoints _my_grp)-1];
		
		_targetGRP= (_my_grp getVariable ['jac_captureWpTargetGrp','']);
		if ( ((str _targetGRP) isEqualTo _wpName) && ((units _targetGRP) findIf {alive _x} != -1) ) then
		{
			_my_leader doWatch [_target_g # 0, _target_g # 1, (_target_g # 2) +1];
			_setdirto=(_my_leader getDir [_target_g # 0, _target_g # 1, _target_g # 2]);
			_my_leader setFormDir _setdirto; sleep 0.25;
			_my_leader playActionNow 'gestureCeaseFire';
			_my_leader doWatch objNull; [250, _my_leader, 'GIVE UP, NOW!'] remoteExecCall ['jac_fnc_zeusAddMSG3D', 0];
			_myCurators= ([_my_leader] call jac_fnc_objectCurators);
			[_my_leader, _target_g, _targetGRP, _myCurators] remoteExec ['jac_fnc_wp_captureTarget', 0];
		};
	};
};
";

jac_fnc_wp_captureTarget=
{
	params ['_unit', '_target_g', '_targetGRP', '_myCurators'];
	if !(local _targetGRP) exitWith {false};
	//check target status and apply captive status or deny surrender
	_target= (leader _targetGRP); _targetGRPunits= (units _targetGRP); sleep 0.75;
	if ( (_targetGRP getVariable["jac_isPanicked", false]) && !(_targetGRP getVariable["jac_isPOW", false]) ) then
	{
		if ( ( count (allUnits select {( (alive _x) && ([(side _targetGRP), (side _x)] call BIS_fnc_sideIsEnemy) && ((leader _targetGRP) distance _x < 60) )}) > //number of enemies nearby higher than friendlies
		(count (allUnits select {( (alive _x) && ((side _x) isEqualTo (side _targetGRP)) && ((leader _targetGRP) distance _x < 60) )})) ) ) then
		{						
			[300, _unit, "ENEMY CAPTURED"] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0];
			_targetGRP setVariable ['jac_actionMode', "maneuverMode", true];
			_targetGRP setVariable["jac_gotSuppressedTime", CBA_missionTime, true];
			_targetGRP setVariable["jac_isPanicked", false, true];						
			_targetGRP setSpeedMode "LIMITED"; _targetGRP setFormation "FILE";
			_targetGRP setVariable ['jac_isPOW', true, true];
			
			//POW groups are forced to walk standing, can't change formation
			{
				_unitCaptive= _x; 
				_unitCaptive setCaptive true; _unitCaptive forceWalk true;
				_unitCaptive disableAI "CHECKVISIBLE"; _unitCaptive disableAI "AUTOTARGET"; _unitCaptive disableAI "TARGET"; _unitCaptive enableAI "PATH";
				_unitCaptive setUnitCombatMode 'BLUE'; _unitCaptive setBehaviourStrong 'AWARE';
				_unitCaptive switchMove '';	 _unitCaptive playMoveNow '';
				_unitCaptive setVariable ['jac_isPOW', true, true];
				_unitCaptive setUnitPos "UP"; _unitCaptive enableAI "ANIM";
				_unitCaptive spawn {sleep 1; _this setVariable["is_pinned", false, true]; _this setSuppression 0;	_this disableAI "SUPPRESSION"; _this setVariable["jac_isPanicked", false, true]; _this action ["SwitchWeapon",_this,_this,-1]};
				
				// stop the fog of war routine
				_jacFog_targetsArray= format ["jacFog_targetsArray_%1", str (_unitCaptive getVariable["jacFog_handler", -1])];
				[_unitCaptive, _jacFog_targetsArray] call jac_fnc_endFoWSystem;
			}forEach _targetGRPunits;
			
			_zeusList= [_target] call jac_fnc_objectCurators;
			[[_targetGRPunits, _zeusList, _myCurators],
			{
				params ["_targetGRPunits", "_zeusList", "_myCurators"];
				{_x removeCuratorEditableObjects [_targetGRPunits, true]}forEach _zeusList;	//remove previous owner				
				sleep 0.1; {_x addCuratorEditableObjects [_targetGRPunits, true]}forEach _myCurators;		//assign to new
			}] remoteExecCall ['spawn', 2];	//spawn
		}
		else //capture failed
		{
			///panicked status is removed in any case so the units can move again - if the capture attempt failed they'll engage
			if !(_targetGRP getVariable["jac_isPOW", false]) then
			{
				_targetGRP setVariable["jac_isPanicked", false, true];
				_targetGRPunits doWatch _unit; _targetGRPunits doTarget _unit;			
				_targetGRP setVariable["jac_gotSuppressedTime", CBA_missionTime, true];
				{_x setSuppression 0} forEach _targetGRPunits;
				[250, _unit, "CAPTURE FAILED!"] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0];
				[_targetGRP, false] remoteExecCall  ["jac_fnc_setPanickedStatus", 0];
			};
		};
	};
};

/////GET IN VEHICLE WAYPOINT	
jac_fnc_addGetInWP=
{
	params ["_grp", "_targetPos", "_vehicle"];
	if !(local _grp) exitWith {false};
	
	(units _grp) doFollow (leader _grp);	// to make unit move in case it's in doStop mod
	_wp = _grp addWaypoint [_targetPos, -1, (count (waypoints  _grp))+1, "GETIN"];
	
    _wp waypointAttachVehicle _vehicle;
	_wp setWaypointType "SCRIPTED";
	_wp setWaypointStatements [wp_GETINcond,""];
};

wp_GETINcond="
_my_leader=this;
_my_grp=(group _my_leader);
if !(local _my_grp) exitWith {false};

_wp= (currentWaypoint _my_grp);
_target_g= getWPPos [_my_grp, _wp];
_transportOBJ= waypointAttachedVehicle [_my_grp, _wp];

if ((this distance2D (_target_g) <= 14) && (waypointDescription [_my_grp, _wp] != 'wp_GETINcondActivated')) then 
{
	deleteWaypoint [_my_grp,_wp];
	_my_grp addWaypoint [getPosASL _my_leader, -1, -1, 'tempWP'];
		
	[_my_grp, _wp] setWaypointDescription 'wp_GETINcondActivated';
	[{
		params ['_my_leader', '_target_g', '_my_grp', '_wp', '_transportOBJ'];
		deleteWaypoint [_my_grp, (count waypoints _my_grp)-1];
		[_transportOBJ, _my_grp] call jac_fnc_getInTransportWP;
	}, [_my_leader, _target_g, _my_grp, _wp, _transportOBJ], 0.1] call CBA_fnc_waitAndExecute;
};
";

jac_fnc_getInTransportWP=
{
	params ["_transportOBJ", "_grp"];

	_leaderGrp= (leader _grp);
	_myVehicle = (objectParent _leaderGrp);

	if ((isNull _myVehicle) or (_myVehicle == _transportOBJ)) then // either on foot or vehicle is the same
	{
		if ((units _grp findIf {alive _x} != -1) && !(_grp getVariable ["jac_isDugIn",false])) then	//someone is still alive and not in dug in mode
		{
			if ((_transportOBJ distance2D _leaderGrp < 14)) then	//in range of the vehicle
			{
				_grp setVariable ['jac_doFormUp', false, true]; //cancel stacking behind vehicle
				
				[250, _leaderGrp, "GET IN!"] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0];
				
				{if !(alive _x) then {_transportOBJ deleteVehicleCrew _x}} forEach crew _transportOBJ;	//delete dead vehicle crew members to open space
				
				[{
					params ["_transportOBJ", "_grp"];

					(units _grp) allowGetIn true;
					_grp addVehicle _transportOBJ;
				
					{if ( isNull (objectParent _x) ) then{ _x setVariable ['jac_endScriptedMove', true, true]}; _x setVariable ["jac_isClearingBuilding", false, true]}forEach (units _grp);	//cancel scripted move
				
					[{
						params ["_transportOBJ", "_grp"];

						(units _grp) orderGetIn true;
						{
							if !((objectParent _x) isEqualTo _transportOBJ) then
							{
								if ((damage _x) > 0.49) then {_x setDamage 0.4};		//heal units to prevent them from disembarking to heal
	
								if (_transportOBJ emptyPositions "Driver" > 0) exitWith {_x assignAsDriver _transportOBJ; _x moveInDriver _transportOBJ};
								if (_transportOBJ emptyPositions "Gunner" > 0) exitWith {_x assignAsGunner _transportOBJ; _x moveInGunner _transportOBJ};
								if (_transportOBJ emptyPositions "Commander" > 0) exitWith {_x assignAsCommander _transportOBJ; _x moveInCommander _transportOBJ};
								if (_transportOBJ emptyPositions "Turret" > 0) exitWith {_x moveInAny _transportOBJ};
								if (_transportOBJ emptyPositions "CargoNoFFV" > 0) exitWith {_x assignAsCargo _transportOBJ; _x moveInCargo [_transportOBJ, 0, true]};
								_x moveInAny _transportOBJ;
							};
						} forEach (units _grp);
						
						_transportOBJ setUnloadInCombat [false, false];
						_transportOBJ allowCrewInImmobile [true, true];				
				
						[{
							params ["_transportOBJ", "_grp"];
				
							//make sure the effectiveCommander of vehicle is always in the same group as the driver 
							_vehCommanderGRP= (group (effectiveCommander _transportOBJ));
							_driverGRP= (group (driver _transportOBJ));
							if !(_vehCommanderGRP isEqualTo _driverGRP) then {_transportOBJ setEffectiveCommander (leader _driverGRP)};

							//after finished embarking, check if vehicle had its ammo list initiated
							if ( (count (_transportOBJ getVariable ["jac_interactiveMenuArrayAmmoList", []])) == 0 ) then
							{[[_transportOBJ], true] remoteExecCall ["jac_fnc_initSelectAmmo", 0, true]};	//add weapon selection menu for the suppression action
						
						}, [_transportOBJ, _grp], 0.5] call CBA_fnc_waitAndExecute;
					}, [_transportOBJ, _grp], 0.5] call CBA_fnc_waitAndExecute;
				}, [_transportOBJ, _grp], 0.5] call CBA_fnc_waitAndExecute;
			}
			else
			{[250, _transportOBJ, "MOVE CLOSER TO THE VEHICLE!"] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0]};
		};
	};	
};



///////// FIRE SUPPORT SYSTEM

//LEGACY WP BASED FIRE SUPPORT SYSTEM
jac_fnc_wp_mortarSupport=
{
params ["_missionType", "_grpArray", "_targetPos", "_clientID"];
 _mortarTargetPos= _targetPos;
 _pos_mod=_mortarTargetPos select 2;
 _pos_mod=_pos_mod+jac_z_mod;
 _mortarTargetPos set [2,_pos_mod];
 _tacticalMenu_mouse_pos= +_mortarTargetPos;
 _hasFO= false;
  {
	if !(local _x) then {continue};
	_my_leader= (leader _x);
	if ((_x getVariable ["jac_isFO", 0]) > 0) then
	{
		_hasFO= true;
		//
		_leader_pos=eyePos _my_leader;
		_pos_mod=_leader_pos select 2;
		_pos_mod=_pos_mod;
		_leader_pos set [2,_pos_mod];
	
		_collision_lineArray= lineIntersectsSurfaces [_leader_pos, _mortarTargetPos, _my_leader, vehicle (_my_leader), true, 1, "VIEW", "NONE", true];
		if (count _collision_lineArray !=0) then
		{
			_collision_line= (_collision_lineArray # 0);
			_pos_mod=_collision_line select 0 select 2;
			_pos_mod=_pos_mod+jac_z_mod;
			_mortarTargetPos set [0, _collision_line select 0 select 0];
			_mortarTargetPos set [1, _collision_line select 0 select 1];
			_mortarTargetPos set [2, _pos_mod];  
		}
		else
		{
			_mortarTargetPos= +_tacticalMenu_mouse_pos;
		};
	
		//add WP
		_grp=_x;
		_wp = _grp addWaypoint [_mortarTargetPos, -1, (count (waypoints _grp))+1, _missionType];  // WP name will store the mission type
		_wp setWaypointType "SCRIPTED";
		_wp setWaypointStatements [wp_MORTARcond,""];
		_wp setWaypointDescription (str _clientID);	//client id to send messages from the server
	};
	if !(_hasFO)  then {[200, _my_leader, "NO FORWARD OBSERVER SELECTED!"] remoteExecCall ["jac_fnc_zeusAddMSG3D", _clientID]};
  }forEach _grpArray;
};


wp_MORTARcond="
_my_leader=this;
_my_grp=(group _my_leader);
if !(local _my_grp) exitWith {false};

_wp= (currentWaypoint _my_grp);
_target_g= getWPPos [_my_grp,_wp];
_missionType= waypointName [_my_grp,_wp];
_clientID= parseNumber (waypointDescription [_my_grp, _wp]);

_FOrange= _my_grp getVariable ['jac_isFO', jac_isFOdefaultRange];
_tooClose= false;
_inRange= false;
if (this distance2D (_target_g) < 60) then {_tooClose= true; [200, _my_leader, 'TARGET POSITION IS TOO CLOSE!'] remoteExecCall ['jac_fnc_zeusAddMSG3D', _clientID]};
if (this distance2D (_target_g) <= _FOrange) then {_inRange= true};

if ((_inRange) && (waypointDescription [_my_grp, _wp] != 'wp_McondActivated')) then {
[_my_grp, _wp] setWaypointDescription 'wp_McondActivated';

  [_my_leader, _target_g, _my_grp, _wp, _missionType, _tooClose] spawn { params ['_my_leader', '_target_g', '_my_grp', '_wp', '_missionType', '_tooClose'];

  deleteWaypoint [_my_grp,_wp];
  _my_grp addWaypoint [getPosASL _my_leader, -1, -1, 'tempWP'];
  sleep 0.1; 
  deleteWaypoint [_my_grp, (count waypoints _my_grp)-1]; 
  if !(_tooClose) then {[_my_leader, _target_g, _missionType] call jac_fnc_wp_MORTARscript};
 };
};
";

jac_fnc_wp_MORTARscript= 
{
params ["_FOcaller", "_target_pos", "_missionType"];
_callerGroup= group _FOcaller;
_senderGrpLeaderRadioRange= (_callerGroup getVariable ["jac_my_radio_range", 0]);
_FOspottingRange= (_callerGroup getVariable ["jac_isFO", 0]);

if !(alive _FOcaller) exitWith {[300, _FOcaller, "LOST CONTACT WITH FORWARD OBSERVER!"] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0]};
if (_FOcaller getVariable ["jac_fireMissionActive", false]) exitWith {[200, _FOcaller, "FORWARD OBSERVER IS OCCUPIED!"] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0]};
if ((_FOcaller distance2D _target_pos) > _FOspottingRange) exitWith {[200, _FOcaller, "TARGET IS TOO FAR AWAY!"] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0]};

_foundMortarUnit= false;  //if found a suitable unit
_isInRadioRange= true;
scopeName "jac_fnc_wp_MORTARscript_main";
 {
 _mortarGroup= _x;
 _mortarUnit= leader _x;
 _mortarTurret= vehicle _mortarUnit;
 
 //for allGroups, find suitable mortar team; same faction, is a mortar, mortar is deployed, has ammo, in range, not on fire mission already 
 if ((side _FOcaller == side _mortarUnit) && !(isNull objectParent _mortarUnit)) then {
  
  //  get ammo type, if any
  _ammoType="";
  if (_missionType == "HE shells") then {_ammoType= _mortarGroup getVariable ["jac_MortarHE", ""]};
  if (_missionType == "smoke shells") then {_ammoType= _mortarGroup getVariable ["jac_MortarSMK", ""]};
  
  // check if in comms range of caller or a HQ that can relay info 
  _isInRadioRange= [_callerGroup, _mortarGroup] call jac_fnc_checkCommsList;
  
   if ((_isInRadioRange) && (someAmmo _mortarTurret) && (_ammoType != "") && (_target_pos inRangeOfArtillery [[_mortarTurret], _ammoType]) && !(_mortarGroup getVariable ["jac_fireMissionActive", false])) then { //found mortar, call fire mission
   _foundMortarUnit=true;
      
   //can't execute fire mission in MANEUVER MODE
   if (_mortarGroup getVariable ['jac_actionMode', 'combatMode'] isEqualTo 'maneuverMode') then 
   {[325, _mortarUnit, "switch to COMBAT mode to fire"] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0]; _foundMortarUnit=false; continue};
	
	[_FOcaller, _callerGroup, _target_pos, _missionType, _mortarTurret, _ammoType, _mortarGroup, _mortarUnit] spawn { params ["_FOcaller", "_callerGroup", "_target_pos", "_missionType", "_mortarTurret", "_ammoType", "_mortarGroup", "_mortarUnit"];	

		//look at target, announce fire mission
		_callerGroup setVariable ["jac_doEndFireMission", false, true];
		[550, _FOcaller, ("requesting " + _missionType)] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0];
		_FOcaller doWatch _target_pos; //_FOcaller selectWeapon (binocular _FOcaller);
		if !(unitIsUAV _FOcaller) then {[_FOcaller, 'PATH'] remoteExec ["disableAI", 0]};			//don't disable path for UAVs, to prevent them from flying away uncontrolled
		_FOcaller setVariable ["jac_fireMissionActive", true, true];
		
		_myLogicOBJ= _mortarGroup getVariable ["jac_myTurretObjectLogic", objNull];		
		_azimuth= (_mortarTurret getDir (_target_pos));
		if !(isNull _myLogicOBJ) then {_myLogicOBJ setDir _azimuth};		
		_mortarGroup setFormDir _azimuth;
		_mortarGroup setVariable ["jac_fireMissionActive", true, true];
		
		//draw target position marker, id is the mortar unit
		_target_posMarker= _target_pos;
		[[_target_posMarker],
		{
			params ['_target_posMarker'];
			jac_draw3dIconList pushBack [_target_posMarker, "\A3\ui_f\data\map\markers\handdrawn\objective_CA.paa", _target_posMarker];
		}] remoteExecCall ["call", 0];
 
		//call fire mission script 3 times, each time making the area hit smaller and more accurate
		//if the FO is killed, cancel fire mission immediately
        
		[300, _mortarUnit, "calculating fire mission..."] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0];
		sleep 16;
		_repeatMortarFire= jac_mortarSalvoSizeConstant;		//jac_mortarSalvoSizeConstant= 3;
		_mortarFireArea= jac_mortarFireAreaConstant;       //jac_mortarFireAreaConstant= 60;
		if (_missionType == "smoke shells") then {_mortarFireArea= (_mortarFireArea * 0.5)};
		_areaErrorFactor= 1;
		
		while {_repeatMortarFire >0 } do
		{
			//FO died
			if !(alive _FOcaller) exitWith 
			{[300, _mortarUnit, "LOST CONTACT WITH FORWARD OBSERVER!"] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0]; _FOcaller setVariable ["jac_fireMissionActive", false, true]; _mortarGroup setVariable ["jac_fireMissionActive", false, true]};
			//mortar destroyed
			if !(alive _mortarTurret) exitWith 			
			{[300, _FOcaller, "LOST CONTACT WITH SUPPORT UNIT!"] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0]; if (!(_FOcaller getVariable ["jac_isDugIn",false])&& !(_FOcaller getVariable ["jac_isInCover",false])) then {_FOcaller enableAI "PATH"}; _FOcaller setVariable ["jac_fireMissionActive", false, true]; _mortarGroup setVariable ["jac_fireMissionActive", false, true]};
			//cancel action
			if (_callerGroup getVariable ["jac_doEndFireMission",false]) exitWith 
			{_callerGroup setVariable ["jac_doEndFireMission", false, true]; if (!(_FOcaller getVariable ["jac_isDugIn",false])&& !(_FOcaller getVariable ["jac_isInCover",false])) then {_FOcaller enableAI "PATH"}; _FOcaller setVariable ["jac_fireMissionActive", false, true]; _mortarGroup setVariable ["jac_fireMissionActive", false, true]};
			
			_targetPosLengthMod= (random [(_mortarFireArea * (_areaErrorFactor - 0.05)), (_mortarFireArea * (_areaErrorFactor - 0.025)), (_mortarFireArea * (_areaErrorFactor))]);
			_currentTargetPos= (_target_pos getPos [_targetPosLengthMod, (random 359)]);
			
			[[_mortarTurret, _currentTargetPos, _ammoType, _mortarGroup],  
			{
				params ['_mortarTurret', '_currentTargetPos', '_ammoType', '_mortarGroup'];
				if !(local _mortarGroup) exitWith {false};
				if ((jac_infinite_ammo_suppress_setting_veh) && !(jac_forceAmmoUseSetting)) then {_mortarTurret setvehicleammo 1};
				_mortarTurret doArtilleryFire [_currentTargetPos, _ammoType, 1];
			}] remoteExecCall ["call", 0];
			
			_repeatMortarFire= (_repeatMortarFire - 1);
			_areaErrorFactor= (_areaErrorFactor - 0.475);
			[300, _mortarUnit, "FIRING!"] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0];
			sleep jac_mortarFireWaitTime;	//12
			//increase smoke effect
			if (_missionType == "smoke shells") then {_smoke = "SmokeShell" createVehicle _currentTargetPos};
		};
		
		////delete target position marker, id is the mortar unit
		[[_target_posMarker],  
		{
			params ['_target_posMarker'];
			{if ((_x # 2) isEqualTo _target_posMarker) then {jac_draw3dIconList deleteAt _forEachIndex}}forEach jac_draw3dIconList;
		}] remoteExecCall ["call", 0];
		
		[300, _mortarUnit, "ROUNDS COMPLETE!"] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0];
		_mortarGroup setVariable ["jac_fireMissionActive", false, true];
		_FOcaller setVariable ["jac_fireMissionActive", false, true];
		_FOcaller doWatch objNull;
		
		if (!(_FOcaller getVariable ["jac_isDugIn",false])&& !(_FOcaller getVariable ["jac_isInCover",false])) then {[_FOcaller, 'PATH'] remoteExec ["enableAI", 0]};
	}; //spawn	
	
	//breakTo "jac_fnc_wp_MORTARscript_main";
   };
  };
 }forEach allGroups;
 
 if (!(_foundMortarUnit) && (_isInRadioRange)) then {[300, _FOcaller, "NO FIRE SUPPORT AVAILABLE AT THE MOMENT!"] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0]};
 if (!(_foundMortarUnit) && !(_isInRadioRange)) then {[350, _FOcaller, "OUT OF RADIO RANGE"] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0]};
};


//NEW FIRE SUPPORT MENU
//open a menu with 3 collumns and 2 buttons for set area mode and cancel
//1- list of units
//2- drop down menu, click to select the ammo type from the list or keep selected "don't fire"
//3- drop down menu, select the type of fire mission - suppression= low intensity, long duration; destruction= full magazine salvo, short duration
//confirm to enter the set target area mode
jac_fnc_openFireSupportMenu=
{
	params ["_caller"];
	
	//build arty list - is alive - same side - has a vehicle/turret - is artillery - not in a fire mission already - has ammo - is in comms range
	ctrlFireSupportMenuSelectionCaller= _caller;
	setCuratorSelected [];	//unselect units
	[jac_minimalistZeusInterface] call jac_fnc_minimalistUIactivate;
	
	_callerGroup= group _caller; _menu_pos= [((safeZoneW - (tactical_menuW*3)) / 2 + safeZoneX), ((safeZoneH - (tactical_menuH)) / 2 + safeZoneY)];
	_artyList= (allGroups select {( (alive leader _x) && ((side _caller) isEqualTo (side (leader _x))) && !(isNull objectParent (leader _x)) && ((vehicle (leader _x)) getVariable ["jac_MortarHE", ""] != "") &&
	([_callerGroup, _x] call jac_fnc_checkCommsList) )});
	
	if (count _artyList == 0) exitWith {titleText ["NO ARTILLERY UNITS AVAILABLE AT THE MOMENT!", "PLAIN"]};
	_menuRow= 0; ctrlFireSupportMenuSelection= [];	//list of arrays with unit, ammo, mission type
	{
		//list of units
		_group= _x; _artyUnit= vehicle (leader _group);
		_ctrl1 = findDisplay 312 ctrlCreate ["RscButton", -1];
		ctrlFireSupportMenuList pushBack _ctrl1;
		_ctrl1 ctrlSetPosition [_menu_pos # 0,(((_menu_pos # 1) + 0.001) + ((0.0395)*_menuRow)),tactical_menuW,tactical_menuH];
		_ctrl1 ctrlSetText ((str _group) select [1]);
		_ctrl1 ctrlSetFont jac_ctrlFont;
		_ctrl1 ctrlSetBackgroundColor [0, 0, 0, 0.85];
		_ctrl1 ctrlCommit 0;

		//add an ammo list menu for each unit
		//_ammoList= (getArray (configFile >> "CfgVehicles" >> typeOf _artyUnit >> "Turrets" >> "MainTurret" >> "magazines"));
		//_turretPath=_artyUnit unitTurret (gunner _artyUnit); if (count _turretPath > 0) then {_ammoList= _artyUnit magazinesTurret _turretPath};
		//_ammoList= _ammoList arrayIntersect _ammoList;
		_ammoData= (_artyUnit getVariable ["jac_interactiveMenuArrayAmmoList", []]); _ammoList= [];
		if (count _ammoData > 0) then { {_ammoList pushBack (_x # 4) # 3}forEach _ammoData };
		
		_ctrl2 = findDisplay 312 ctrlCreate ["RscCombo", -1];	
		ctrlFireSupportMenuList pushBack _ctrl2;
		_ctrl2 ctrlSetPosition [(_menu_pos # 0) + (tactical_menuW + 0.002),(((_menu_pos # 1) + 0.001) + ((0.0395)*_menuRow)),tactical_menuW,tactical_menuH];
		_ctrl2 ctrlSetFont jac_ctrlFont;
		_ctrl2 ctrlSetBackgroundColor [0.2, 0.2, 0.2, 0.85];
		_ctrl2 lbAdd "Don't Fire";
		{_ctrl2 lbAdd (format ['%1', (getText (configFile >> "CfgMagazines" >> _x >> "displayName"))])}forEach _ammoList;
		_ctrl2 setVariable ["jac_ammoList", _ammoList];
		
		//set fire mission parameters = [arty unit, magazine, fire mission type]
		//friendly arty units that are not in the player zeus control list will be set to 'dont fire' by default
		if ( (count _ammoList > 0) && ( _artyUnit in (curatorEditableObjects (getAssignedCuratorLogic player)) ) ) then
		{_ctrl2 lbSetCurSel 1;  ctrlFireSupportMenuSelection pushBack [_group, _ammoList # 0, "3 Rounds"]}
		else
		{_ctrl2 lbSetCurSel 0; ctrlFireSupportMenuSelection pushBack [_group, "Don't Fire", "3 Rounds"]};
		_ctrl2 ctrlCommit 0;
		
		//add menu to select fire mission type
		_ctrl3 = findDisplay 312 ctrlCreate ["RscCombo", -1];   
		ctrlFireSupportMenuList pushBack _ctrl3;
		_ctrl3 ctrlSetPosition [(_menu_pos # 0) + ((tactical_menuW + 0.002)*2),(((_menu_pos # 1) + 0.001) + ((0.0395)*_menuRow)),tactical_menuW,tactical_menuH];
		_ctrl3 ctrlSetFont jac_ctrlFont;
		_ctrl3 ctrlSetBackgroundColor [0.2, 0.2, 0.2, 0.85];
		_ctrl3 lbAdd "3 Rounds"; _ctrl3 lbAdd "Full Magazine"; _ctrl3 lbAdd "Target Suppression";
		_ctrl3 lbSetCurSel 0; 
		_ctrl3 ctrlCommit 0;
		
		////event handler to update fire mission parameters		
		{
			_x setVariable ["jac_myMenuRow", _menuRow];
			_x setVariable ["jac_myMenuIndex", (_forEachIndex + 1)];
			_x ctrlAddEventHandler ["LBSelChanged", {params ["_control", "_lbCurSel", "_lbSelection"];
				if (_lbCurSel != 0) then	//the first index is reserved
				{
					_menuRow= _control getVariable ["jac_myMenuRow", objNull];
					_menuIndex= _control getVariable ["jac_myMenuIndex", objNull];
					_ammoList= _control getVariable ["jac_ammoList", []];
					_value= (_control lbText _lbCurSel);		
					
					if (count _ammoList > 0) then {_value= _ammoList # (_lbCurSel-1)};
					(ctrlFireSupportMenuSelection # _menuRow) set [_menuIndex, _value];
				} else
				{
					_menuRow= _control getVariable ["jac_myMenuRow", objNull];
					_menuIndex= _control getVariable ["jac_myMenuIndex", objNull];
					_value= (_control lbText _lbCurSel);
					(ctrlFireSupportMenuSelection # _menuRow) set [_menuIndex, _value];
				};
			}];
		}forEach [_ctrl2, _ctrl3];
		_menuRow=_menuRow+1;
	} forEach _artyList;
	
	/// set fire mission name button
	_ctrl1 = findDisplay 312 ctrlCreate ["RscEdit", -1];  
	ctrlFireSupportMenuList pushBack _ctrl1;
	_ctrl1 ctrlSetPosition [(_menu_pos # 0) + (tactical_menuW + 0.002),(((_menu_pos # 1) + 0.001) - ((0.0395)*3)),tactical_menuW,tactical_menuH];
	_ctrl1 ctrlSetFont jac_ctrlFont;
	_ctrl1 ctrlSetBackgroundColor jac_ctrlColor;
	_ctrl1 ctrlSetText 'FIRE MISSION ID';	
	_ctrl1 ctrlCommit 0;
	
	/// set target area button
	_ctrl = findDisplay 312 ctrlCreate ["RscButton", -1];  
	ctrlFireSupportMenuList pushBack _ctrl;
	_ctrl ctrlSetPosition [(_menu_pos # 0) + (tactical_menuW + 0.002),(((_menu_pos # 1) + 0.001) - ((0.0395)*2)),tactical_menuW,tactical_menuH];
	_ctrl ctrlSetFont jac_ctrlFont;
	_ctrl ctrlSetBackgroundColor jac_ctrlColor;
	_ctrl ctrlSetText 'Set Target Area';
	_ctrl ctrlCommit 0;
	_ctrl setVariable ["jac_setFireSupportAreaNameCtrl", _ctrl1];
	
	_ctrl ctrlAddEventHandler ["MouseButtonUp", {_thisCtrl = (_this # 0); _button = (_this # 1);
		///left click to enter the set target area mode
		if ((_button == 0) && !(jac_setFireSupportArea)) then	
		{
			jac_ambushKillzoneP1=[]; jac_ambushGroups=[]; jac_setAmbushArea= false; jac_setFireSupportArea= true;
			_ctrlText= (ctrlText (_thisCtrl getVariable ["jac_setFireSupportAreaNameCtrl", _thisCtrl]));
			jac_setFireSupportAreaName= (_ctrlText select [0, 20]);
			if (jac_setFireSupportAreaName isEqualTo "FIRE MISSION ID") then {jac_setFireSupportAreaName= "Fire Mission "+ (str (round CBA_missionTime))};
			setCuratorSelected [vehicle (leader ctrlFireSupportMenuSelectionCaller)];
			[jac_minimalistZeusInterface] call jac_fnc_minimalistUIactivate;
			titleText ["CLICK AND DRAG THE LEFT MOUSE BUTTON TO SET THE TARGET AREA", "PLAIN"];
			call jac_fnc_closeFireSupportMenu;
		};
	}];
	
	/// cancel button
	_ctrl = findDisplay 312 ctrlCreate ["RscButton", -1];  
	ctrlFireSupportMenuList pushBack _ctrl;
	_ctrl ctrlSetPosition [(_menu_pos # 0) + (tactical_menuW + 0.002),(((_menu_pos # 1) + 0.001) - ((0.0395)*1)),tactical_menuW,tactical_menuH];
	_ctrl ctrlSetFont jac_ctrlFont;
	_ctrl ctrlSetBackgroundColor jac_ctrlColor;
	_ctrl ctrlSetText 'Cancel';
	_ctrl ctrlCommit 0;
	
	_ctrl ctrlAddEventHandler ["MouseButtonDown", {_thisCtrl = (_this # 0); _button = (_this # 1);
		if (_button == 0) then	{call jac_fnc_closeFireSupportMenu};
	}];
};

jac_fnc_closeFireSupportMenu=
{
	{ctrlDelete _x} forEach ctrlFireSupportMenuList; ctrlFireSupportMenuList= [];
};

//execute the firing routine based on the given parameters; number of rounds, target area and type of fire mission
//[arty unit, magazine, fire mission type, target area]
jac_fnc_fireSupportFiringRoutine=
{
	params ["_group", "_magazine", "_fireMission", "_targetArea", "_logic", "_dropSmokeShell"];
	if !(local _group) exitWith {false};
	
	_unit= (vehicle (leader _group)); _commander= (effectiveCommander _unit); _fireGroup= _group; _canFire= true;
	
	///check conditions for a fire mission
	if (_fireGroup getVariable ["jac_fireMissionActive", false]) then 
	{_msg= "FIRE MISSION IN PROGRESS"; [325, _commander, _msg] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0]; _unit sideChat _msg; _canFire= false}; //fire mssion already in progress
	
	if !((_targetArea # 4) inRangeOfArtillery [[_unit], _magazine]) then 
	{_msg= "OUT OF RANGE"; [325, _commander, _msg] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0]; _unit sideChat _msg; _canFire= false};	//check if it's in range		
	
	if !(someAmmo _unit) then 
	{_msg= "OUT OF AMMO"; [325, _commander, _msg] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0]; _unit sideChat _msg; _canFire= false};	//no ammo	
	
	if (_fireGroup getVariable ['jac_actionMode', 'combatMode'] isEqualTo 'maneuverMode') then 
	{_msg= "switch to COMBAT mode to fire"; [325, _commander, _msg] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0]; _unit sideChat _msg; _canFire= false}; //can't execute fire mission in MANEUVER MODE		
	
	if (_magazine == "Don't Fire") then
	{_msg= "NO AMMO SELECTED"; [325, _commander, _msg] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0]; _canFire= false};
	
	if (_canFire) then
	{
		if (_dropSmokeShell) then {"SmokeShellOrange" createVehicle (_targetArea # 4)};	//drop orange smoke grenade on target position
		
		_interval= 0; _toFire= 3;
		switch (_fireMission) do 
		{
			case "Full Magazine": {_toFire= (getNumber (configFile >> "CfgMagazines" >> _magazine >> "count"))};
			case "Target Suppression": {_toFire= 120; _interval= 10};
		};
		
		if (_toFire == 1) then {_toFire= 40};	//fix for RHS bm-21 that has ammo count in config == 1
		
		//make unit look to target pos (otherwise it miht not fire all shots)
		_azimuth= (_unit getDir ((_targetArea # 4)));
		_myLogicOBJ= _fireGroup getVariable ["jac_myTurretObjectLogic", objNull];
		if !(isNull _myLogicOBJ) then {_myLogicOBJ setDir _azimuth};		
		_fireGroup setFormDir _azimuth;
		
		_fireGroup setVariable ["jac_fireMissionActive", true, true];
		[1200, _commander, "preparing fire mission..."] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0];
		_unit sideChat "preparing fire mission...";
		sleep 12;
		_targetAreaW= ((_targetArea # 0) distance (_targetArea # 3));	//upper left to upper right
		_targetAreaH= ((_targetArea # 0) distance (_targetArea # 1)); //upper left to lower left
		_doFire= true; _ii=0;
		
		_ammoName= (getText (configFile >> "CfgMagazines" >> _magazine >> "displayName"));
		
		while {_doFire} do 
		{
			//prepare to fire
			_ii= _ii+1;
			_maxWait= (CBA_missionTime + 120);
			
			//load correct ammo
			if ((currentMagazine _unit) != _magazine) then 
			{
				_msg= format ["loading %1...", _ammoName];
				[325, _commander, _msg] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0];
				_unit sideChat _msg;
				_unit loadMagazine [(_unit unitTurret (gunner _unit)), (currentWeapon _unit), _magazine];
			};
			
			//wait until correct ammo is loaded or cancelled fire mission - don't put waitUntil unit ready here
			waitUntil { sleep 0.01; ( ( ((currentMagazine _unit) == _magazine) ) or (_fireGroup getVariable ["jac_doEndFireMission",false]) or !(alive _unit) or (CBA_missionTime > _maxWait) )};
			
			if ( (_fireGroup getVariable ["jac_doEndFireMission",false]) or !(alive _unit) ) exitWith {_doFire= false};	//if should end fire mission
			
			//divide target area in a grid of 1 meter squares - select a random position from the grid
			_posX= round (random _targetAreaW); _posY= round (random _targetAreaH);
			_targetPos= (_targetArea # 0) getPos [_posX, ((_targetArea # 0) getDir (_targetArea # 3)) ];
			_targetPos= _targetPos getPos [_posY, ((_targetArea # 0) getDir (_targetArea # 1)) ];
			
			//fire
			_unit doartilleryfire [_targetPos, _magazine, 1];
			_msg= format ["FIRING %1", _ammoName];
			[325, _commander, format ["FIRING %1", _ammoName]] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0];
			_unit sideChat _msg;
			
			//wait unit is ready or cancelled fire mission - waitUntil unit ready has to be here
			_intervalTime=  (CBA_missionTime + _interval);
			_maxWait= (CBA_missionTime + 120);
			waitUntil { sleep 0.01; ( ( (unitReady _unit) && (CBA_missionTime > _intervalTime) ) or (_fireGroup getVariable ["jac_doEndFireMission",false]) or !(alive _unit) or (CBA_missionTime > _maxWait) )};
			
			if ((jac_infinite_ammo_suppress_setting_veh) && !(jac_forceAmmoUseSetting)) then {_unit setvehicleammo 1};	//restore ammo if setting is on

			//if should end fire mission
			if !(someAmmo _unit) exitWith {_msg= "OUT OF AMMO"; [325, _commander, _msg] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0]; _unit sideChat _msg; _doFire= false; };	//no ammo
			if ( (_ii >= _toFire) or (_fireGroup getVariable ["jac_doEndFireMission",false]) or !(alive _unit) ) exitWith {_doFire= false}; // arty unit destroyed or cancel action
		};
		
		if (alive _unit) then
		{
			_waitTime= 4; _msg= "ROUNDS COMPLETE";
			if (_fireGroup getVariable ["jac_doEndFireMission",false]) then {_waitTime= 1; _msg= "CANCELLED FIRE MISSION"};
			sleep _waitTime;
			[325, _commander, _msg] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0];
			_unit sideChat _msg;
		};
	};
	
	//reset variables to call fire mission again
	_logic setVariable ["jac_fireMissionActive", false, true];
	_fireGroup setVariable ["jac_doEndFireMission", false, true];
	_fireGroup setVariable ["jac_fireMissionActive", false, true];	
};


////fire support logic object menu
jac_fnc_fireSupportLogicObjectMenuInit=
{
	params ["_obj", "_fireSupportMenuParameters", "_FOunit"];	
	_optionData= _obj getVariable ['jac_interactiveMenuArray', []];
	_optionData pushBack ["EXECUTE FIRE MISSION", "jac_fnc_confirmFireMission", "Start the Fire Mission.", 0, _obj];	//start fire mission
	_optionData pushBack ["CANCEL", "jac_fnc_deleteContextSensitiveMenuHandler", 'Cancel the fire mission.', 0, _obj];				//cancel
	_obj setVariable ['jac_interactiveMenuArray', _optionData, true];
	
	_obj setVariable ["jac_my3dIcon", "\A3\ui_f\data\map\markers\military\end_CA.paa", true];
	_obj setVariable ["jac_my3dIconText", jac_setFireSupportAreaName, true];
	jac_logicIconList pushBack _obj;
	
	_obj setVariable ['jac_fireSupportMenuParameters', _fireSupportMenuParameters, true];

	//add suppression logic obj to zeus
	_zeusList= [_FOunit] call jac_fnc_objectCurators;
	[[_obj, _zeusList],
	{
		params ["_obj", "_zeusList"];
		{_x addCuratorEditableObjects [[_obj],true]}forEach _zeusList;
	}] remoteExecCall ['call', 2];
};

jac_fnc_confirmFireMission=
{
	params ["_ctrlPos", "_ctrlOBJ", "_grpArray", "_optionalArgument"];
	if !(local _ctrlOBJ) exitWith {false};
	if !(_ctrlOBJ getVariable ["jac_fireMissionActive", false]) then
	{
		_ctrlOBJ setVariable ["jac_fireMissionActive", true, true];
		_ctrlOBJ spawn {sleep 15; _this setVariable ["jac_fireMissionActive", false, true]};
		_area= _ctrlOBJ getVariable ['jac_fireSupportArea', []]; _fireSupportMenuParameters= _ctrlOBJ getVariable ['jac_fireSupportMenuParameters', []];		
		
		{
			_dropSmokeShell= ( (jac_confirmFireMissionSmokeShell) && (_forEachIndex ==  0) );
			[(_x # 0), (_x # 1), (_x # 2), _area, _ctrlOBJ, _dropSmokeShell] remoteExec ['jac_fnc_fireSupportFiringRoutine', 0];	//[arty unit, magazine, fire mission type, target area]
		}forEach _fireSupportMenuParameters;
	} else
	{[400, _ctrlOBJ, "WAIT BEFORE CALLING THIS FIRE MISSION AGAIN!"] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0]};
};


//////// LASER TARGETING waypoint
//add laser designator WP
jac_fnc_addActivateLaserDesignatorWP=
{
params ["_grp", "_targetPos"];
if !(local _grp) exitWith {false};

	(units _grp) doFollow (leader _grp);	// to make unit move in case it's in doStop mod
	{if (!(_grp getVariable ['jac_isDugIn', false]) && (_x getVariable ['jac_isInCover', false]) ) then {_x enableAi 'PATH'}}forEach (units _grp);	//enable path if in cover, but not dug in
	
	_wp = _grp addWaypoint [_targetPos, -1, (count (waypoints  _grp))+1, "laser designator"];
	_wp setWaypointType "SCRIPTED"; _wp setWaypointStatements ["true", wp_laserDesignatorStatement];
};

wp_laserDesignatorStatement="
_my_leader=this;
_my_grp=(group _my_leader);
if !(local _my_grp) exitWith {false};

_wp= (currentWaypoint _my_grp);
_target_g= getWPPos [_my_grp,(currentWaypoint _my_grp)];

if ((this distance (_target_g) <= jac_isFOdefaultRange) && (waypointDescription [_my_grp, _wp] != 'wp_fn_laserDesignatorStatementcondActivated')) then 
{
	[_my_grp, _wp] setWaypointDescription 'wp_fn_laserDesignatorStatementcondActivated';
	[_my_leader, _target_g, _my_grp, _wp] spawn { params ['_my_leader', '_target_g', '_my_grp', '_wp'];
		deleteWaypoint [_my_grp,_wp];
		_my_grp addWaypoint [getPosASL _my_leader, -1, -1, 'tempWP']; sleep 0.1;
		deleteWaypoint [_my_grp, (count waypoints _my_grp)-1];
		_losPos= AGLToASL _target_g;
		_collision_lineArray= lineIntersectsSurfaces [(eyePos _my_leader), AGLToASL _target_g, _my_leader, vehicle _my_leader, true, 1, 'VIEW', 'NONE', true];
		if (count _collision_lineArray !=0) then {_losPos= (_collision_lineArray # 0) # 0};
		if ((_losPos distance AGLToASL _target_g) > 6) then {titleText ['THE LASER DESIGNATOR DOESNT HAVE LINE OF SIGHT TO THE TARGET POSITION', 'PLAIN']} else
		{[_my_leader, _target_g] call wp_fn_laserDesignatorStatement};
	};
};
";

//create laser target for guided missiles and shells
wp_fn_laserDesignatorStatement=
{
	params ["_unit", "_targetPos"];
	_laser= _unit getVariable ["jac_myLaserTargetLogic", objNull];
	if !(isNull _laser) then {deleteVehicle _laser; sleep 1};	//cancelling previous laser target
	
	if (isNull objectParent _unit) then {_unit selectWeapon (binocular _unit)};
	[300, _unit, "PAINTING TARGET WITH LASER"] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0];

	_targetType= if ((side _unit) getfriend west > 0.6) then {"LaserTargetW"} else {"LaserTargetE"};
	_laserTarget= createvehicle [_targetType, _targetPos,[],0,"CAN_COLLIDE"];
	_laserLogic= createagent ['jac_wargameLogic', [0,0,0],[],0,'CAN_COLLIDE']; sleep 0.25;
	_laserLogic setPos _targetPos; _laserTarget setPos _targetPos;
	_laserTarget attachTo [_laserLogic];
	_unit reveal _laserTarget; _unit dowatch _laserTarget; _unit dotarget _laserTarget;
	(side _unit) reportRemoteTarget [_laserTarget, 36000]; _laserTarget confirmSensorTarget [(side _unit), true];
	
	_unit setVariable ["jac_myLaserTargetLogic", _laserLogic, true];
	_startPos= getPosASL _unit; _target_g= getPosASL _laserTarget;
	[_unit, _laserLogic, _laserTarget] call jac_fnc_laserDesignatorLogicObjectMenuInit;
	_attachTarget= (position _laserLogic) nearEntities [["Man", "Air", "Ship", "LandVehicle"], 5];
	if (count _attachTarget > 0) then {_laserLogic attachTo [(_attachTarget # 0)]};
	jac_drawLaserList pushBack _unit;
	
	[{
		params ["_unit", "_targetPos", "_laserTarget", "_startPos", "_laserLogic"];
		_target_g= getPosASL _laserTarget; _losPos= _target_g;
		_collision_lineArray= lineIntersectsSurfaces [(eyePos _unit), _target_g, _unit, vehicle _unit, true, 1, 'VIEW', 'NONE', true];
		if (count _collision_lineArray != 0) then {_losPos= (_collision_lineArray # 0) # 0};
		( !(alive _unit) or ( !((vehicle _unit) isKindOf "Air") && ((getPosASL _unit distance2D _startPos) > 2) ) or !(alive _laserLogic) or ((group _unit) getVariable ["jac_isCancelAction", false]) or ((_losPos distance _target_g) >= 5) );
	}, {
		params ["_unit", "_targetPos", "_laserTarget", "_startPos", "_laserLogic"];
		deleteVehicle _laserLogic; deleteVehicle _laserTarget;
		if (alive _unit) then {[300, _unit, "CANCEL LASER DESIGNATION"] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0]};
		_unit setVariable ["jac_myLaserTargetLogic", objNull, true];
	}, [_unit, _targetPos, _laserTarget, _startPos, _laserLogic]] call CBA_fnc_waitUntilAndExecute;
};

jac_fnc_laserDesignatorLogicObjectMenuInit=
{
	params ["_unit", "_obj", "_laserTarget"];
	_optionData= _obj getVariable ['jac_interactiveMenuArray', []];
	_optionData pushBack ["TARGET LASER", "jac_fnc_targetLaserDesignator", "Tell the selected vehicle to fire a guided munition at this Laser Target if available.", 1, _obj];	//start fire mission
	_optionData pushBack ["CANCEL", "jac_fnc_deleteContextSensitiveMenuHandler", 'Stop the laser designator.', 0, _obj];				//cancel
	_obj setVariable ['jac_interactiveMenuArray', _optionData, true];
	_obj setVariable ["jac_my3dIcon", "\A3\ui_f\data\map\markers\military\destroy_CA.paa", true]; jac_logicIconList pushBack _obj;

	//add suppression logic obj to zeus
	_zeusList= [_unit] call jac_fnc_objectCurators;
	[[_obj, _zeusList],
	{
		params ["_obj", "_zeusList"];
		{_x addCuratorEditableObjects [[_obj],true]}forEach _zeusList;
	}] remoteExecCall ['call', 2];
};

jac_fnc_targetLaserDesignator=
{
params ["_ctrlPos", "_ctrlOBJ", "_grpArray", "_optionalArgument"];
if (_ctrlOBJ getVariable ["jac_executingLaserFire", false]) exitWith {false};
[_ctrlPos, _ctrlOBJ, _grpArray, _optionalArgument] spawn { params ["_ctrlPos", "_ctrlOBJ", "_grpArray", "_optionalArgument"];
	//check if selected vehicle has a laser guided weapon and use it on the laser target
	{
		_ctrlOBJ setVariable ["jac_executingLaserFire", true];
		_veh= vehicle leader _x;
		
		//find weapon that fires laser guided ammo
		_ammoData= (_veh getVariable ["jac_interactiveMenuArrayAmmoList", []]);
		
		//make sure the currently selected ammo is the first on the list
		if !((_veh getVariable ["jac_vehSelectedWeapon", ""]) isEqualTo "") then
		{_ammoData insert [ 0, [[ "","","",1, [(_veh getVariable ["jac_vehSelectedWeapon", ""]), (_veh getVariable ["jac_vehSelectedMuzzle", ""]), (_veh getVariable ["jac_vehSelectedTurretPath", ""]), (_veh getVariable ["jac_vehSelectedMagazine", ""])] ]] ]};
		
		_targetType= if ((side leader _x) getfriend west > 0.6) then {"LaserTargetW"} else {"LaserTargetE"};
		
		{
			_weapon= ((_x # 4) # 0); _muzzle= ((_x # 4) # 1); _magazine= ((_x # 4) # 3); _turretPath=((_x # 4) # 2); _unit= _veh turretUnit _turretPath;
			_ammo= (getText (configFile >> "CfgMagazines" >> _magazine >> "ammo"));
			if (("4" in (getText (configFile >> "CfgAmmo" >> _ammo >> "weaponLockSystem"))) or ((getNumber (configFile >> "CfgAmmo" >> _ammo >> "laserLock")) == 1) 
			or ((getText (configFile >> "CfgAmmo" >> _ammo >> "effectsMissile")) == "CruiseMissile") ) then 
			{
				////add event handler to point missile at the target
				_veh setVariable ["jac_hasEHTargetLaserDesignatorDoFire", true, true];
				_veh setVariable ["jac_myLaserTargetLogic", _ctrlOBJ, true];
				if !(_veh getVariable ["jac_hasEHTargetLaserDesignator", false]) then
				{
					_veh setVariable ["jac_hasEHTargetLaserDesignator", true, true];
					_EHLaser = _veh addEventHandler ["Fired", {
						params ["_unit", "_weapon", "_muzzle", "_mode", "_ammo", "_magazine", "_missile", "_gunner"];
						if ((vehicle _unit) getVariable ["jac_hasEHTargetLaserDesignatorDoFire", true]) then
						{
							if (isNull _missile) then {_missile = nearestObject [(vehicle _unit), _ammo]};
							_laserTargetList= ( attachedObjects ((vehicle _unit) getVariable ["jac_myLaserTargetLogic", objNull]));
							if (count _laserTargetList > 0) then 
							{
								_missileTarget= (_laserTargetList # 0);
								[_missile, _missileTarget] spawn 
								{
									params ['_missile', '_missileTarget'];
									sleep 0.1;
									_result= _missile setMissileTarget _missileTarget;
								};//spawn
							};
							(vehicle _unit) setVariable ["jac_hasEHTargetLaserDesignatorDoFire", false, true];
						};
					}];
				};
			
				_laserTarget = ((position _ctrlOBJ nearEntities [_targetType,10])) param [0,objnull];
				if (isNull _laserTarget) then {_laserTarget= createvehicle [_targetType, getPosATL _ctrlOBJ,[],0,"CAN_COLLIDE"]; _laserTarget attachTo [_ctrlOBJ]};
				_veh loadMagazine [_turretPath, _weapon, _magazine];
				_veh reveal _laserTarget; _veh dowatch _laserTarget; _veh dotarget _laserTarget;
				if (alive _veh) then {[300, _veh, "TARGETING LASER"] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0]};
				_ii= 0; waitUntil {sleep 0.001; _ii= (_ii+1); ( (_veh aimedAtTarget [_laserTarget] > 0) or (_ii >= 10)) };
				_unit fireattarget [_laserTarget, _muzzle]; break;
			};
		}forEach _ammoData;
		
		sleep 1; _ctrlOBJ setVariable ["jac_executingLaserFire", false];
	}forEach (_grpArray select { (!(isNull objectParent leader _x) && (local leader _x)) });
};
};



////// RESPAWN SYSTEM
// STRONPOINT - dug in respawn
jac_fnc_respawnUnitStrongpoint=
{
params ["_unit"];
_unit setVariable ["jac_hasRespawnUnitStrongpointEH",true, true];
	_unit addMPEventHandler ["MPKilled",
	{
		_unit= (_this # 0);
		if (local _unit) then 
		{
			if (_unit getVariable["jac_isPanicked", false]) then 
			{
				_unit switchMove (_unit getVariable["jac_isPanickedefaultAnim", '']);
				_unit playMoveNow (_unit getVariable["jac_isPanickedefaultAnim", '']);
				_unit setVariable["jac_isPanicked", false, true];
				_unit enableAI "ANIM";
			};
		[_unit] spawn {	params ["_unit"];
		
		_my_group= group _unit;
		if ( (_unit getVariable ['jac_isWRGunit', false]) && !(_unit getVariable ['jac_respawnListAdded', false]) ) then {[_unit, _my_group] call jac_fnc_respawnListAdd};		//is a WARGAME enabled unit that will automatically respawn from main base
		
		if (!(_my_group getVariable ["jac_isDugIn",false]) or !jac_wargameStrongpointRespawnSystemActive) exitWith {true};
		
		_my_id=_unit;
		_my_id_str= (str _my_id);
		_curatorP= ([_my_id] call jac_fnc_objectCurators);
		_my_type_is= (typeOf _unit);
		_my_loadout_is= (getUnitLoadout _unit);
		_mySupplyLoadout= (_unit getVariable ["jac_mySupplyLoadout", nil]);
		_my_stance_is= unitPos _unit;
		if (_my_stance_is == "AUTO") then {_my_stance_is= "UP"};
		_myPos = (_unit getVariable ["myDigInPos", (getPosATL _unit)]);
		_myformdir = formationDirection _unit;
		_mySkill = _unit getVariable ["jac_myDefaultSkill", 0.5];
		_myRank = rank _unit;
		_myFace= face _unit;
		_myName = name _unit;
		_myTurret = _unit getVariable ["jac_myTurretObject",objNull];
		_myObjLogic = _unit getVariable ["jac_myTurretObjectLogic",objNull];
		_myCoverObj = _unit getVariable ["jac_myCoverObject",objNull];
		_unitTraitList= [];
		_unitTraitList= (getAllUnitTraits _unit);
		
			sleep jac_strongpoint_respawnDefault;							
			
			/// check if respawn is still active
			waitUntil { sleep 2; 
			if !(_my_group getVariable ['strongpoint_active', true]) exitWith { true };		if !(_my_group getVariable ['strongpoint_blocked', false]) exitWith { true };	false};
			
			sleep 1;			
			
			//squad ran out of reinforcement points - set strongpoint_active = false
			_current_strength= _my_group getVariable ['strongpoint_health', jac_strongpoint_healthDefault];
			if ((_current_strength-1) < 0) exitWith {_my_group setVariable ['strongpoint_active', false]; jac_3D_msgList pushBack [400, (leader _my_group), "OUT OF REINFORCEMENTS!"]; [[_myCoverObj,_myTurret, _myObjLogic]] call jac_fnc_deleteObjs}; 	/// message
			_my_group setVariable ['strongpoint_health', (_current_strength-1), true];
			
			if !(_my_group getVariable ['strongpoint_active', true]) exitWith { [[_myCoverObj,_myTurret, _myObjLogic]] call jac_fnc_deleteObjs };	//if strongpoint_active == false, delete cover/turret objects

			/// create unit, set traits, go prone, teleport back to last position, return to last stance
			
			_spawn_unit= _my_group createUnit [_my_type_is, [1,1,1], [], 0, 'NONE'];
			sleep 1;
			
			//remove old unit data from respawn list
			[_my_id_str, _my_group] remoteExec ["jac_fnc_respawnListRemove", 0];
			
			//add unit to curator
			[
				[_spawn_unit, _curatorP], 
				{
					_unitToAdd= (_this # 0); _curatorP= (_this # 1);
					{_x addCuratorEditableObjects [[_unitToAdd],true]}forEach _curatorP;
				}
			] remoteExecCall ['call', 2];
			
			// MP compatibility code
			[
			[_spawn_unit, _my_id, _my_group, _myPos, _myCoverObj, _myTurret, _myObjLogic, _my_loadout_is, _mySupplyLoadout, _mySkill, _myRank, _myFace, _myName, _myformdir, _curatorP, _my_stance_is, _unitTraitList],  
			{
				params  ["_spawn_unit", "_my_id", "_my_group", "_myPos", "_myCoverObj", "_myTurret", "_myObjLogic", "_my_loadout_is", "_mySupplyLoadout", "_mySkill", "_myRank", "_myFace", "_myName", "_myformdir", "_curatorP", "_my_stance_is", "_unitTraitList"];

				deleteVehicle _my_id;
			
				_spawn_unit setVariable ["myDigInPos", _myPos];
				_spawn_unit setVariable ["jac_isDugIn", true, true];
				_spawn_unit setVariable ["jac_myCoverObject",_myCoverObj];
				_spawn_unit setVariable ["jac_myTurretObject",_myTurret];
				_spawn_unit setVariable ["jac_myTurretObjectLogic",_myObjLogic, true];
				_spawn_unit disableAI "PATH";
				_spawn_unit setUnitPos _my_stance_is;
				_spawn_unit setUnitLoadout _my_loadout_is;
				if !(isNil "_mySupplyLoadout") then {_spawn_unit setVariable ["jac_mySupplyLoadout", _mySupplyLoadout]};
				_spawn_unit setSkill _mySkill;
				_spawn_unit setRank _myRank;
				_spawn_unit setFace _myFace;
				_spawn_unit setName _myName;
				_spawn_unit setDir _myformdir;
				{_spawn_unit setUnitTrait _x} forEach _unitTraitList;
				[_spawn_unit] allowGetIn true;	
			
				//set to the correct position
				if (isNull _myTurret) then
				{
					[_spawn_unit, _myPos] spawn { params ["_spawn_unit", "_myPos"];
					for "_i" from 5 to 0 step -1 do {
					sleep 1;
					_spawn_unit setPosATL _myPos;
					}}; //spawn
				};
			
				//get inside turret if any				
				if (!(isNull _myTurret) and (alive _myTurret)) then 
				{
				[_myTurret, _spawn_unit, _my_group] spawn { params ["_myTurret", "_spawn_unit", "_my_group"];
					for "_i" from 10 to 0 step -1 do {
						_spawn_unit assignAsGunner _myTurret;						
						_spawn_unit moveInGunner _myTurret;
						sleep 1;
						if (gunner _myTurret == _spawn_unit) exitWith {true};
					};
					[_my_group, _spawn_unit] remoteExec ["selectLeader", 0];
				}}; //spawn
			}] remoteExecCall ["call", 0];
		};//spawn
		};//is local							
	}];
};



///////// dig in status
// while dug in, for allGroups, check if an enemy leader is within 25 meters of my leader, if yes, respawning is blocked
jac_fnc_digInStrongpointStatus=
{
_myGrp= (_this # 0);
if !(local _myGrp) exitWith {false};

[_myGrp] spawn { params ["_myGrp"];
while {_myGrp getVariable ["jac_isDugIn",false]} do
{
sleep 2;
_myGrpLeader= leader _myGrp;
_is_blocked= (_myGrp getVariable ["strongpoint_blocked",false]);
_is_active= (_myGrp getVariable ["strongpoint_active",true]);
_myGrpSide= side _myGrp;
_pointIsBlocked= false;

/// find an enemy leader close enough
{
 if ((units _x) findIf {alive _x} != -1) then 
 {
	_grpLeader= leader _x;
	_grpSide= side _x;

	if ((_grpSide != _myGrpSide) && ((_grpLeader distance2D _myGrpLeader) <= 25)) then 
	{
		if ([_myGrpSide, _grpSide] call BIS_fnc_sideIsEnemy) exitWith {_pointIsBlocked= true};
	};
 };
sleep 0.001;
}forEach allGroups;


//// status check
if (units _myGrp findIf {alive _x} == -1) then 
{
	/// squad position is overrun and destroyed
	if (_pointIsBlocked) exitWith
	{
		_myGrp setVariable ["jac_isDugIn",false, true];
		_myGrp setVariable ["strongpoint_blocked", false];
		_myGrp setVariable ['strongpoint_active', false];
	
		[350, _myGrpLeader, "SQUAD POSITION WAS OVERRUN!"] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0];
		_mySide= _myGrpLeader getVariable ['jac_mySide', civilian];
		[_myGrp, _mySide, objNull] call jac_fnc_squadRespawn;
	};

	/// squad ran out of reinforcements 
	if !(_is_active) exitWith
	{
		_myGrp setVariable ["jac_isDugIn",false, true];
		_myGrp setVariable ["strongpoint_blocked", false];
	
		_mySide= _myGrpLeader getVariable ['jac_mySide', civilian];
		[_myGrp, _mySide, objNull] call jac_fnc_squadRespawn;
	};
};


/// temporarily blocked
if (_pointIsBlocked) then
{
	if ((!_is_blocked) && (_is_active)) then 
	{
		_myGrp setVariable ["strongpoint_blocked", true];
		[300, _myGrpLeader, "REINFORCEMENTS DISABLED!"] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0];
	sleep 3;
	};
}
else 	/// temporarily unblocked
{
	if ((_is_blocked) && (_is_active))  then 
	{
		_myGrp setVariable ["strongpoint_blocked", false];
		[300, _myGrpLeader, "REINFORCEMENTS ENABLED"] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0];
	sleep 3;
	};
};

}; //while
}; //spawn
};



////// main base respawn
//when an unit dies, add its data to the group's jac_myRespawnlist
//repawn all units from the list when the group is destroyed or redeploys at main base
//if the unit respawns because of a dug in position, remove data from the list and do nothing

//add data
jac_fnc_respawnListAdd=
{
params ["_unit", "_grp"];
_leaderGrp= (leader _grp);
if ( (_leaderGrp getVariable ['jac_isWRGunit', false]) or (jac_enableWRGfeatureAll) ) then
{
_unit setVariable ['jac_respawnListAdded', true, true]; //avoid duplicating
_mySide= _unit getVariable ['jac_mySide', civilian];

_respawnArray= [];
_respawnArray pushBack (getAllUnitTraits _unit); 	//0 = array with traits

_respawnArray pushBack (_unit);						//1 
_respawnArray pushBack (typeOf _unit);				//2
_respawnArray pushBack (getUnitLoadout _unit);		//3
_respawnArray pushBack (_unit getVariable ["jac_myDefaultSkill", 0.5]);				//4
_respawnArray pushBack (rank _unit);				//5
_respawnArray pushBack (face _unit);				//6
_respawnArray pushBack (name _unit);				//7
_respawnArray pushBack ([_unit] call jac_fnc_objectCurators);		//8 = list of curators that can control this unit
_respawnArray pushBack _mySide;						//9
_respawnArray pushBack (_unit getVariable ["jac_mySupplyLoadout", nil]);    //10
_respawnArray pushBack (str _unit);					//11 = unique id for data removal

//append respawn list
_respawnList= [];
_respawnList= _grp getVariable ['jac_myRespawnlist', []];

_respawnList pushBack _respawnArray;
_grp setVariable ['jac_myRespawnlist', _respawnList, true];

//check if group is all dead, initiate count to respawn
	if (units _grp findIf {alive _x} == -1) then 
	{
		[_unit, _grp, _mySide] spawn {	params ["_unit", "_grp", "_mySide"];
			sleep jac_strongpoint_respawnDefault;
			//check again and call respawn function
			if ((units _grp findIf {alive _x} == -1) && !(_grp getVariable ["jac_isDugIn",false])) then
			{			
				[_grp, _mySide, objNull] call jac_fnc_squadRespawn;
			};
		};
	};
};//only for WRG units
};

//remove data
jac_fnc_respawnListRemove=
{
params ["_unit", "_grp"]; 		//use _unit id as a string
_respawnList= [];
_respawnList= _grp getVariable ['jac_myRespawnlist', []];
_index= _respawnList findIf {(_x # 11) isEqualTo _unit};

 if (_index != -1) then
 {
	_respawnList deleteAt _index;
	_grp setVariable ['jac_myRespawnlist', _respawnList, true];
 };
};

////respawn functions
//called when group is destroyed or calls reinforcements from a main base or resupply point
//
// arg '_supplyBase' // isNull = function called from a main base, any other value is == the object where reinforcements are called from in case it's not a main base
//
jac_fnc_squadRespawn= 
{
params ["_grp", "_mySide", "_supplyBase"]; 
if !(local _grp) exitWith {false};

if (_grp getVariable ["jac_grpIsRespawning",false]) exitWith {false}; 
_grp setVariable ["jac_grpIsRespawning",true, true]; 

 [_grp, _mySide, _supplyBase] spawn { params ["_grp", "_mySide", "_supplyBase"]; 
 sleep 1; 
  
 /// get base object 
 _myBase= objNull;
 if (isNull _supplyBase) then 
 {
	_myBase= jac_mainBaseList getOrDefault [_mySide, objNull];
 }
 else
 {
	_myBase= _supplyBase;	//when respawning from a point other than a main base
 };
  if (isNull _myBase) exitWith {_grp setVariable ["jac_grpIsRespawning",false, true]}; //exits if no respawn base was found
  
 _respawnArray= []; 
 _respawnArray= _grp getVariable ['jac_myRespawnlist', []]; 
  { 
   _respawnArrayIndex= _x; 
   _unitTraitList= []; 
   _unitTraitList= (_respawnArrayIndex # 0);    
   _my_type_is=  (_respawnArrayIndex # 2);       
   _unit= _grp createUnit [_my_type_is, [1,1,1], [], 0, 'NONE']; 
   sleep 1; 
    
   //add unit to curator    
   _curatorP= (_respawnArrayIndex # 8); 
   [ 
   [_unit, _curatorP],  
   {  
    _unitToAdd= (_this # 0); 
    _curatorP= (_this # 1); 
    {_x addCuratorEditableObjects [[_unitToAdd],true]}forEach _curatorP; 
   }] remoteExecCall ['call', 2]; 
       
   {_unit setUnitTrait _x} forEach _unitTraitList; 
   _unit setUnitLoadout (_respawnArrayIndex # 3); 
   _mySupplyLoadout= (_respawnArrayIndex # 10); 
   if !(isNil "_mySupplyLoadout") then {_unit setVariable ["jac_mySupplyLoadout", _mySupplyLoadout], true}; 
   _unit setSkill (_respawnArrayIndex # 4); 
   _unit setRank (_respawnArrayIndex # 5); 
   _unit setFace (_respawnArrayIndex # 6); 
   _unit setName (_respawnArrayIndex # 7); 
       
   [_unit, (getPosATL _myBase), false, true] spawn jac_fnc_redeployDrop;
     
   _myID= (_respawnArrayIndex # 1); 
   deleteVehicle _myID; //delete old body 
  }forEach _respawnArray; 
 sleep 1; 
 _grp setVariable ['jac_myRespawnlist', [], true]; //clean respawn list 
 _grp setVariable ["jac_grpIsRespawning",false, true]; 
 sleep 1; 
 (units _grp) joinSilent _grp; // return to formation
 }; //spawn 
};


///////// CUSTOM MOVE WAYPOINT
//resolve movement
//sort which kind of move action should be used based on unit type and position to move - infantry, vehicle reverse
jac_fnc_moveGroup=
{
	params ["_movePos", "_group", "_clientID", '_ctrl', '_shift', '_jac_requireShiftToUseTakeCoveraction', '_myGroupCoverList', '_doubleRightClick'];
	if ( !(local _group) or (isPlayer (leader _group)) ) exitWith {false};	//doesn't work on players

	_leader= (leader _group); _vehLeader= objectParent _leader;
	_isInf = (isNull _vehLeader);		//ON FOOT

	if !(_isInf) then	 //if is a vehicle
	{
		//if adding as a new waypoint or replacing current wps, reset some group behaviour
		_WPPosCurrent= getWPPos [_group, (currentWaypoint _group)];
		if (!(_WPPosCurrent isEqualTo [0,0,0]) && !(_ctrl)) then
		{
			//if not in scripted move, doFollow leader
			if ( ((driver _vehLeader) getVariable ['jac_endScriptedMove', false]) or !(isNull (_vehLeader getVariable ['jac_myConvoyParent', objNull])) or !(isNull (_vehLeader getVariable ['jac_myConvoyChild', objNull])) ) then 
			{_vehLeader doFollow _vehLeader};

			_vehLeader setVariable ['jac_myConvoyParent', objNull, true];	//cancel convoy following
			_vehLeader setVariable ["jac_isReversing", false, true];   //cancel reverse move
		};
		
		/// REVERSE MOVE
		if ( (_vehLeader isKindOf "LandVehicle") && !(_vehLeader isKindOf "StaticWeapon") ) then {[_leader, _vehLeader, _movePos, _clientID, _group] call jac_fnc_vehicleMove};
	};

	//// infantry move - not in dug in mode
	if ((_isInf) && !(_group getVariable ["jac_isDugIn",false])) then {[_movePos, _group, _clientID, _ctrl, _shift, _jac_requireShiftToUseTakeCoveraction, _myGroupCoverList] call jac_fnc_infantryMove};
};

// vehicle reverse move
jac_fnc_vehicleMove=
{
	params ["_leader", "_vehLeader", "_movePos", "_clientID", "_group"];
	_doReverse= true;
	_reverseDisMax= jac_reverseRange;
	_vehSpeed = (speed _vehLeader);
	_vehLeaderPos= getPosASL _leader;
	_vehSize= (sizeOf (typeOf _vehLeader)) * 0.25;
	
	if (fuel _vehLeader == 0) then {[225, _vehLeader, "OUT OF FUEL"] remoteExecCall ["jac_fnc_zeusAddMSG3D", _clientID]; _doReverse= false};
	if !(alive (driver _vehLeader)) then {[225, _vehLeader, "NO DRIVER"] remoteExecCall ["jac_fnc_zeusAddMSG3D", _clientID]; _doReverse= false};
	if ((_vehLeader getHitPointDamage "hitEngine") >= 0.9) then {[225, _vehLeader, "ENGINE DAMAGED"] remoteExecCall ["jac_fnc_zeusAddMSG3D", _clientID]; _doReverse= false};
	
	/// reverse move
	//determine if the player is clicking behind the vehicle (within X angle and within X distance)
	_disToReverse= (_movePos distance _vehLeaderPos);
	if ((_vehSpeed <= 20) && (_disToReverse <= _reverseDisMax) && !(_vehLeader getVariable ["jac_isReversing", false])) then
	{
		_unitDirection= (((getDir _vehLeader) + 180) mod 360); _targetDirection= (_vehLeader getDir _movePos);
		if !([_unitDirection, _targetDirection, 32.5] call jac_targetDirInFOV) then {_doReverse = false};		// Check if difference is within the acceptable angle range

		if (_doReverse) then
		{
			[200, _vehLeader, "REVERSE!"] remoteExecCall ["jac_fnc_zeusAddMSG3D", _clientID];				
			(driver _vehLeader) disableAI "MOVE"; _vehLeader disableBrakes true;

			//find position to move to
			_reversePos= _movePos;	//the position to reverse to

			/// delete current waypoints
			for "_i" from count waypoints _group - 1 to 0 step -1 do {deleteWaypoint [_group, _i]};

			_vehLeader setPosATL (getPosATL _vehLeader vectorAdd [0,0,0.1]); _vehLeader setDir ((_targetDirection + 180) mod 360); _vehLeader setVectorUp surfaceNormal position _vehLeader;	//turns to align with the reverse direction
			_vehLeader setVelocity [0,0,0];
			doStop _leader; _leader doFollow _leader;
			_vehLeader engineOn true;

			////apply velocity until reaches the position or certain amount of time has passed, to prevent weird behaviours
			_vehLeader setVariable ["jac_isReversing", true, true];
			_speed = -1.2;
			_reverseTimeout= (CBA_missionTime + (_disToReverse + 10));

			_PARAMS = [_leader, _reversePos, _vehSize, _vehLeader, _speed, _reverseDisMax, _reverseTimeout];
			[{
				(_this # 0) params ["_leader", "_reversePos", "_vehSize", "_vehLeader", "_speed", "_reverseDisMax", "_reverseTimeout"];

				_vehLeaderPos= getPosASL _leader;
				_targetDistance= ((_vehLeaderPos distance (_reversePos)) - _vehSize);	//get the actual distance
					
				_vel = velocity _vehLeader;
				_dir = direction _vehLeader;

				// increase speed up to a certain point
				if (abs(speed _vehLeader) <= 14) then
				{
					_vehLeader setVelocity [(_vel select 0) + (sin _dir * _speed), (_vel select 1) + (cos _dir * _speed), (_vel select 2)];
					if (abs _speed < 2.5) then {_speed = _speed*1.2};	//increase speed
				};

				//until reaches target position or has gone too far or for too long, avoid errors
				if ( !(alive _vehLeader) or (_targetDistance <= 0) or (_targetDistance > _reverseDisMax) or (CBA_missionTime > _reverseTimeout) or !(_vehLeader getVariable ['jac_endScriptedMove', true]) or !(_vehLeader getVariable ['jac_isReversing', true]) ) then
				{
					_vehLeader setVariable ["jac_isReversing", false, true];
					(driver _vehLeader) enableAI "MOVE"; _vehLeader disableBrakes false;
					
					//if !((currentCommand _vehLeader) isEqualTo "STOP") then {doStop _leader};
					
					if ( ((driver _vehLeader) getVariable ['jac_endScriptedMove', false]) or !(isNull (_vehLeader getVariable ['jac_myConvoyParent', objNull])) or !(isNull (_vehLeader getVariable ['jac_myConvoyChild', objNull])) ) then 
					{doStop _leader; _vehLeader doFollow _vehLeader};
					
					[(_this # 1)] call CBA_fnc_removePerFrameHandler;
				};
			} , 0.1, _PARAMS] call CBA_fnc_addPerFrameHandler;
		};
	}
	else
	{_doReverse= false};
};

// infantry move
jac_fnc_infantryMove=
{
	params ["_movePos", "_group", "_clientID", '_ctrl', '_shift', '_jac_requireShiftToUseTakeCoveraction', '_myGroupCoverList'];
	
	_isCoverMove= false;
	_canQuickMove= jac_improvedPathfinder;
	_wpTargetID= (count(waypoints _group) - 1); _wpTarget= [_group, _wpTargetID];
	
	//if adding as a new waypoint or replacing current wps, reset group behaviour	
	_WPPosCurrent= getWPPos [_group, (currentWaypoint _group)];	
	if (!(_WPPosCurrent isEqualTo [0,0,0]) && !(_ctrl)) then
	{
		(units _group) doFollow (leader _group);
		{
			_x setVariable ['jac_doMoveCoverPos', [], true];
			if (!(_group getVariable ['jac_isDugIn', false]) && (_x getVariable ['jac_isInCover', false]) ) then {_x enableAi 'PATH'};	//enable path if in cover, but not dug in
			_moveToMarker= _x getVariable ['jac_moveToMarker', objNull]; deleteVehicle _moveToMarker;		
			_x setVariable ['jac_isMovingToCoverPos', false, true];
			_x setVariable ['jac_pathCalculated', false];
			_x setVariable ['jac_endMovingToCoverPos', true, true];
			_x setVariable ['jac_endScriptedMove', true, true];
			_x setVariable ["jac_isClearingBuilding", false, true];
		}forEach (units _group);
		_group setVariable ['jac_doFormUp', false, true];	//cancel stacking behind vehicle
	};

	if ((_WPPosCurrent isEqualTo [0,0,0]) && !(_ctrl)) then {_wpTarget= _group addWaypoint [_movePos, -1]};

	if (count _myGroupCoverList > 0) then
	{
		if ( !(_jac_requireShiftToUseTakeCoveraction) or (_jac_requireShiftToUseTakeCoveraction && _shift) ) then
		{
			_wpTarget setWaypointType "SCRIPTED";
			_wpTarget setWaypointStatements [wp_InfMoveCond, str _clientID];	//client id is stored as string in the execution statement...
			_wpTarget setWaypointName 'WP Move To Cover';
			_wpTarget setWaypointDescription (str _myGroupCoverList);
			
			///clear cover list
			//sleep 1;
			_group setVariable ["jac_myGroupCoverList", [], true];
			_canQuickMove= false;
			_isCoverMove= true;
		};
	};
};


wp_InfMoveCond=
"
if !(local this) exitWith {false};
[this] call jac_fnc_wp_InfMoveCond;
true;
";

//should always be scheduled
jac_fnc_wp_InfMoveCond=
{
params ['_thisWP'];
_my_leader= _thisWP;
if ((_my_leader getVariable["jac_isPanicked", false]) or (_my_leader getVariable["jac_isPOW", false])) exitWith {false};	//panicked units can't move

scopeName 'main';
_group=(group _my_leader);
_wp= (currentWaypoint _group);

if !(waypointName [_group, _wp] isEqualTo 'WP Move Activated') then
{
	[_group, _wp] setWaypointName 'WP Move Activated';
	_clientID= parseNumber (str ((waypointStatements [_group, _wp]) # 1));
	[200, leader _group, 'TAKE COVER!'] remoteExecCall ["jac_fnc_zeusAddMSG3D", _clientID];
	leader _group playActionNow 'GestureGo';
	
	_unitList= (units _group); _arrayString=  (waypointDescription [_group, _wp]);
	deleteWaypoint [_group,_wp]; doStop _unitList; //sleep 0.2;
	if ((typeName _arrayString) == 'STRING') then
	{
		if (count _arrayString > 2) then
		{
			_myGroupCoverList= parseSimpleArray _arrayString;
			_smallerListSize= ((count _myGroupCoverList) min (count _unitList)) -1;
	
			//add move to cover action to each unit
			for '_i' from 0 to _smallerListSize do
			{
				if (_i >= (count _myGroupCoverList)) then {breakTo 'main'};
				_unit = (_unitList # _i);
				_moveToPos= ((_myGroupCoverList # _i) # 0);
				_coverDir= (_myGroupCoverList # _i) # 2;
				_coverHeight= (_myGroupCoverList # _i) # 3;
				
				_unit setVariable ['jac_isMovingToCoverPos', true, true];
				[_unit, _moveToPos, _coverDir, _coverHeight] spawn jac_fnc_UnitDoTakeCover;
			};
		};
	};
};
};

////// COVER SYSTEM FUNCTIONS
/// move to cover and apply effects
jac_fnc_UnitDoTakeCover=	//must be executed scheduled
{
params ['_unit', '_moveToPos', '_coverDir', '_coverHeight'];

	_unit setVariable ['jac_doMoveCoverPos', (ASLToATL (_moveToPos)), true]; _isDoMove= false;
	
	_moveToMarker = createVehicle ['Sign_Arrow_Green_F', [0,0,0], [], 0, "CAN_COLLIDE"];
	[_moveToMarker, true] remoteExecCall ["hideObjectGlobal", 2];
	
	_moveToMarker setPosATL ((ASLToATL (_moveToPos)) vectorAdd [0,0,0.25]);
	_moveToMarker setVectorUp surfaceNormal (position _moveToMarker);
	_unit setVariable ['jac_moveToMarker', _moveToMarker, true];
	
	//for all zeus that can control the group, unhide the marker
	[{
		params ["_moveToMarker", "_unit"];
		[[_moveToMarker, _unit],
		{
			params ["_moveToMarker", "_unit"];
			_zeusList=[]; {_zeusList pushBack (owner _x)}forEach ( (allCurators) select {( _unit in (curatorEditableObjects _x) )} );
			[[_moveToMarker],
			{
				params ["_moveToMarker"];
				if (!(isNull findDisplay 312) && (jac_doDraw3Dmarker)) then {_moveToMarker hideObject false};	//unhide marker for me
			}] remoteExecCall ['call', _zeusList];
		}] remoteExecCall ['call', 2];
	}, [_moveToMarker, _unit], 0.25] call CBA_fnc_waitAndExecute;

	_unitPosASL= getPosASL _unit;
	if ( ( ((getPosATL _unit) distance (ASLToATL _moveToPos)) < 300 ) && ([_unitPosASL,  _moveToPos] call jac_fnc_linePath) ) then
	{
		_path= [[(ASLToAGL _moveToPos), false, '', 0], [(ASLToAGL _moveToPos), false, '', 0]];
		[_unit, _path, jac_scriptedMoveSpacing, jac_scriptedMoveTurnSpd, false] remoteExecCall ["jac_fnc_scriptedMove", 0];
	}
	else 
	{_unit doMove (ASLToATL (_moveToPos)); _isDoMove= true};
	
	_unit setVariable ['jac_endMovingToCoverPos', false, true];
	
	_PARAMS= [_unit, _moveToPos, _coverDir, _coverHeight, _moveToMarker, _isDoMove];
	[{
	params ['_unit', '_moveToPos', '_coverDir', '_coverHeight', '_moveToMarker', '_isDoMove'];
	( !(alive _unit) or ((getPosATL _unit) distance (ASLToATL _moveToPos) < 2.5) or !(_unit getVariable ['jac_isMovingToCoverPos', false]) or (_unit getVariable ['jac_endMovingToCoverPos', false]) )}, 
	{
		params ['_unit', '_moveToPos', '_coverDir', '_coverHeight', '_moveToMarker', '_isDoMove'];
	
		deleteVehicle _moveToMarker;
	
		// when any unit reaches the destination, it will check if others in the group have done the same; then teleport the ones who are taking too long
		if (((getPosATL _unit) distance (ASLToATL _moveToPos)) <= 2.5) then
		{
			if (_coverHeight == 2) then {_unit setUnitPos 'UP'} else {_unit setUnitPos 'MIDDLE'};	//adjust stance to cover height
			
			// if not moving directly to cover using -scripted move- will call the function to set and monitor the cover status
			if (_isDoMove) then {_unit setDir (_unit getDir _moveToPos); _unit setPosATL (ASLToATL _moveToPos); doStop _unit; [_unit, _moveToPos, _coverDir] call jac_scriptedMoveTakeCover};
			
			// turn the formation towards the cover direction
			if ((leader _unit) == _unit) then {(group _unit) setFormDir _coverDir};
			
			// teleport stragglers to their cover position
			{
				_moveToPosOG= _x getVariable ['jac_doMoveCoverPos', []];
				[{
					params ['_moveToPosOG', '_unit'];
					_moveToPos= _unit getVariable ['jac_doMoveCoverPos', []];
					if ((_moveToPosOG isEqualTo _moveToPos) && ((count _moveToPos) > 1) && (alive _unit)) then 
					{
						if ( (((getPosATL _unit) distance (_moveToPos)) > 2.25) && (((getPosATL _unit) distance (_moveToPos)) < 500) ) then {_unit setDir (_unit getDir _moveToPos); _unit setPosATL (_moveToPos); doStop _unit};
					};
				}, [_moveToPosOG, _x], 5] call CBA_fnc_waitAndExecute;
			}forEach (units (group _unit));
		};
	}, _PARAMS] call CBA_fnc_waitUntilAndExecute;
};

//// set the in cover status
jac_scriptedMoveTakeCover=
{
params ['_unit', '_moveToPos', '_coverDir'];

	_unit setVariable ['jac_isMovingToCoverPos', false, true]; _unit setVariable ['jac_pathCalculated', false];
	_unit setVariable ['jac_isInCover', true, true]; _unit setVariable ['jac_myCoverDir', _coverDir, true];
	_unit setVariable ['jac_endScriptedMove', true, true];
	
	doStop _unit; _unit doWatch objNull;
	
	if (jac_AICoverSkillBonus) then {_unit disableAI 'AIMINGERROR'; _unit setVariable ['jac_AimBonus', true, true]};
	[200, _unit, "in cover +"] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0];
	
	// waitUntil to disable cover status
	[{
		params ['_unit', '_moveToPos', '_coverDir'];
		( !(alive _unit) or (((getPosATL _unit) distance (ASLToATL _moveToPos)) > 2.25) )},
	{
		params ['_unit', '_moveToPos', '_coverDir'];
		_unit setVariable ['jac_isInCover', false, true];	
		_unit enableAI 'AIMINGERROR'; _unit setVariable ['jac_AimBonus', false, true]; [200, _unit, "left cover -"] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0];	
	}, [_unit, _moveToPos, _coverDir]] call CBA_fnc_waitUntilAndExecute;	
};



jac_drawDebugLineArray3D=[];	//save data used to draw/visualize the cover search system
jac_coverSearchDis=4;	//
jac_coverSpacing= 1.5;	//press ALT to change spacing, increasing up to the max value then cycling back to the smallest interval

//// find cover positions for the selected group
//from the mouse position shoot a line into 8 directions, 3 meters size
//find the first object that can be used as cover, the direction from the center will determine the formation direction and center position
//from this center point check around for more cover positions for the rest of the group, leftovers will move together with teammates
//check in a rectangular pattern starting from the formation center position, the new cover positions be facing the same direction as the _formationCenterPosDir

//move soldiers to positions, after reaching a position, setPos to be more accurate and set stance
//max number of positions is jac_coverMaxPos, if leader reaches his position, after some time teleport stragglers to their positions

jac_fnc_findCoverFormation=
{
params ["_centerPos", "_group"];
	scopeName "main";
	_myGroupCoverList= [];
	_myFunctionId= CBA_missionTime; jac_currentFindCoverFunction= _myFunctionId;
	
	//find the formation center position and direction
	_heading= [0,45,90,135,180,225,270,315]; //_heading is used to have control over the direction cover will be searched
	_formationCenterPosDir= [_centerPos, jac_coverSearchDis, _heading] call jac_fnc_findCoverFromPos; //returns array with format # 0 = position; # 1 = distance to cover object; # 2 = direction; # 3 = cover height (1= crouch; 2= stand)
	
	if (count _formationCenterPosDir > 0) then 
	{
		///add the first position
		_myGroupCoverList= _group getVariable ["jac_myGroupCoverList", []];
		_myGroupCoverList pushBack _formationCenterPosDir;
		
		_groupSize= count (units _group);
		_posCount= (count _myGroupCoverList);
		if ((_posCount >= jac_coverMaxPos) or (_posCount >= _groupSize)) exitWith {breakTo "main"}; //if only has 1 unit in the group, exit
		
		//check in a rectangular pattern starting from the formation center position, the new cover positions should be facing the same direction as the _formationCenterPosDir
		_angle= (_formationCenterPosDir # 2);	   //formation cover direction
		_heading= [_angle];
		
		//layers to check - in a line pattern, going back and forth, forward and backward increasing the distance from the formation center
		for "_i" from 0 to 1 do
		{
			_verticalDis= _i * (jac_coverSearchDis); //to change the layer to check
			_verticalHeading= [_angle, ((_angle + 180) mod 360)];
			{
				_centerPosVertical= AGLToASL (_centerPos getPos [_verticalDis, _x]);
				_centerPosVertical set [2, (_centerPos # 2)];	//adjust height
				
				//positions to check for each horizontal row - in a line pattern, going back and forth, left and right increasing the distance from the formation center
				for "_ii" from 1 to 25 do
				{
					_horizontalDis= _ii * (0.75);	//horizontal spacing between searches - 0.5
					_horizontalHeading= [((_angle + 90) mod 360), ((_angle + 270) mod 360)];	//left and right in relation to formation cover direction
					{
						_centerCheckPos= AGLToASL (_centerPosVertical getPos [_horizontalDis, _x]);
						_centerCheckPos set [2, (_centerPosVertical # 2)];	//adjust height						
						
						if (_myGroupCoverList findIf {(_centerCheckPos) distance (_x # 0) < jac_coverSpacing} == -1) then	//check spacing from nearby positions
						{
							_newCoverCenterPosDir= [_centerCheckPos, jac_coverSearchDis, _heading] call jac_fnc_findCoverFromPos;
							if (count _newCoverCenterPosDir > 0) then 
							{
								//save the new cover pos data until there are enough for every unit or the max limit is reached 								
								_myGroupCoverList pushBack _newCoverCenterPosDir;
							};
						};
						
						//if reached max positions exit and update the list for every local
						_posCount= (count _myGroupCoverList);
						if ( (_posCount >= _groupSize) or (_posCount >= jac_coverMaxPos) or (jac_currentFindCoverFunction != _myFunctionId) ) exitWith {breakTo "main"};
						//sleep 0.0001;
					}forEach _horizontalHeading;
				};
			}forEach _verticalHeading;
		};
	};

	_group setVariable ["jac_myGroupCoverList", _myGroupCoverList];
	jac_canUpdateGroupCoverPos= true;
	
	///handle cover position markers
	{_x hideObject true; _x setVehiclePosition [[0,0,0], [], 0, "CAN_COLLIDE"]}forEach jac_3DMarkerCoverPosition; jac_3DMarkerCoverPosition= [];	//reset cover position markers
	{
		if (_forEachIndex < (count jac_3DMarkerCoverPositionModelsArray)) then 
		{
			jac_mouseIsNearCover= true;
			_coverAnimation= 'amovpknlmstpsraswrfldnon'; if ((_x # 3) > 1) then {_coverAnimation= 'amovpercmstpsraswrfldnon'};
			_agent = jac_3DMarkerCoverPositionModelsArray # _forEachIndex;
			_agent switchMove _coverAnimation; _agent setDir (_x # 2);
			_agent setVehiclePosition [((_x # 0) vectorAdd [0,0,0.2]), [], 0, "CAN_COLLIDE"]; jac_3DMarkerCoverPosition pushBack _agent;
			_agent hideObject false;
		};
	}forEach _myGroupCoverList;
};

////look for cover around the given center position, returns array with format # 0 = cover position; # 1 = distance to cover object; # 2 = direction to cover; # 3 = cover height (1= crouch; 2= stand)

jac_fnc_findCoverFromPos=
{
params ["_centerPos", "_distance", "_heading"];
if (count _centerPos < 3) exitWith {false};

_collisionPosData= [];
_modHeight= 0.5;
_centerPos= _centerPos vectorAdd [0,0,_modHeight];

{
scopeName "forEachHeading"; 
_direction= _x;
_lineColor= [0,0,1,1];
_checkPos= AGLToASL (_centerPos getPos [_distance, _direction]);	//_heading is used to have control over the direction cover will be searched, default _distance = 3
_checkPos set [2, (_centerPos # 2)];

_collisionLineArray= (lineIntersectsSurfaces [_centerPos, _checkPos, jac_3DMousePositionMarker, objNull, true, 1, "VIEW", "NONE", true]);
_finalPos= _checkPos;

//jac_drawDebugLineArray3D pushBack [_centerPos, _finalPos, [0,0,1,1]];	////blue line no collision

	if (count _collisionLineArray > 0) then
	{
		_collisionLine= (_collisionLineArray # 0);
		if (true)	then //not terrain	//!((_collisionLine # 2) isEqualTo objNull)	//this is here for debugging using terrain as cover or not
		{
			scopeName "collisionCheck"; 
			_collisionPosOG = (_collisionLine # 0);	//the actual collision pos with cover object
			_collisionPosMod= AGLToASL (_collisionPosOG getPos [1.75, ((_direction + 180) mod 360)]);	//the cover position will be a bit further back from the actual collision pos to properly place the unit behind cover
			_collisionPosMod set [2, ((_collisionPosOG # 2) - _modHeight)];	//set the correct height
			_collisionPos= _collisionPosMod;
			
			_collisionDistance = _collisionPos distance _centerPos;
			if ((_collisionPosOG distance _centerPos) <= 0.5) then {breakTo "forEachHeading"};

			//jac_drawDebugLineArray3D set [(count jac_drawDebugLineArray3D)-1, [_centerPos, _collisionPosOG, [1,0,0,1]]];	////red line has collision
			
			//validate cover position checking vision height to set stance accordingly (stand or crouch) and if there's ground to stand on
			_hasGround= false;
			_isBlocked= false;			
			if (count ((lineIntersectsSurfaces [_collisionPos, (_collisionPos vectorAdd [0,0, -0.5]), jac_3DMousePositionMarker, objNull, true, 1, "VIEW", "NONE", true]) # 0) > 0) then {_hasGround= true}; //check for ground to stand on
			
			/// only check for los if position is valid, _hasGround
			_losHeight= 1;	_losHeightLastBlocked= 1;
			if (_hasGround) then
			{
				_losCheckPos= _checkPos;	
				//when using terrain as cover, extend the los check to prevent using the slope of a hill
				if ((_collisionLine # 2) isEqualTo objNull) then {_newCheckPos= AGLToASL (_centerPos getPos [22, _direction]); _newCheckPos set [2, (_centerPos # 2)]; _losCheckPos= _newCheckPos};
				
				_losHeightArray= [0.4, 0.7, 0.9, 1.5]; //all the height points to check - prone= 50cm, crouch= 80cm , standing= 145cm
				{
					_losHeight= _x;
					_losCollisionLineArray= (lineIntersectsSurfaces [(_centerPos vectorAdd [0,0,(_losHeight-_modHeight)]), (_losCheckPos vectorAdd [0,0,(_losHeight-_modHeight)]), jac_3DMousePositionMarker, objNull, true, 1, "VIEW", "NONE", true]);
					_losCollisionLine= (_losCollisionLineArray # 0);
					//jac_drawDebugLineArray3D pushBack [(_centerPos vectorAdd [0,0,(_losHeight-_modHeight)]), (_losCheckPos vectorAdd [0,0,(_losHeight-_modHeight)]), [0,1,1,1]];
					if ((count _losCollisionLineArray) > 0) then {_isBlocked= true; _losHeightLastBlocked=_losHeight; continue};  //los is obstructed
					_isBlocked= false;		//los is clear, continue code to save this pos data
				}forEach _losHeightArray;
			};
			_losHeight= ceil (_losHeightLastBlocked + 0.2);	////1= crouch; 2= stand
			
			if ((_isBlocked) or !(_hasGround)) then {continue};	//los is obstructed or no ground, go to next pos to check

			//save the collision pos, distance, direction and cover height, keep the collision pos that is the closest to the mousePos
			if (count _collisionPosData == 0) then 
			{
				_collisionPosData= [_collisionPos, _collisionDistance, _x, _losHeight];
				//jac_drawDebugLineArray3D set [(count jac_drawDebugLineArray3D)-1, [_centerPos, _collisionPosOG, [0,1,0,1]]];	////set the closest collision to green
			}
			else
			{
				if (_collisionDistance <= (_collisionPosData # 1)) then
				{
					_collisionPosData= [_collisionPos, _collisionDistance, _x, _losHeight];
					//{if ((_x # 2) isEqualTo [0,1,0,1]) then {jac_drawDebugLineArray3D set [_forEachIndex, [(_x # 0), (_x # 1), [1,0,0,1]]]}} forEach jac_drawDebugLineArray3D; //set previous green collisions to red
					//jac_drawDebugLineArray3D set [(count jac_drawDebugLineArray3D)-1, [_centerPos, _collisionPosOG, [0,1,0,1]]];	////set the new closest collision to green
				};
			};
		};
	};
} forEach _heading;	//array with the directions to check

//hint str _collisionPosData;
_collisionPosData;
};


////PATH DRAWING

//press TAB to enter path drawing mode
//create a logic object in front of every curator controlled unit
//grab the logic object with the mouse and move it around to draw path

///main routine
jac_fnc_pathDrawingMode=
{
	jac_isPathDrawingActive= true; jac_canClosePathDrawing= false;
	titleText ["DRAW A MOVEMENT PATH BY DRAGGING THE ARROW ASSOCIATED TO A UNIT", "PLAIN"];
		
	////add logic object that will be grabbed by the player to draw path
	_curator= (getAssignedCuratorLogic player); _objectListFull= (curatorEditableObjects _curator); _objectList= [];
	if (jac_pathPlanningOnlyForControled) then {_objectListFull= jac_unitControlCheck};	//if true planning mode should only spawn handles for the selected units and vehicles, not every existing unit. this behavior can be toggled with an addon setting.

	if ( (count _objectListFull) > 0 ) then
	{
		jac_pathColorList= [[0,0,0,0]];	//reset list of colors picked - add neutral color so it isn't picked
		
		//populate array with all zeus controlled soldiers and vehicle drivers
		_objectList= _objectListFull select { ( ( (alive _x) && ((vehicle _x) isKindOf 'CAManBase') ) or ( (alive _x) && ((vehicle _x) isKindOf 'LandVehicle' ) && (_x == (driver (vehicle _x))) ) ) };
		if ( (count _objectList) > 0 ) then	{jac_pathLogicList= []; [_objectList, true] call jac_fnc_pathLogicCreate}; ////each unit will have logic object associated with it for path drawing
	};
	
	if ( (count _objectList) == 0 ) exitWith {jac_isPathDrawingActive= false; jac_canClosePathDrawing= true; terminate _thisScript};
		
	//// visual effects	
	jac_pathDrawingModeEffect = ppEffectCreate ["colorCorrections", 1501];
	jac_pathDrawingModeEffect ppEffectEnable true;
	jac_pathDrawingModeEffect ppEffectAdjust [0.85, 1.0, 0, [0, 0, 0, 0] , [1, 1, 1, 1] , [0.299, 0.587, 0.114, 0]];
	jac_pathDrawingModeEffect ppEffectCommit 0;
	_msg= format ["-Hold CTRL and left click to delete a portion of the path.\n
		-Delete the arrow with the DELETE key to reset the path.\n \n
		-For infantry: hold the ALT key to move in any direction while facing towards the arrow direction.\n
		-Hold the SHIFT key to rotate the arrow.\n \n
		-For vehicles: hold the SHIFT key to force the path drawn to go along the nearest road.
	"];
	hintSilent _msg;
		
	jac_isPathDrawingActiveCutMsgId= CBA_missionTime; _isPathDrawingActiveCutMsgIdOld= jac_isPathDrawingActiveCutMsgId;
	[{
		params ["_isPathDrawingActiveCutMsgIdOld"];
		if (jac_isPathDrawingActiveCutMsgId == _isPathDrawingActiveCutMsgIdOld) then {titleText ["", "PLAIN"]; hintSilent ""};
	}, [_isPathDrawingActiveCutMsgIdOld], 3] call CBA_fnc_waitAndExecute;
		
	//// path drawing routine
	_configText= (gettext(configfile  >> "CfgVehicles" >> "gm_ge_army_bibera0" >> "gm_Bridge"  >> "BridgeType"));
	jac_isDragging= false;	jac_mouseCollisionType= "FIRE";
		
	/// toggle minimalist interface
	if (jac_pathPlanningMinimalistHUD) then 
	{
		jac_minimalistZeusInterfaceOGSetting= jac_minimalistZeusInterface;
		jac_minimalistZeusInterface= true;
		{((findDisplay 312) displayCtrl _x) ctrlShow !(jac_minimalistZeusInterface)} forEach [450, 16805, 16806, 16809, 15283];
		[jac_minimalistZeusInterface] call jac_fnc_minimalistUIactivate
	};
};

//creates path logic associated to the given list of unit
jac_fnc_pathLogicCreate=
{
	params ['_unitList', '_setTime'];
	{
		_unit= _x; _pos= getPosATL _unit; _direction= getDir _unit;
		_unitSize= 1.25; _isVehicle= false;	//2.25
		if !(isNull (objectParent _unit)) then {_isVehicle= true; _unitSize= ((sizeOf (typeOf (vehicle _unit))) * 0.25) + 0.75 };
		
		_myPos= (_pos getPos [_unitSize, _direction]);	//position to place the logic, just in front of the unit
		_myPos set [2, (_pos # 2)];	//set correct height
		
		/// set path color code
		_pathColor= _unit getVariable ['jac_myPathColor',[0,0,0,0]]; //default neutral color
		if (_pathColor isEqualTo [0,0,0,0]) then
		{
			while { ( (jac_pathColorList findIf {_x isEqualTo _pathColor} != -1) && (count jac_pathColorList < 600) ) } do {_pathColor= [(random 1.1), (random 1.1), (random 1.1), 1]}; //get new color
		};
		if (jac_pathColorList findIf {_x isEqualTo _pathColor} == -1) then {jac_pathColorList pushBack _pathColor}; //add new color to list
		
		_unit setVariable ["jac_isPathPlanningMode", true];
		_unit setVariable ["jac_drawPathArray", []];
		_unit setVariable ["jac_myUnitSize", _unitSize];
		_unit setVariable ["jac_isVehiclePath", _isVehicle];
		_unit setVariable ['jac_myPathColor', _pathColor];
		_unit setVariable ['jac_myPathDir', (getDir _unit)];
		
		//3D arrow used as handler
		_txt= format ["#(rgb,8,8,3)color(%1,%2,%3,0.5)", (_pathColor # 0), (_pathColor # 1), (_pathColor # 2)];
		
		_zMod= 0.25; _scale= 1.25;
		if (_isVehicle) then {_zMod= 0; _scale= 1.5};
		_marker= createSimpleObject ['Sign_Arrow_Direction_Cyan_F', [0,0,0], true];
		_marker setObjectTexture [0, _txt];
		_marker setDir (getDir _unit);
		_marker setPosATL (_myPos vectorAdd [0,0,_zMod]);
		_unit setVariable ['jac_myPathMarker', _marker];
		_marker attachTo [_unit];
		_marker setObjectScale _scale;
		_marker setVariable ["jac_pathMarkerScale", _scale];
		
		jac_pathMarkerList pushBack _marker;
		jac_pathLogicList pushBackUnique _unit;
		_unit setVariable ["jac_isResetingPath", false];
		
		if (jac_pathPlanningForceStop) then {_unit disableAI "PATH"; _unit setVariable ["jac_pathPlanningForceStopActive", true, true]};
	}forEach _unitList;
	
	[{
		params ["_setTime"];
		jac_canClosePathDrawing= true;
		/// reduce game speed
		if (_setTime) then
		{
			jac_savedGameSpeed= jac_game_speed;
			setAccTime jac_pathPlanningSpeed; jac_game_speed= jac_pathPlanningSpeed;
		};	
	}, [_setTime], 0.0001] call CBA_fnc_waitAndExecute;
};

/// add new path points when dragging a path arrow with the mouse in path planning mode
jac_fnc_addDrawPathPoint=
{
	params ["_pathLogic", "_targetPos", "_stickToRoad", "_roadReferencepos"];	//_pathLogic here is actually the unit itself
	jac_isDragging=true;
	
	_addedPath= false;	//whether a new path position is added
	_drawPathArray= _pathLogic getVariable ["jac_drawPathArray", []];
	_countDrawPathArray= count _DrawPathArray;
	_drawPathMarkersArray= _pathLogic getVariable ["jac_drawPathMarkersArray", []];
	_drawPathMapMarkersArray= _pathLogic getVariable ["jac_drawPathMapMarkersArray", []];
	_marker= _pathLogic getVariable ["jac_myPathMarker", objNull]; detach _marker;	//path arrow
	
	if ( ((_pathLogic distance ASLToAGL _targetPos) <= 1.85) && (_countDrawPathArray == 0) ) exitWith {false};
	_pathPos= []; 	
	if (_countDrawPathArray > 0) then {_pathPos= AGLToASL (_drawPathArray # ((_countDrawPathArray) - 1) # 0)} else {_pathPos= (getPosASL _pathLogic)};
	_pathPosOld= _pathPos;
	_pathDir= _pathLogic getVariable ['jac_myPathDir', (getDir _pathLogic)];
	
	// set the kind of path, infantry or vehicles - set variables
	_unit= _pathLogic;
	_myUnitSize= _pathLogic getVariable ["jac_myUnitSize", 1];

	_LOD1= 'GEOM'; _LOD2= 'VIEW';
	_spacing= jac_pathPlanningSpacing;
	_pathInterval= 6;
	_maxHeightChange= 2;
	_maxInclination= 0.9; //0.85
	_unitHeight= 0.3;	//how low will check for ground to stand on
	_roadHeightMod= 0.1;
	_collisionLineHeight= 0.45;	//height of the collision line checking from one position to the other
	_checkHeadingDis= 0.1; _checkHeading= [0, 45, 90, 135, 180, 225, 270, 315];	//for the ground check, will check 4 positions around the center, all must return some ground to stand on
	_avoidance= 0.3;	//spacing given from obstacles near the path pos (path smoothing)
	
	_isVehicle= _pathLogic getVariable ["jac_isVehiclePath", false];
	if (_isVehicle) then 
	{
		_spacing= jac_pathPlanningSpacingVeh; _pathInterval= 12; _unit= (vehicle _unit); _LOD1= 'NONE'; _LOD2= 'FIRE';
		_maxHeightChange= (_myUnitSize + 5); _maxInclination= 0.9; _unitHeight= 0.7; _collisionLineHeight= 0.65; _checkHeadingDis= 0.25; _roadHeightMod= 0.6; _avoidance= 3;
	};
	
	// path magnetization to roads; if shift is being held
	_magnetizationValue= 600;
	if ( (_stickToRoad) && (_isVehicle) ) then
	{
		//as long as the mouse hasn't moved more than _magnetizationValue away from the last _pathPos, use the road post that is nearest to the midle point between the _pathPos and the mouse pos
		_dis= _roadReferencepos distance _pathPos; _road= objNull;
		if (_dis < _magnetizationValue) then
		{
			//get the road that is inbetween the last post and mouse pos
			_checkPos= [ASLToAGL _roadReferencepos, _pathPos, (_dis *0.7)] call jac_fnc_findInterpolatedPoint;
			_road= [_checkPos, _magnetizationValue] call BIS_fnc_nearestRoad;
		};
		if !(isNull _road) then {_targetPos= (getPosASL _road) vectorAdd [0,0,_roadHeightMod]; _maxHeightChange= 150; _maxInclination= 1};
	};
	
	/// check if the target position is valid and add to the path
	_dis= (_pathPos distance _targetPos);
	if ( (_dis > _spacing) && ( _dis < 1000 ) && ( (( _targetPos # 2) - ( _pathPos # 2)) < (_maxHeightChange) ) && ( ((_pathPos vectorFromTo _targetPos) # 2) < _maxInclination) )  then //if within correct distance, max height and inclination change
	{
		// check if the new position is valid, collision line, //the 'GEOM' > 'VIEW' order is to check for fences
		_hasLineOfSight= true;	_hasDoor= false;
		_collisionLine= ( (lineIntersectsSurfaces [( (_pathPos) vectorAdd [0,0,_collisionLineHeight]), ( (_targetPos) vectorAdd [0,0,_collisionLineHeight]), _unit, jac_3DMousePositionMarker, true, 1, _LOD1, _LOD2, true]) );								
	 
		// check if the collision should be ignored for some special cases
		if ( (count _collisionLine) > 0 ) then 
		{
			_hasLineOfSight= false; _collisionPoint= ASLToAGL(_collisionLine # 0 # 0);

			// ignore collision with a door
			_isDoor= [_collisionPoint] call jac_fnc_findNearestDoor;
			if ( ((_isDoor # 0 # 0) != -1) && ((_isDoor # 1) < 1) ) then {_hasLineOfSight= true; _hasDoor=true};	//door close enough to the collision point

			// ignore collision with infantry
			if ( ((_collisionLine # 0) # 2) isKindOf 'CAManBase' ) then {_hasLineOfSight= true};
		};
		
		if !(_hasLineOfSight)  then	// if no direct line of sight to the position, try to find a path to it
		{
			if ( !(jac_isSearchingPath) && (_dis < 1000) ) then
			{
				private _uniType= "man"; if (_stickToRoad) then {_uniType= "wheeled_APC"};
				jac_isSearchingPathTimer= CBA_missionTime +10;
				[_pathPos, _targetPos, _uniType] call jac_fnc_callPathfinder;
				
				//after path is found, add it to the path array
				[{( !(jac_isSearchingPath) or !(jac_isPathDrawingActive) or (CBA_missionTime >= jac_isSearchingPathTimer) )}, {
					params ['_pathPos', '_pathLogic', '_pathPosOld', '_marker', '_isVehicle', '_drawPathMapMarkersArray', '_LOD1', '_LOD2', '_collisionLineHeight'];
					if (count jac_pathAgent > 0) then 
					{
						//if the last path pos hasn't been update manually already
						_pathPosCheck= [];
						private _drawPathArray= _pathLogic getVariable ["jac_drawPathArray", []];
						if (count _drawPathArray > 0) then {_pathPosCheck= AGLToASL (_drawPathArray # ((count _drawPathArray) - 1) # 0)} else {_pathPosCheck= _pathPos};
						
						if (_pathPosCheck isEqualTo _pathPos) then
						{
							[jac_pathAgent, _pathLogic, _pathPosOld, _marker, _isVehicle, _drawPathMapMarkersArray, _LOD1, _LOD2, _collisionLineHeight] call jac_fnc_insertDrawPathPoints;
						};
					};
				}, [_pathPos, _pathLogic, _pathPosOld, _marker, _isVehicle, _drawPathMapMarkersArray, _LOD1, _LOD2, _collisionLineHeight]] call CBA_fnc_waitUntilAndExecute;
			};			
		}
		else
		{
			jac_isSearchingPath= false;	//cancel path searching when another path point is manually added
			
			// check if position has ground to stand on
			_hasMechBridge= false; _groundCheck = 0;
			_groundCheck= _checkHeading findIf 
			{
				_hasNoGroundCol= true; _checkPos= AGLToASL (_targetPos getPos [_checkHeadingDis, _x]); _checkPos set [2, (_targetPos # 2)];
				_lineCollisionGround= (lineIntersectsSurfaces [(_checkPos vectorAdd [0,0,0]), (_checkPos vectorAdd [0,0, (- _unitHeight)]), _unit, jac_3DMousePositionMarker, true, 1, "GEOM", "FIRE", true]);
				if ((count _lineCollisionGround) > 0) then
				{
					_hasNoGroundCol= false;
					_configText= (gettext(configfile  >> "CfgVehicles" >> "gm_ge_army_bibera0" >> "gm_Bridge"  >> "BridgeType"));	//handle mech bridge exception to allow vehicles over it
					if ((typeOf ((_lineCollisionGround # 0) # 2)) isEqualTo _configText) then {_hasMechBridge= true};
				};
				_hasNoGroundCol;
			};

			if ((_groundCheck == -1) or (_hasMechBridge)) then	////if it finds even 1 checked position with no collision, rejects this position unless it is a mech bridge underneath
			{
				/// path smoothing
				if (_stickToRoad) then
				{
					//every new path point, check if there is direct los (and is within a certain distance) to a few points before the previous path point and replace the path points inbetween with the new path point
					_countArray= (count _drawPathArray);
					if (_countArray >= 5) then
					{
						for "_i" from _countArray -5 to _countArray -2 step +1 do 
						{
							_checkPos= (_drawPathArray # _i # 0);
							_smoothingDis= _checkPos distance ASLToAGL _targetPos;
							if ( (_smoothingDis <= 25) && (_smoothingDis >= 5) ) then
							{
								_lineCollision= (lineIntersectsSurfaces [(_targetPos vectorAdd [0,0,0.5]), (AGLToASL _checkPos vectorAdd [0,0, 0.5]), _unit, jac_3DMousePositionMarker, true, 1, _LOD1, _LOD2, true]);
								if ((count _lineCollision) == 0) then {_drawPathArray deleteRange [_i +1, _countArray -1]; break};	//will delete the last
							};
						};
					};
				};
				
				/// update path data
				_addedPath= true;
				_pathPos= _targetPos;
				_moveDir= (_pathPosOld getDir _pathPos);
				_pathDir= _moveDir;
				_isTacticalMov= false;
				_animationDirName='';
				_facingDirMod= 0;
				
				//if holding alt will maintain looking at the direction the path arrow is pointed at
				if ((jac_holdLAlt) && !(_isVehicle)) then
				{
					_pathDir= getDir _marker;
					_isTacticalMov= true;
				};
				[_pathDir, _moveDir] call jac_getScriptedMoveAnimDirectionArray params ['_animationDirName', '_facingDirMod'];

				_pathLogic setVariable ['jac_myPathDir', _pathDir];
				_drawPathArray pushBack [(ASLToAGL _pathPos), _isTacticalMov, _animationDirName, _facingDirMod]; //[_pathPos, _isTacticalMov, _animationDirName, _facingDirMod];
				_pathLogic setVariable ["jac_drawPathArray", _drawPathArray];	//update path positions array
				_pathPosOld= _pathPos;

				/// move path arrow
				_marker setDir _pathDir;
				_marker setPosATL ((ASLToATL _pathPos) vectorAdd [0,0,0.5]);
				_marker setVectorUp surfaceNormal position _marker;
				_scale= ((curatorCamera modelToWorld [0,0,0]) distance _marker) * 0.02;
				_marker setObjectScale ((_marker getVariable ["jac_pathMarkerScale", 1.5]) * (1.25 + _scale));

				/// map markers
				/*
				if (_isVehicle) then
				{
					jac_markerGlobal= (jac_markerGlobal + 1);
					_mapMarkerName= ("jac_pathMarker_" + (str jac_markerGlobal));	//get an unique name
					_mapMarker = createMarkerLocal [_mapMarkerName, (ASLToAGL _pathPos)];
					_mapMarker setMarkerTypeLocal "mil_dot"; _mapMarker setMarkerAlphaLocal 0;	//will be invisible, is used only for the vehicle path following functions
					_drawPathMapMarkersArray pushBack _mapMarkerName;
					_pathLogic setVariable ["jac_drawPathMapMarkersArray", _drawPathMapMarkersArray];	//update path map markers array
				};*/

				//if multiple units are selected while in Planning Mode, dragging a path arrow will synchronize those units paths with the arrow being moved
				//get units that are currently selected except for the owner of the path arrow being dragged and synchronize those units path arrows with the arrow being moved 
				if (jac_selectedPathLogic == _pathLogic) then
				{
					private _units= (curatorSelected # 0) select { ( ( (alive _x) && (_x != _pathLogic) ) && ( ((vehicle _x) isKindOf 'CAManBase') or ( ((vehicle _x) isKindOf 'LandVehicle' ) && (_x == (driver (vehicle _x))) ) ) ) };												
					private _pathSize= (count _drawPathArray); private _stackPathData= [];
					{
						if ( (_pathSize-1) > (_pathInterval * (_forEachIndex +1)) ) then	//after each path interval set a new path point for the selected units 
						{
							//check if it has a path arrow
							if !(isNull (_x getVariable ["jac_myPathMarker", objNull])) then
							{
								private _3DMousePositionRelative= _drawPathArray # ( (_pathSize) - (_pathInterval * (_forEachIndex +1)) ) # 0;
								_stackPathData pushBack [_x, AGLToASL _3DMousePositionRelative];
							};
						};
					}forEach _units;
					
					{[(_x # 0), (_x # 1), false, []] call jac_fnc_addDrawPathPoint }forEach _stackPathData; //call for each unit
				};
			};
		};
	};
	_addedPath;
};


// suffix added to the end of move animation name // r, l = 180; f, b = 0; br = 90; bl = 270; fl = 135;  fr = 225 315
jack_animationDirectionArr= [ ['f', 0], ['fr', 315], ['r', 180], ['br', 90], ['b', 0], ['bl', 270], ['l', 180], ['fl', 135] ];

/// returns the proper animation direction to be used in scripted move when tactical step is used
jac_getScriptedMoveAnimDirectionArray=
{
	params ["_facingDir", "_moveDir"];

	// Calculate relative angle between movement and facing directions
	_relativeAngle = (_moveDir - _facingDir + 360) mod 360;

	// Determine index using 45-degree sectors offset by 22.5 degrees
	_index= (floor((_relativeAngle + 22.5) / 45)) mod 8;
	_animationDir= jack_animationDirectionArr # _index # 0;
	_facingDirMod= ((_index * 45) mod 360) + (jack_animationDirectionArr # _index # 1);
	[_animationDir, _facingDirMod];
};

/// force inserts the given path points in a path array
jac_fnc_insertDrawPathPoints=
{
	params ["_pathPositions", "_pathLogic", "_pathPosOld", "_marker", "_isVehicle", "_drawPathMapMarkersArray", "_LOD1", "_LOD2", "_collisionLineHeight"];
	
	private _drawPathArray= _pathLogic getVariable ["jac_drawPathArray", []];
	private _pathPos= []; private _oldPathPos= []; private _pathDir= 0; _hasLineOfSight= true;
	{
		_pathPos= _x; 
		
		_hasLineOfSight= true;	_hasDoor= false;

		_collisionLine= ( (lineIntersectsSurfaces [( (_pathPosOld) vectorAdd [0,0,0.25]), ( (_pathPos) vectorAdd [0,0,0.25]), _pathLogic, jac_3DMousePositionMarker, true, 1, _LOD1, _LOD2, true]) );
		// check if the collision should be ignored for some special cases
		if ( (count _collisionLine) > 0 ) then 
		{
			_hasLineOfSight= false; _collisionPoint= ASLToAGL(_collisionLine # 0 # 0);

			// ignore collision with a door
			_isDoor= [_collisionPoint] call jac_fnc_findNearestDoor;
			if ( ((_isDoor # 0 # 0) != -1) && ((_isDoor # 1) < 1) ) then {_hasLineOfSight= true; _hasDoor=true};	//door close enough to the collision point

			// ignore collision with infantry
			if ( ((_collisionLine # 0) # 2) isKindOf 'CAManBase' ) then {_hasLineOfSight= true};
		};			
	
		if !(_hasLineOfSight) exitWith {_pathPos= []; break};

		_pathDir= _pathPosOld getDir _pathPos;
		_pathLogic setVariable ['jac_myPathDir', _pathDir];
		_drawPathArray pushBack [(ASLToAGL _pathPos), false, '', 0];
		
		_pathLogic setVariable ["jac_drawPathArray", _drawPathArray];	//update path positions array
		_pathPosOld= _pathPos;

		/// map markers
		/*if (_isVehicle) then
		{
			jac_markerGlobal= (jac_markerGlobal + 1);
			_mapMarkerName= ("jac_pathMarker_" + (str jac_markerGlobal));	//get an unique name
			_mapMarker = createMarkerLocal [_mapMarkerName, (ASLToAGL _pathPos)];
			_mapMarker setMarkerTypeLocal "mil_dot"; _mapMarker setMarkerAlphaLocal 0;	//will be invisible, is used only for the vehicle path following functions
			_drawPathMapMarkersArray pushBack _mapMarkerName;
			_pathLogic setVariable ["jac_drawPathMapMarkersArray", _drawPathMapMarkersArray];	//update path map markers array
		};*/
	}forEach _pathPositions;
	
	if (count _pathPos > 0) then
	{
		/// move path arrow
		_marker setDir _pathDir;
		_marker setPosATL ((ASLToATL _pathPos) vectorAdd [0,0,0.5]);
		_marker setVectorUp surfaceNormal position _marker;
		_scale= ((curatorCamera modelToWorld [0,0,0]) distance _marker) * 0.02;
		_marker setObjectScale ((_marker getVariable ["jac_pathMarkerScale", 1.5]) * (1.25 + _scale));
	};
};

/// delete and reset path marker
jac_fnc_resetPath=
{
	params ["_pathLogic"];
	_pathLogic setVariable ["jac_isResetingPath", true];
	
	/// delete path map markers
	{deleteMarkerLocal _x}forEach (_pathLogic getVariable ["jac_drawPathMapMarkersArray", []]);
	_pathLogic setVariable ["jac_drawPathMapMarkersArray", []];
	
	/// delete and reset 3D arrow
	deleteVehicle (_pathLogic getVariable ["jac_myPathMarker", objNull]);
	[{
		params ['_pathLogic'];
		[[_pathLogic], false] call jac_fnc_pathLogicCreate;
	}, [_pathLogic], 0.0001] call CBA_fnc_waitAndExecute;
};

///Hold CTRL while dragging the path arrow to delete a portion of the path
jac_fnc_deleteDrawPathPoint=
{
	params ["_pathLogic"];

	_drawPathArray= _pathLogic getVariable ["jac_drawPathArray", []];
	_countPath= (count _drawPathArray);
	
	if ((_countPath == 1) && !(_pathLogic getVariable ["jac_isResetingPath", false]) ) exitWith {[_pathLogic] call jac_fnc_resetPath};
	
	if (_countPath > 1) then
	{
		_marker= _pathLogic getVariable ["jac_myPathMarker", []];

		_pathPosOld= (_drawPathArray # (_countPath -1) # 0);	//current post that will be deleted
		_pathPos= (_drawPathArray # (_countPath -2) # 0);		//pos that will become the current pos
	
		_pathDir= _pathPos getDir _pathPosOld;
		_pathLogic setVariable ['jac_myPathDir', _pathDir];		
		_pathLogic setVariable ["jac_drawPathArray", _drawPathArray];
		
		_marker setDir _pathDir;
		_marker setPosATL (_pathPos vectorAdd [0,0,0.25]);
		_marker setVectorUp surfaceNormal position _marker;
		_scale= ((curatorCamera modelToWorld [0,0,0]) distance _marker) * 0.02;
		_marker setObjectScale ((_marker getVariable ["jac_pathMarkerScale", 1.5]) * (1.25 + _scale));
		
		_drawPathMapMarkersArray= _pathLogic getVariable ["jac_drawPathMapMarkersArray", []];
		_countMapPath= (count _drawPathMapMarkersArray);
		if (_countMapPath > 1) then
		{
			deleteMarkerLocal (_drawPathMapMarkersArray # (_countMapPath -1));
			_drawPathMapMarkersArray deleteAt (_countMapPath -1);
			_pathLogic setVariable ["jac_drawPathMapMarkersArray", _drawPathMapMarkersArray];	//update path map markers array
		};
		_drawPathArray deleteAt (_countPath -1);
	}
};

///Hold CTRL and click a path position path arrow to delete a portion of the path
jac_fnc_cutPathAtPosition=
{
	params ["_pathPos", "_unit"];
	_pathArray= _unit getVariable ["jac_drawPathArray", []];
	_id= -1;

	_countPath= (count _pathArray);
	if ( (_countPath == 1) && !(_unit getVariable ["jac_isResetingPath", false]) ) then {[_unit] call jac_fnc_resetPath}
	else
	{
		_id= _pathArray findIf {(_x # 0) isEqualTo _pathPos};
		if (_id != -1) then 
		{			
			_marker= _unit getVariable ["jac_myPathMarker", []];

			_pathPosOld= (_pathArray # (_id-1) # 0);	//current post that will be deleted
	
			_pathDir= _pathPosOld getDir _pathPos;
			_unit setVariable ['jac_myPathDir', _pathDir];		
			_unit setVariable ["jac_drawPathArray", _pathArray];
		
			_marker setDir _pathDir;
			_marker setPosATL (_pathPos vectorAdd [0,0,0.25]);
			_marker setVectorUp surfaceNormal position _marker;
			_scale= ((curatorCamera modelToWorld [0,0,0]) distance _marker) * 0.02;
			_marker setObjectScale ((_marker getVariable ["jac_pathMarkerScale", 1.5]) * (1.25 + _scale));
			
			_pathArray deleteRange [_id, ((count _pathArray) -_id)];	//delete path positions
			
			//delete map markers
			_drawPathMapMarkersArray= _unit getVariable ["jac_drawPathMapMarkersArray", []];
			_countMapPath= (count _drawPathMapMarkersArray);
			if (_countMapPath > 1) then
			{
				for "_i" from _countMapPath -1 to _id step -1 do
				{
					deleteMarkerLocal (_drawPathMapMarkersArray # (_i));
					_drawPathMapMarkersArray deleteAt (_i);
				};
				_unit setVariable ["jac_drawPathMapMarkersArray", _drawPathMapMarkersArray];	//update path map markers array
			};
		};
	};
	_unit setVariable ["jac_drawPathArray", _pathArray];
};


/// closing path mode
jac_fnc_closePathDrawingMode=
{
	if ( !(jac_isPathDrawingClosing) && (jac_isPathDrawingActive) ) then
	{
		jac_pathArrowSetDir= objNull;
		jac_isPathDrawingClosing= true;
		//reset effects	
		jac_mouseCollisionType= "VIEW";
		setAccTime jac_savedGameSpeed;	jac_game_speed= jac_savedGameSpeed;
		ppEffectDestroy jac_pathDrawingModeEffect;

		///resolve path movement
		_didExecute= false;
		if (count jac_pathLogicList > 0) then 
		{
			_pathLogicListCount= count jac_pathLogicList;
			for "_i" from 0 to _pathLogicListCount do
			{
				_logic= jac_pathLogicList # _i;
				if (isNil "_logic") then {continue};
				
				if (jac_pathPlanningForceStop) then {_logic enableAI "PATH"; _logic setVariable ["jac_pathPlanningForceStopActive", false, true]};
				
				_drawPathArray= _logic getVariable ["jac_drawPathArray", []];
				_drawPathMapMarkersArray= _drawPathArray;
				
				/*
				_drawPathMapMarkersArrayOriginal= (_logic getVariable ["jac_drawPathMapMarkersArray", []]);
				_drawPathMapMarkersArray= +_drawPathMapMarkersArrayOriginal;				
				_logic setVariable ["jac_drawPathMapMarkersArray", []];
				*/
				
				_logic setVariable ["jac_isPathPlanningMode", false];
		
				if ((count _drawPathArray) > 0) then
				{
					_didExecute= true;
					//to remote exec path following where unit is locals
					_unit= _logic;
					_isVehicle= _logic getVariable ["jac_isVehiclePath", false];
					_veh= (vehicle _unit);
					
					//for vehicle movement, use map markers to determine path 
					//_vehiclePath = _drawPathMapMarkersArray apply { getMarkerPos _x; }; // getting all map markers positions
					_vehiclePath = [];
					_targetSpeed= 1;
					
					if (_isVehicle) then
					{
						{_vehiclePath pushBack (_x # 0)}forEach _drawPathArray;
					
						//determine moving speed based on the vehicle performance and AI speed mode, form up mode
						_unitClassName= typeOf _veh;
						_maxSpeed= _veh getVariable ['jac_convoySpeed', 0];
						if (_maxSpeed == 0) then 
						{
							_maxSpeed= ( ( (getNumber (configFile >> "CfgVehicles" >> _unitClassName >> "maxSpeed")) ) min jac_maxVehicleSpeed );
							if ( (speedMode _veh) isEqualTo "NORMAL") then {_targetSpeed= (_maxSpeed)} else {_targetSpeed= (_maxSpeed * 0.25)};
						}
						else
						{_targetSpeed= _maxSpeed};
					
						if !((_veh getVariable ['jac_myStack', grpNull]) isEqualTo grpNull) then 
						{
							if ( (speedMode _veh) isEqualTo "NORMAL") then {_targetSpeed= 13} else {_targetSpeed= 4.25};
						};
					
						// setting speed - transform from Km/H to M/S
						_targetSpeedMS= (_targetSpeed * (1000 / 3600));
						{ _x pushBack _targetSpeedMS } forEach _vehiclePath;
					};
					
					[_unit, _vehiclePath, _drawPathMapMarkersArray, _drawPathArray, _isVehicle, _targetSpeed] remoteExecCall  ["jac_fnc_unitMovePathDrawing", 0];
				};
			};
			jac_pathLogicList = [];			
		};
		if (_didExecute) then {titleText ['EXECUTE!', 'PLAIN']};
		
		/// delete 3d arrows path handlers
		if (count jac_pathMarkerList > 0) then 
		{
			_pathMarkerListCount= count jac_pathMarkerList;
			for "_i" from 0 to _pathMarkerListCount do
			{
				_marker= jac_pathMarkerList # _i;
				if (isNil "_marker") then {continue};
				deleteVehicle _marker;
			};
		};
		jac_pathMarkerList= [];

		///close path drawing mode		
		if !(count jac_group_control > 0) then {deleteVehicle jac_3DMousePositionMarker};	//delete mouse 3D position marker when no unit is selected and not in path drawing mode
		jac_isPathDrawingActive= false; jac_isPathDrawingClosing= false;
		jac_selectedPathLogic= objNull; jac_hoveredPathLogic= objNull;

		[{ if !(jac_isPathDrawingActive) then {titleText ["", "PLAIN"]; hintSilent ""}; }, [], 2] call CBA_fnc_waitAndExecute;
		
		jac_isSearchingPath= false;	//cancel path searching
		
		/// return to normal interface mode	
		if (jac_pathPlanningMinimalistHUD) then 
		{
			jac_minimalistZeusInterface= jac_minimalistZeusInterfaceOGSetting;
			{((findDisplay 312) displayCtrl _x) ctrlShow !(jac_minimalistZeusInterface)} forEach [450, 16805, 16806, 16809, 15283];
			[jac_minimalistZeusInterface] call jac_fnc_minimalistUIactivate
		};
		
		/// always close aircraft control panel when exiting planning mode
		if (jac_showAircraftControlPanelActive) then
		{
			jac_showAircraftControlPanelActive= false;
			call jac_fnc_closeAircraftControlPanel;
		};
	};
};

jac_fnc_unitMovePathDrawing=
{
	params ['_unit', '_vehiclePath', '_drawPathMapMarkersArray', '_drawPathArray', '_isVehicle', '_targetSpeedMS'];
	
	if !(local _unit) exitWith {false};

	_isPatrol= false;
	
	_firstPos= [( _drawPathArray # 0 # 0 # 0), ( _drawPathArray # 0 # 0 # 1), 0];
	_lastPos= [(_drawPathArray # ((count _drawPathArray) -1) # 0 # 0), (_drawPathArray # ((count _drawPathArray) -1) # 0 # 1), 0];

	if ( ((count _drawPathArray) > 15) && ( _firstPos distance _lastPos <= 3) ) then {_isPatrol= true};	//is a patrol loop
	
	if !(_isVehicle) then 
	{
		//cancel stacking behind vehicle
		_group= group _unit;
		if (_group getVariable ['jac_doFormUp', false]) then {_group setVariable ['jac_doFormUp', false, true]};
		
		//start scripted move
		_unit setVariable ['jac_endScriptedMove', true, true];
		[{
			params ['_unit', '_drawPathArray', '_isPatrol'];
			[_unit, _drawPathArray, jac_scriptedMoveSpacing, jac_scriptedMoveTurnSpd, _isPatrol] remoteExecCall ["jac_fnc_scriptedMove", 0];
		}, [_unit, _drawPathArray, _isPatrol], 0.25] call CBA_fnc_waitAndExecute;
	}
	else
	{
		if ((count _drawPathMapMarkersArray) > 0) then
		{
			_vehicle= (vehicle _unit);		
			{_x setVariable ['jac_endScriptedMove', true, true]} forEach (crew _vehicle);	//to reset current drive on path properly
			_vehicle setVariable ["jac_isReversing", false, true];	//stop reverse move
			_vehicle setVariable ['jac_myConvoyParent', objNull, true];	//cancel convoy following
		
			//make sure the effectiveCommander of vehicle is always in the same group as the driver 
			_vehCommanderGRP= (group (effectiveCommander _vehicle));
			_driverGRP= (group (driver _vehicle));
			if !(_vehCommanderGRP isEqualTo _driverGRP) then {_vehicle setEffectiveCommander (leader _driverGRP)};

			doStop (_vehicle); //To use setDriveOnPath, the AI must not have a formLeader. The easiest way to achieve this is to use doStop to stop the AI

			//waitAndExecute
			_waitAndExecute= (CBA_missionTime +0.25);
			[{
				params ['_vehicle', '_lastPos', '_unit', '_vehiclePath', '_drawPathMapMarkersArray', '_isPatrol', '_targetSpeedMS', '_firstPos', '_waitAndExecute'];				
				( !(_unit getVariable ['jac_InScriptedMoveVeh', false]) && (CBA_missionTime >= _waitAndExecute) );	//only starts a new scripted move after the current one has ended				
			},{
				params ['_vehicle', '_lastPos', '_unit', '_vehiclePath', '_drawPathMapMarkersArray', '_isPatrol', '_targetSpeedMS', '_firstPos', '_waitAndExecute'];
				
				{_x setVariable ['jac_endScriptedMove', false, true]}forEach (crew _vehicle);
				_doMove=true; _moveTimer= (CBA_missionTime +5);
				_unit setVariable ['jac_doStartScriptedMoveVeh', true];
				
				_unit setVariable ['jac_scriptedMoveTime', _moveTimer];
				
				//_addPerFrameHandler - movement loop
				[{
					(_this # 0) params ['_vehicle', '_lastPos', '_unit', '_vehiclePath', '_drawPathMapMarkersArray', '_isPatrol', '_doMove', '_moveTimer', '_targetSpeedMS', '_firstPos'];
					
					// end movement because unit is dead, driver changed or movement was cancelled
					_endScriptedMove= ((driver _vehicle) getVariable ['jac_endScriptedMove', true]);
					if ( !(alive _unit) or !(alive _vehicle) or (_vehicle != _vehicle) or (_endScriptedMove) or !((driver _vehicle) isEqualTo _unit) ) then {_doMove= false};
	
					// detect when reached the end pos
					if ( (CBA_missionTime > _moveTimer) ) then
					{
						if ( ((getPos _vehicle) distance (_lastPos) ) <= 14 ) then
						{
							_unit setVariable ['jac_doStartScriptedMoveVeh', true];	//to start moving again after completede a loop
							if !(_isPatrol) then {_doMove= false};	//if is not a patrol end movement
						};
					};
					
					// start moving				
					if ( (_doMove) && !(_endScriptedMove) && (_unit getVariable ['jac_doStartScriptedMoveVeh', false]) ) then
					{
						_unit setVariable ['jac_InScriptedMoveVeh', true];
						_unit setVariable ['jac_doStartScriptedMoveVeh', false];
						_vehicle setDriveOnPath _vehiclePath;
					};
					
					// fail-safe, checks if unit is actually moving and try to move it again if not
					if ( (_doMove) && (_unit getVariable ['jac_InScriptedMoveVeh', false]) && ( CBA_missionTime > (_unit getVariable ['jac_scriptedMoveTime', 0]) ) && ( ((getPos _vehicle) distance ([_firstPos # 0, _firstPos # 1, _firstPos # 2])) <= 9 ) ) then
					{
						_unit setVariable ['jac_scriptedMoveTime', (CBA_missionTime +5)];
						_vehicle setDriveOnPath _vehiclePath;
					};
					
					// exit movement loop
					if !(_doMove) then
					{
						_unit setVariable ['jac_InScriptedMoveVeh', false];
						{_x setVariable ['jac_endScriptedMove', true, true]}forEach (crew _vehicle);
						//{deleteMarker _x}forEach _drawPathMapMarkersArray;	/// delete path map markers
						
						//to resume normal movement
						[{
							params ['_vehicle'];
							_group= (group (driver _vehicle));
							if !((driver _vehicle) getVariable ['jac_InScriptedMoveVeh', false]) then 
							{
								_wpPos= (getWPPos [_group, (currentWaypoint _group)]);
								if (_wpPos isEqualTo [0,0,0]) then 
								{
									if !(_vehicle getVariable ['jac_doSuppressAction', false]) then {[[_group]] call jac_fnc_cancel_orders};
								}
								else
								{_vehicle doFollow _vehicle};
							};
						}, [_vehicle], 0.25] call CBA_fnc_waitAndExecute;
				
						[(_this # 1)] call CBA_fnc_removePerFrameHandler;
					};
	
					// maintain movement speed within the max limit
					_speedOver= ( (speed _vehicle) - _targetSpeedMS );
					if (_speedOver > 0) then //vehicle is above speed limit
					{
						_vel = velocity _vehicle; _dir = getDir _vehicle; _speed= (-1 * (speed _vehicle)) *0.1;
						_vehicle setVelocity [(_vel select 0) + (sin _dir * _speed), (_vel select 1) + (cos _dir * _speed), (_vel select 2)];
					};
				}, 0.001, [_vehicle, _lastPos, _unit, _vehiclePath, _drawPathMapMarkersArray, _isPatrol, _doMove, _moveTimer, _targetSpeedMS, _firstPos]] call CBA_fnc_addPerFrameHandler;
			}, [_vehicle, _lastPos, _unit, _vehiclePath, _drawPathMapMarkersArray, _isPatrol, _targetSpeedMS, _firstPos, _waitAndExecute]] call CBA_fnc_waitUntilAndExecute;
		};
	};
};


//find the appropriate animation that interpolates with my current animation or with one of the animations my current animation interpolates with
//because if the unit is idle, it will only find the animation that links idle animations to movement animations
//make a funnel to filter out the best possible or least worst result
jac_fnc_scriptedMoveGetAnimationName=
{
	params ['_unit'];	scopeName "mainScope";
	_scriptedMoveAnim= (animationState _unit); _scriptedMoveAnimEnd= _scriptedMoveAnim;
	if (("aidlpercmstpsraswrfldnon" in _scriptedMoveAnim) or ("aidlpknlmstpsraswrfldnon" in _scriptedMoveAnim)) then {_scriptedMoveAnim = 'AmovPercMrunSrasWrflDf'}; //can't always find animations from the iddle state
	_animationList=[]; _animationList1= []; _animationList2= []; _animationListFinal= [];

	_configMoves= (getText (configFile >> "CfgVehicles" >> typeOf _unit >> "moves"));
	{_animationList1 append (getArray (configfile >> _configMoves >> "States" >> _scriptedMoveAnim >> _x))}forEach ["InterpolateFrom", "InterpolateTo", "ConnectTo", "connectFrom"];	
	_animationList1= _animationList1 arrayIntersect _animationList1;

	{
		if ((typeName _x) != 'STRING') then {continue}; _animationState= _x;
		{_animationList2 append (getArray (configfile >> _configMoves >> "States" >> _animationState >> _x))}forEach ["InterpolateFrom", "InterpolateTo", "ConnectTo", "connectFrom"];
	}forEach _animationList1;
  
	if !("melee" in str _animationList1) then {
		_animationList2= _animationList2 arrayIntersect _animationList2;
		{
			if ((typeName _x) != 'STRING') then {continue}; _animationState= _x;
			{_animationListFinal append (getArray (configfile >> _configMoves >> "States" >> _animationState >> _x))}forEach ["InterpolateFrom", "InterpolateTo", "ConnectTo", "connectFrom"];
		}forEach _animationList2;
	};
	_animationList append _animationList1; _animationList append _animationList2;
	_animationList= _animationList arrayIntersect _animationList; _animationListFinal= (_animationList select {(typeName _x) == 'STRING'});

	//animation is based on the unit stance (prone, crouch, standing), speed mode (slow, fast) and weapon (primary, pistol or unarmed)
	_grp= group _unit; _moveDir= ["Df"]; _spd= ["Mrun"]; _spd2= ["Mrun"]; _spd3= ["Mrun"]; _spd4= ["Mrun"]; _stance= ["Pnon"]; _weaponHolding= ["Sras"]; _weaponType= ["Wnon"]; 
	_speedMode= speedMode _grp; _stanceUnit= stance _unit; if ( ((_unit getHit "legs") > 0.5) or (isForcedWalk _unit) or ((surfaceNormal (getPosASL _unit)) # 2 < 0.8) ) then {_speedMode= "LIMITED"};
	switch (_speedMode) do
	{
		case "NORMAL"; case "FULL"; case "AUTO": {_spd= ["Mrun", "run", "Run", "sprint", "Sprint", "Move_Fast"]; _spd2= ["run_", "Run_", "sprint", "Sprint"]; _spd3= ["Mwlk", "walk", "Walk"]; _spd4= ["walk_", "Walk_", "Move_Slow"]};
		case "LIMITED": {_spd= ["Mwlk", "walk", "Walk", "Move_Slow"]; _spd2= ["walk_", "Walk_"]; _spd3= ["Mrun", "run", "Run", "sprint", "Sprint"]; _spd4= ["run_", "Run_", "sprint", "Sprint", "Move_Fast"]};
	};
	
	switch (_stanceUnit) do
	{
		case "STAND": {_stance= ["Perc"]};
		case "CROUCH": {_stance= ["Pknl"]};
		case "PRONE": {_stance= ["Ppne"]; _weaponHolding= ["Slow"];  _spd= ["Mrun"]};
	};

	_weapon= [(primaryWeapon _unit), (handgunWeapon _unit), (secondaryWeapon _unit)] findIf {_x == (currentWeapon _unit)};
	if ( !((currentWeapon _unit) == "") && !(_weapon == -1) ) then {_weaponType= [["Wrfl", "Wpst", "Wlnr"] # _weapon]};	//weapon type, in case no weapon or unknown weapon use "non"
	if ( ((_stance # 0) isEqualTo "Perc") && ((_grp getVariable ['jac_actionMode', "combatMode"]) isEqualTo 'maneuverMode') && (_speedMode == "LIMITED") ) then 
	{_weaponHolding= ["Slow"]; if ( (_weaponType # 0) != "Wpst" ) then {_moveDir= ["Df_ver2"]}};
	
	//find moving animation - "Amov" = is a movement action - "Df" = direction of movement forward
	_myAnimCheck= ("Amov" + (_stance # 0) + (_spd # 0) + (_weaponHolding # 0) + (_weaponType # 0) + (_moveDir # 0));
	if (_myAnimCheck in _animationListFinal) then {_scriptedMoveAnim= _myAnimCheck} else
	{
		_animationListTest= [_spd, _spd2, _spd3, _spd4, ["Amov"], _moveDir, _weaponType, _stance, _weaponHolding]; _animFilteredList= +_animationListFinal;
		for "_i" from 0 to (count _animationListTest - 1) step +1 do {
			_arrayF= (_animFilteredList select {_checkAnim= _x; (({(_checkAnim find _x) != -1} count (_animationListTest # _i)) > 0) });		
			if (count _arrayF > 0) then {_animFilteredList= +_arrayF};
		};
	   
		_count= 0;
		{
			_test= _x; _cnt= ({_x == _test} count _animFilteredList);
			if (_cnt > _count) then {_scriptedMoveAnim= _test; _count= _cnt};
		}forEach _animFilteredList;
	};
	
	//handle exceptions
	if (_scriptedMoveAnim isEqualTo "Deatclaw_SprintTransit_IN") then {_scriptedMoveAnim= "Deathclaw_Sprint"};
	if (_scriptedMoveAnim isEqualTo "Goliaph_Run_B") then {_scriptedMoveAnim= "goliaph_run"}; 

	[_scriptedMoveAnim, _scriptedMoveAnimEnd];
};

//////scripted move function
// based on BIS_fnc_scriptedMove, to manipulate unit movement; must run in scheduled environment

jac_fnc_scriptedMove=
{
params ['_unit', '_path', '_completionRadius', '_turnSpeedCoef', '_isPatrol'];

if (!(local _unit) or (_unit getVariable["jac_isPanicked", false])) exitWith {false};	//panicked units can't use scripted move

[_unit] call jac_fnc_scriptedMoveGetAnimationName params ["_scriptedMoveAnim", "_scriptedMoveEndAnim"];
_unit setVariable ['jac_scriptedMoveAnimAnimOG', _scriptedMoveAnim];

// if starts in tactical step mode
_isTacticalMov= (_path # 0 # 1);
_animationDirName= (_path # 0 # 2);
if (_isTacticalMov) then
{
	_scriptedMoveAnim= ((_scriptedMoveAnim select [0, (count _scriptedMoveAnim -1)]) + _animationDirName);
};

_unit setVariable ['jac_scriptedMoveisTacticalMov', _isTacticalMov];
_unit setVariable ['jac_scriptedMoveanimationDirName', _animationDirName];
_unit setVariable ['jac_scriptedMovefacingDirMod', (_path # 0 # 3)];

_unit setVariable ['jac_scriptedMoveAnimAnim', _scriptedMoveAnim];
_unit setVariable ['jac_scriptedMoveEndAnim', _scriptedMoveEndAnim];
_unit setVariable ['jac_scriptedMoveData', [(speedMode (group _unit)), /*(stance _unit),*/ (((_unit getHit "legs") > 0.5) or (isForcedWalk _unit)), ((group _unit) getVariable ['jac_actionMode', "combatMode"]), ((surfaceNormal (getPosASL _unit)) # 2 < 0.8)], true];	//save data for animation update in case of change to the unit status

_unit setVariable ['jac_endScriptedMove', false, true];
_unit setVariable ['jac_pauseScriptedMove', false];	//when the unit pauses movement to engage a target
_unit setVariable ['jac_targetScriptedMove', objNull];

////if is a patrol loop
_unit setVariable ['jac_isScriptedPatrol', _isPatrol, true];

_unit disableAI "MOVE";	_unit disableAI "ANIM";
_unit playMoveNow _scriptedMoveAnim;

//EH to keep movement animation playing
_unit addEventHandler ["AnimDone",
{
	params ["_unit", "_anim"];
	if !(typeName  _unit != "OBJECT") exitWith { _unit removeEventHandler [_thisEvent, _thisEventHandler] }; // exit if movement is canceled
	if (_unit getVariable ['jac_endScriptedMove', false]) exitWith { _unit removeEventHandler [_thisEvent, _thisEventHandler] }; // exit if movement is canceled
	if !(_unit getVariable ['jac_pauseScriptedMove', false]) then	//wait if the movement is paused, i.e. for combat
	{
		[_unit] call jac_fnc_updateScriptedMoveAnim;
		_unit playMoveNow (_unit getVariable ['jac_scriptedMoveAnimAnim', '']); // loop animation
	};
}];
	
///EHs for combat mode
//damaged by enemy
_unit addEventHandler ["Hit",
{
	params ["_unit", "_source", "_damage", "_instigator"];		
	
	if (_unit getVariable ['jac_endScriptedMove', false]) exitWith { _unit removeEventHandler [_thisEvent, _thisEventHandler] }; // exit if movement is canceled
	
	_maxDis= 25; _weaponType= "non";
	_weapon= [(primaryWeapon _unit), (handgunWeapon _unit), (secondaryWeapon _unit)] findIf {_x == (currentWeapon _unit)};
	if ( !((currentWeapon _unit) == "") && !(_weapon == -1) ) then {_weaponType= ( ["rfl", "pst", "lnr"] # _weapon)};	//weapon type, in case no weapon or unknown weapon use default "non"
	
	switch (_weaponType) do {case "non"; case "lnr": {_maxDis= 1}; case "pst": {_maxDis= 25}};
	if ((_unit distance _instigator) < _maxDis)  then {_unit setVariable ['jac_targetScriptedMove', _instigator]};			
}];

/// scripted movement in combat mode helper - search for a target, pause movement while unit engages, resume movement after target is dead or fled
_grp= group _unit; _maxDis= 75; _weaponType= "non";
_weapon= [(primaryWeapon _unit), (handgunWeapon _unit), (secondaryWeapon _unit)] findIf {_x == (currentWeapon _unit)};
if ( !((currentWeapon _unit) == "") && !(_weapon == -1) ) then {_weaponType= ( ["rfl", "pst", "lnr"] # _weapon)};	//weapon type, in case no weapon or unknown weapon use default "non"
switch (_weaponType) do
{
	case "non"; case "lnr": {_maxDis= 1};
	case "pst": {_maxDis= 50};
};

_scriptedMoveCombat= [{
	(_this # 0) params ['_unit', '_grp', '_weaponType', '_maxDis'];
	[_unit, _grp, _weaponType, _maxDis] call jac_fnc_scriptedMoveCombat;
} , 0.1, [_unit, _grp, _weaponType, _maxDis]] call CBA_fnc_addPerFrameHandler;

/// scripted movement check for doors helper script
_scriptedMoveOpenDoors= [{
	(_this # 0) params ['_unit'];
	[_unit] call jac_fnc_scriptedMoveOpenDoors;
} , 0.5, [_unit]] call CBA_fnc_addPerFrameHandler;


/// scripted move routine
_unit setVariable ['jac_scriptedMoveStartNewWP', true];
_unit setVariable ['jac_scriptedMoveIndex', 0];
_unit setVariable ['jac_scriptedMovePos', (_path # 0 # 0)];
_unit setVariable ['jac_scriptedDis', 9999];
_pathSize= (count _path) -1;

_scriptedMoveHandle= 
	[{
		(_this # 0) params ['_unit', '_path', '_completionRadius', '_turnSpeedCoef', '_isPatrol', '_pathSize', '_scriptedMoveOpenDoors', '_scriptedMoveCombat'];

		if (_unit getVariable ['jac_scriptedMoveStartNewWP', true]) then
		{
			_index= _unit getVariable ['jac_scriptedMoveIndex', 0];
			_unit setVariable ['jac_scriptedMoveStartNewWP', false];
			_unit setVariable ['jac_scriptedMovePos', (_path # _index # 0)];
			_unit setVariable ['jac_scriptedMoveIndex', (_index) +1];
			
			//at the start of a new path point check for changes in animation direction (if using tactical movement)	[_pathPos, _isTacticalMov, _animationDirName, _facingDirMod]
			_scriptedMoveAnim= (_unit getVariable ['jac_scriptedMoveAnimAnimOG', '']);
			_isTacticalMov= (_path # _index # 1);
			_animationDirName= (_path # _index # 2);
			if (_isTacticalMov) then {_scriptedMoveAnim= ((_scriptedMoveAnim select [0, (count _scriptedMoveAnim -1)]) + _animationDirName)};
			
			_unit setVariable ['jac_scriptedMoveAnimAnim', _scriptedMoveAnim];
			_unit setVariable ['jac_scriptedMoveisTacticalMov', _isTacticalMov];			
			_unit setVariable ['jac_scriptedMoveanimationDirName', _animationDirName];
			_unit setVariable ['jac_scriptedMovefacingDirMod', (_path # _index # 3)];
			
			//swap to proper animation
			if ( (_index > 0)  && !(((animationState _unit)) isEqualTo toLower _scriptedMoveAnim) ) then {_unit playMoveNow _scriptedMoveAnim};
		};
		
		//turning routine
		if !(_unit getVariable ['jac_pauseScriptedMove', false]) then	 //when the unit pauses movement to engage a target
		{
			vectorDir _unit params ["_xDir", "_yDir"];
			_relDir= _unit getRelDir (_unit getVariable ['jac_scriptedMovePos', []]);
			_relDir= ((_relDir + (_unit getVariable ['jac_scriptedMovefacingDirMod', 0]) + 0) mod 360);		//for r or l +180; for br +90;  for lr +45
			
			_jump= 1;
			if ((_relDir >= 9) && (_relDir <= 351)) then {_jump= 9};
			_a = [_jump, -_jump] select (_relDir < 180);
			_unit setVectorDir [cos _a * _xDir - sin _a * _yDir, sin _a * _xDir + cos _a * _yDir, 0];
		};
		
		//reached waypoint
		_dis= (_unit distance2D (_unit getVariable ['jac_scriptedMovePos', [0,0,0]]));
		if (_dis <= _completionRadius) then
		{
			_unit setVariable ['jac_scriptedMoveStartNewWP', true];
			//reached last position
			if ((_unit getVariable ['jac_scriptedMoveIndex', 0]) > _pathSize) then
			{
				//if is not a patrol loop, stop moving at the last path pos
				if !(_unit getVariable ['jac_isScriptedPatrol', false]) then {_unit setVariable ['jac_endScriptedMove', true, true]} else {_unit setVariable ['jac_scriptedMoveIndex', 0]};
			};
		};
		
		// EXIT if movement is canceled
		if ( (_unit getVariable ['jac_endScriptedMove', false]) or !(alive _unit) ) exitWith
		{
			//damage nearest window
			[getPosATL _unit] call jac_fnc_findNearestWindow params ["_selectedWindowPos", "_closestDis", "_building", "_WindowName"];
			if (_closestDis < 2.25) then {_building setHitPointDamage [_WindowName, 1]};
			
			//at the last position, check if it has cover nearby and order unit to take cover
			if (jac_isContextCoverSystemActive) then
			{
				_heading= [0,45,90,135,180,225,270,315]; //_heading is used to have control over the direction cover will be searched
				_coverPosDir= [(getPosASL _unit), 2.5, _heading] call jac_fnc_findCoverFromPos;
				if (count _coverPosDir > 0) then {[_unit, (getPosASL _unit), (_coverPosDir # 2)] call jac_scriptedMoveTakeCover};
			};
			
			//non leader units will doStop if it's the last position
			if (leader _unit != _unit) then { if (_unit distance2D (_path # _pathSize # 0) < 3) then {doStop _unit} };// else {(group _unit) setFormDir (getDir (leader _unit))};						
			
			//end scripted movement
			_unit setVariable ['jac_isScriptedPatrol', false, true];
			_unit enableAI "MOVE"; _unit enableAI "ANIM";
			_unit switchMove (_unit getVariable ['jac_scriptedMoveEndAnim', '']); _unit playActionNow (_unit getVariable ['jac_scriptedMoveEndAnim', '']);
			_unit doWatch objNull;
			
			//end perframe EHs
			[_scriptedMoveOpenDoors] call CBA_fnc_removePerFrameHandler; [_scriptedMoveCombat] call CBA_fnc_removePerFrameHandler; [(_this # 1)] call CBA_fnc_removePerFrameHandler;
		};
	} , 0.0001, [_unit, _path, _completionRadius, _turnSpeedCoef, _isPatrol, _pathSize, _scriptedMoveOpenDoors, _scriptedMoveCombat]] call CBA_fnc_addPerFrameHandler;
};


///check if animation should change based on formation speed, stance, if unit legs are damaged, going up a hill
jac_fnc_updateScriptedMoveAnim=
{
	params ["_unit"];
	_doUpdateAnim= false;
	_scriptedMoveDataNew= [(speedMode (group _unit)), /*(stance _unit),*/ (((_unit getHit "legs") > 0.5) or (isForcedWalk _unit)), ((group _unit) getVariable ['jac_actionMode', "combatMode"]), ((surfaceNormal (getPosASL _unit)) # 2 < 0.8)];
	{ if !(_x isEqualTo (_scriptedMoveDataNew # _forEachIndex)) then {_doUpdateAnim= true; break} }forEach (_unit getVariable ['jac_scriptedMoveData',[]]);
	
	if (_doUpdateAnim) then
	{
		[_unit] call jac_fnc_scriptedMoveGetAnimationName params ["_scriptedMoveAnim", "_scriptedMoveEndAnim"];
		
		if (_unit getVariable ['jac_scriptedMoveisTacticalMov', false]) then
		{
			_animationDirName= (_unit getVariable ['jac_scriptedMoveanimationDirName', 'f']);
			_scriptedMoveAnim= ((_scriptedMoveAnim select [0, (count _scriptedMoveAnim -1)]) + _animationDirName);
			_scriptedMoveEndAnim= _scriptedMoveAnim;
		};

		_unit setVariable ['jac_scriptedMoveAnimAnim', _scriptedMoveAnim];
		_unit setVariable ['jac_scriptedMoveEndAnim', _scriptedMoveEndAnim];
		_unit setVariable ['jac_scriptedMoveData', _scriptedMoveDataNew];
	};
};

///scripted movement check for doors helper script
jac_fnc_scriptedMoveOpenDoors=
{
	params ['_unit'];
	_isDoor= [getPosATL _unit] call jac_fnc_findNearestDoor;
	if  ( ((_isDoor # 0 # 0) != -1) && ((_isDoor # 1) <= 2.5) ) then 
	{
		(_isDoor # 2) animate [(format ['%1_rot', (_isDoor # 3)]), 1];
		(_isDoor # 2) animate [(format ['%1A_move', (_isDoor # 3)]), 1];
		(_isDoor # 2) animate [(format ['%1B_move', (_isDoor # 3)]), 1];
	};	
};


////scripted movement in combat mode helper
jac_fnc_scriptedMoveCombat=
{
params ['_unit', '_grp', '_weaponType', '_maxDis'];

//only if in combat mode and not suppressing
if ( ((_grp getVariable ['jac_actionMode', "combatMode"]) isEqualTo 'combatMode') && !(_grp getVariable ['jac_doSuppressAction', false]) && isNull(_unit getVariable ['jac_targetScriptedMove', objNull]) ) then {

_targetUnit= objNull;

///make sure has line of sight to the aimPos of the target
_foundTarget = (_unit getVariable ['jac_myCurrentTarget', objNull]);
if !(_foundTarget isEqualTo objNull) then 
{
	if ( (_foundTarget isKindOf "CAManBase") && ((_foundTarget distance _unit) < _maxDis) ) then 
	{
		if (count (lineIntersectsSurfaces [(eyePos _unit), (aimPos _foundTarget), _unit, _foundTarget, true, 1, 'GEOM', "VIEW", true]) == 0) then {_targetUnit= _foundTarget};
	};
};

//halt scripted movement to engage until target is dead or out of sight, then resume movement
if (alive _targetUnit) then
{
	_unit setVariable ['jac_targetScriptedMove', _targetUnit];
	_unit setVariable ['jac_pauseScriptedMove', true];	//the unit pauses scripted movement to engage the target

	_unit enableAI "MOVE"; _unit enableAI "ANIM";
	//_unit switchMove ""; _unit playMoveNow "";

	//wait until unit resumes scripted movement
	[{
		(_this # 0) params ['_unit', '_targetUnit', '_maxDis'];
		
		_unit doWatch (vehicle _targetUnit); _unit doTarget (vehicle _targetUnit); _unit doFire (vehicle _targetUnit);
		_collisionLine= (lineIntersectsSurfaces [(eyePos _unit), (aimPos _targetUnit), _unit, _targetUnit, true, 1, 'GEOM', "VIEW", true]);		///make sure still has line of sight to the aimPos of the target
		
		if ( !(alive _targetUnit) or !(alive _unit) or !(count ( _collisionLine ) == 0) or ((_unit distance _targetUnit) > _maxDis) ) exitWith
		{
			_unit setVariable ['jac_pauseScriptedMove', false];
			_unit setVariable ['jac_targetScriptedMove', objNull];
			
			if (alive _unit) then
			{
				//resume loop animation
				_unit disableAI "MOVE"; _unit disableAI "ANIM";
				[_unit] call jac_fnc_updateScriptedMoveAnim;
				_unit playMoveNow (_unit getVariable ['jac_scriptedMoveAnimAnim', '']);
			};
			[(_this # 1)] call CBA_fnc_removePerFrameHandler;
		};
	}, 1,  [_unit, _targetUnit, _maxDis]] call CBA_fnc_addPerFrameHandler;
};

};//only if in combat mode and not suppressing

};


//// move aircraft or projectile following the given path
// [_pathPos, _isTacticalMov, _animationDirName, _facingDirMod];
// recommended _minDistanceToTarget= (_speed *0.1); 
jac_fnc_ScriptedMoveProjectilePath=
{
	params ["_unit", "_path", "_speed", "_minDistanceToTarget", "_destroyProjectile"];
	
	if !(local _unit) exitWith {false};
	
	_pathSize= count _path -1;
	_unit setVariable ['jac_scriptedMoveStartNewWP', true];
	_unit setVariable ['jac_scriptedMoveIndex', 0];
	(driver _unit) setVariable ['jac_endScriptedMove', false, true];
	
	//face direction
	_currentPos = getPosASLVisual _unit;
	_targetPos = _unit getVariable ["jac_scriptedMovePos", []];
	private _forwardVector = vectorNormalized (_targetPos vectorDiff _currentPos);
	private _rightVector = (_forwardVector vectorCrossProduct [0,0,1]) vectorMultiply -1;
	private _upVector = _forwardVector vectorCrossProduct _rightVector;
	_unit setVectorDirAndUp [_forwardVector, _upVector];

	// Loop
	_handler= [{
		(_this # 0) params ["_unit", "_path", "_speed", "_pathSize", "_minDistanceToTarget", "_destroyProjectile"];
		
		// end loop
		_loop= true;
		if ( ((driver _unit) getVariable ['jac_endScriptedMove', false]) or !(alive _unit) ) exitWith
		{
			_loop= false;
			_handler= (_this # 1);
			[{
				params ['_unit', '_destroyProjectile', '_handler'];
				if (_destroyProjectile) then {_unit setDamage 1};
				(driver _unit) setVariable ['jac_endScriptedMove', false, true];
				[_handler] call CBA_fnc_removePerFrameHandler;
			}, [_unit, _destroyProjectile, _handler], 0.25] call CBA_fnc_waitAndExecute;
		};
		
		if (_loop) then
		{
			//get a new path pos
			if (_unit getVariable ['jac_scriptedMoveStartNewWP', true]) then
			{
				private _index= _unit getVariable ['jac_scriptedMoveIndex', 0];
				_unit setVariable ['jac_scriptedMoveStartNewWP', false];
				_unit setVariable ['jac_scriptedMovePos', (AGLToASL (_path # _index # 0))];
				_unit setVariable ['jac_scriptedMoveIndex', (_index) +1];
			
				_currentPos = getPosASLVisual _unit;
				_targetPos = _unit getVariable ["jac_scriptedMovePos", []];
				private _forwardVector = vectorNormalized (_targetPos vectorDiff _currentPos);
				private _rightVector = (_forwardVector vectorCrossProduct [0,0,1]) vectorMultiply -1;
				private _upVector = _forwardVector vectorCrossProduct _rightVector;
				_unit setVectorDirAndUp [_forwardVector, _upVector];
			};
		
			//check if reached the next step
			_currentPos = getPosASLVisual _unit;
			_targetPos = _unit getVariable ["jac_scriptedMovePos", []];
		
			if (((ASLToAGL _currentPos) distance (ASLToAGL _targetPos)) <= _minDistanceToTarget) then
			{
				_unit setVariable ['jac_scriptedMoveStartNewWP', true];
	
				//reached last position
				if ((_unit getVariable ['jac_scriptedMoveIndex', 0]) > _pathSize) then
				{
					//if is not a patrol loop, stop moving at the last path pos
					if !(_unit getVariable ['jac_isScriptedPatrol', false]) then {(driver _unit) setVariable ['jac_endScriptedMove', true, true]} else {_unit setVariable ['jac_scriptedMoveIndex', 0]};
				
					private _forwardVector = vectorNormalized (_targetPos vectorDiff _currentPos);
					private _rightVector = (_forwardVector vectorCrossProduct [0,0,1]) vectorMultiply -1;
					private _upVector = _forwardVector vectorCrossProduct _rightVector;
					_unit setVectorDirAndUp [_forwardVector, _upVector];
				
					private _targetVelocity = _forwardVector vectorMultiply _speed;
					_unit setVelocity _targetVelocity;
					_unit domove (ASLToAGL _targetPos);
				};
			};
		
			//movement routine
			if !(_unit getVariable ['jac_pauseScriptedMove', false]) then
			{
				private _forwardVector = vectorNormalized (_targetPos vectorDiff _currentPos);
				private _targetVelocity = _forwardVector vectorMultiply _speed;
				_unit setVelocity _targetVelocity;
			};
		};
	} , 0.01, [_unit, _path, _speed, _pathSize, _minDistanceToTarget, _destroyProjectile]] call CBA_fnc_addPerFrameHandler;
};


//// move aircraft or projectile towards the given target
jac_fnc_ScriptedMoveProjectileTarget=
{
	params ["_unit", "_target", "_speed", "_minDistanceToTarget", "_destroyProjectile"];
	
	if !(local _unit) exitWith {false};
	(driver _unit) setVariable ['jac_endScriptedMove', false, true];
	
	_currentPos = getPosASLVisual _unit;
	_targetPos = aimPos _target;
	private _forwardVector = vectorNormalized (_targetPos vectorDiff _currentPos);
	private _rightVector = (_forwardVector vectorCrossProduct [0,0,1]) vectorMultiply -1;
	private _upVector = _forwardVector vectorCrossProduct _rightVector;
	_unit setVectorDirAndUp [_forwardVector, _upVector];

	// Loop
	_handler= [{
		(_this # 0) params ["_unit", "_target", "_speed", "_minDistanceToTarget", "_destroyProjectile"];

		// end loop
		_loop= true;
		if ( !(alive _unit) or !(alive _target) or ((driver _unit) getVariable ['jac_endScriptedMove', false]) ) exitWith
		{
			_loop= false;
			_handler= (_this # 1);
			[{
				params ['_unit', '_destroyProjectile', '_handler'];
				if (_destroyProjectile) then {_unit setDamage 1};
				(driver _unit) setVariable ['jac_endScriptedMove', false, true];
				[_handler] call CBA_fnc_removePerFrameHandler;
			}, [_unit, _destroyProjectile, _handler], 0.25] call CBA_fnc_waitAndExecute;
		};
		
		if (_loop) then
		{
			//check if reached the target
			_currentPos = getPosASLVisual _unit;
			_targetPos = aimPos _target;
			if (((ASLToAGL _currentPos) distance (ASLToAGL _targetPos)) <= _minDistanceToTarget) then 
			{
				(driver _unit) setVariable ['jac_endScriptedMove', true, true];

				private _forwardVector = vectorNormalized (_targetPos vectorDiff _currentPos);
				private _rightVector = (_forwardVector vectorCrossProduct [0,0,1]) vectorMultiply -1;
				private _upVector = _forwardVector vectorCrossProduct _rightVector;
				_unit setVectorDirAndUp [_forwardVector, _upVector];
			
				private _targetVelocity = _forwardVector vectorMultiply _speed;
				_unit setVelocity _targetVelocity;
				_unit domove (ASLToAGL _targetPos);
			};
		
			//movement routine
			if !(_unit getVariable ['jac_pauseScriptedMove', false]) then
			{
				private _forwardVector = vectorNormalized (_targetPos vectorDiff _currentPos);
				private _targetVelocity = _forwardVector vectorMultiply _speed;
				_unit setVelocity _targetVelocity;
			};
		};
	} , 0.01, [_unit, _target, _speed, _minDistanceToTarget, _destroyProjectile]] call CBA_fnc_addPerFrameHandler;
};



/////// FORTIFICATION MODE - ENGINEER functions
///
//name, size, classname, duration, cost, rotation
jac_fortificationDataArray= [
	['barbed wire', 8, 'Land_Razorwire_F', 15, 10, 90]
	,['tank trap', 3.25, 'Land_CzechHedgehog_01_F', 45, 25, 0]
	,['sandbag', 2.9, 'Land_BagFence_01_long_green_F', 45, 20, 90]
	,['camouflage net', 18, 'CamoNet_BLUFOR_big_F', 60, 30, 180]
	,['small bunker', 8, 'Land_BagBunker_01_small_green_F', 90, 50, 180]
	,['large bunker', 12, 'Land_BagBunker_01_large_green_F', 180, 100, 180]
	,['ammo box', 1.5, 'VirtualReammoBox_small_F', 30, 100, 180]
	//,['bridge', 20.25, 'BridgeWooden_01', 20, 75, 180]
];

jac_fortificationDataArrayHardcoded= [
	['barbed wire', 8, 'Land_Razorwire_F', 15, 10, 90]
	,['tank trap', 3.25, 'Land_CzechHedgehog_01_F', 45, 25, 0]
	,['sandbag', 2.9, 'Land_BagFence_01_long_green_F', 45, 20, 90]
	,['camouflage net', 18, 'CamoNet_BLUFOR_big_F', 60, 30, 180]
	,['small bunker', 8, 'Land_BagBunker_01_small_green_F', 90, 50, 180]
	,['large bunker', 12, 'Land_BagBunker_01_large_green_F', 180, 100, 180]
	,['ammo box', 1.5, 'VirtualReammoBox_small_F', 30, 100, 180]
];


/// call main building function, determines what to build and set multiple WPs in the proper interval from the first point to the last
jac_fnc_buildingMain=
{
params ['_grp', '_pos1', '_pos2', '_buildingOption'];

_distance= (_pos1 distance _pos2);
_dir= (_pos1 getDir _pos2);
_buildArguments=[];
_dataArray= jac_fortificationDataArray findIf { ((_x # 0) == _buildingOption) };

///minefield
switch (true) do
{
	//dig in
	case (_buildingOption == "dig in"):
	{
		_interval= 0.75;	//2.75
		_numberToBuild= (round(_distance/_interval) max 1);
		_buildArguments= [1.75, _dir];	//_depth, _dir
		for "_i" from 0 to (_numberToBuild -1) step +1 do
		{
			_distance= (_interval * _i);
			_pos= AGLToASL (_pos1 getPos [_distance, _dir]);
			
			[_grp, _pos, _buildingOption, _buildArguments] remoteExec ["jac_fnc_addDiginWP", 0]; sleep 0.1;
		};
	};
	
	//field fortifications
	case (_dataArray != -1):
	{
		_myCase= (jac_fortificationDataArray # _dataArray);
		_interval= (_myCase # 1);
		_type= (_myCase # 2);
		_duration= (_myCase # 3); if ((jac_fortificationModeCheat) && (jac_canUseWargameCheatsSetting)) then {_duration= 0};
		_cost= (_myCase # 4);
		_rot= (_myCase # 5);
		_numberToBuild= (round(_distance/_interval) max 1);
		[_numberToBuild, _cost, _interval, _pos1, _dir, _duration, _grp, _type, _rot] spawn jac_fnc_forAddbuildWP; sleep 0.1;	//_numberToBuild, _cost, _interval, _pos1, _dir, _duration, _grp, _type _rot// ['sandbag', 3, 'Land_BagFence_01_long_green_F', 6, 20, 90]
	};
	
	//place explosive
	default
	{
		_interval= 4;
		_numberToBuild= (round(_distance/_interval) max 1);
		_explosiveList=[]; { _explosiveList append ((magazines _x) select {_x == _buildingOption}) }forEach (units _grp);
		_explosivesNumber= count _explosiveList; if ((jac_fortificationModeCheat) && (jac_canUseWargameCheatsSetting)) then {_explosivesNumber= 999};
		for "_i" from 0 to (_numberToBuild -1) step +1 do
		{			
			if ( ((_explosivesNumber) > _i) ) then
			{
				_distance= (_interval * _i);
				_pos= AGLToASL (_pos1 getPos [_distance, _dir]);
				
				[_grp, _pos, _buildingOption] remoteExec ["jac_fnc_addExplosiveWP", 0]; sleep 0.1;
			};
		};
	};
};
};


///add the waypoints in sequence, check if there are enough resources
jac_fnc_forAddbuildWP=
{
params ["_numberToBuild", "_cost", "_interval", "_pos1", "_dir", "_duration", "_grp", "_type", "_rot"];
for "_i" from 0 to (_numberToBuild -1) step +1 do
{
	_material= [_grp] call jac_fnc_getFortificationPointsGroup;
	if ( ((_material - _cost) >= 0) or ((jac_fortificationModeCheat) && (jac_canUseWargameCheatsSetting)) ) then
	{
		_distance= (_interval * _i);		
		_pos= AGLToASL (_pos1 getPos [_distance, _dir]);
		_buildArguments= [_duration, _dir, _cost, (_interval*0.5), _rot];	//_duration, direction, _offset, _cost, _rot
		
		[_grp, _pos, _type, _buildArguments] remoteExec ["jac_fnc_addbuildWP", 0];
		sleep 0.1;
	}else
	{titleText ["NO RESOURCES TO BUILD", "PLAIN"]; exit};
};
};

//// add new waypoint and set statements
jac_fnc_addbuildWP=
{
	params ["_grp", "_pos", "_type", "_buildArguments"];
	if !(local _grp) exitWith {false};
	
	(units _grp) doFollow (leader _grp);	// to make unit move in case it's in doStop mod
	{if (!(_grp getVariable ['jac_isDugIn', false]) && (_x getVariable ['jac_isInCover', false]) ) then {_x enableAi 'PATH'}}forEach (units _grp);	//enable path if in cover, but not dug in
	
	_wp = _grp addWaypoint [_pos, -1, (count (waypoints  _grp))+1, ""];
	_wp setWaypointName _type;	//type
	_wp setWaypointDescription str (_buildArguments);
	_wp setWaypointStatements ["true",jac_buildWPStatementString];
};

jac_buildWPStatementString=
"
_leader=this;
if !(local _leader) exitWith {false};

_grp= (group _leader);
_wp= [_grp, (currentWaypoint _grp)];

_array= parseSimpleArray (waypointDescription _wp);
[(getWPPos _wp), (_array # 0), (_array # 1), (_array # 2), (_array # 3), (waypointName _wp), _leader, _grp, (_array # 4)] spawn jac_fnc_buildWPStatement;
";

jac_fnc_buildWPStatement=
{
	params ['_pos', '_duration', '_dir', '_cost', '_offset', '_type', '_leader', '_grp', '_rot'];
	_material= [_grp] call jac_fnc_getFortificationPointsGroup;
	if ( ((_material - _cost) >= 0) or ((jac_fortificationModeCheat) && (jac_canUseWargameCheatsSetting)) ) then
	{
		if (!(jac_fortificationModeCheat) or !(jac_canUseWargameCheatsSetting)) then {[_grp, _cost] call jac_fnc_useFortificationPoints; _leader playActionNow 'MedicOther'};
		[250, _leader, "building..."] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0];
		
		_leader setVariable ["jac_unitIsOccupied", true, true];
		_leader disableAI "MOVE";
		_doBuild= true;
		if (_duration > 0) then	
		{
			for '_i' from (_duration max 1) to 0 step -1 do
			{
				sleep 1;
				if ( !(alive _leader) or ((getPosASL _leader) distance (AGLToASL _pos) > 2.5) or (_grp getVariable ["jac_isCancelAction", false]) ) exitWith {_doBuild= false; break};
				if ( (_i > 6 ) && (!(jac_fortificationModeCheat) or !(jac_canUseWargameCheatsSetting)) ) then {_leader playAction 'MedicOther'};
				[100, _leader, (str _i + ' sec.')] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0];
			};
		};
		_leader enableAI "MOVE";
		
		if (_doBuild) then
		{
			_leader setVariable ["jac_unitIsOccupied", false, true];
			_myObj= objNull;
			if !(_type == 'BridgeWooden_01') then {_myObj = createVehicle [_type, [1,1,1], [], 0, "CAN_COLLIDE"]}
			else
			{_myObj= createSimpleObject [_type, [1,1,1], false]};
			sleep 0.1;
		
			//[_myObj, (side _leader)] remoteExec ["jac_fnc_setIsFogOfWarUnit", 0, true];	/// if the given entity should be added to the fog of war check of each player's locality
		
			_placePos= AGLToASL (_pos getPos [_offset, _dir]);
			_myObj setDir ((_dir + _rot) mod 360);
		
			if !(_type == 'BridgeWooden_01') then //handle bridge exception
			{
				_myObj setPosASL _placePos;
				_myObj setVectorUp (surfaceNormal (position _myObj));
			}
			else
			{
				_height = (getTerrainHeightASL (getPosASL _leader));
				_placePos set [2, (_height - 3.8)];
				_myObj setPosASL _placePos;
			};
			
			//ammo box
			if (_type in jac_supplyObjList) then
			{
				[[_myObj], 100, 150, -4, false] remoteExec ["jac_fn_initSupplyPoint", 0];
				_zeusList= [_leader] call jac_fnc_objectCurators;
				{[_x, [[_myObj], true]] remoteExec ["addCuratorEditableObjects", 2]}forEach _zeusList;
			};
		};
	}
	else
	{[250, _leader, "NO RESOURCES TO BUILD!"] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0]};
};


////// PLACE EXPLOSIVE wp
//
wp_placeExplosive="
if !(local this) exitWith {false};
_my_leader=this;
[_my_leader] spawn { params ['_my_leader'];	
	_grp= group _my_leader;
	_target_g = (getWPPos [_grp, (currentWaypoint _grp)]); _explosiveClass= waypointName [_grp, (currentWaypoint _grp)];
	_my_leaderLoadout= getUnitLoadout _my_leader; _restoreMinusOne= true;

	if ( ((magazines _my_leader) find _explosiveClass) == -1 ) then
	{
		_restoreMinusOne= false;
		{
			if ( ((magazines _x) find _explosiveClass) != -1 ) then { if (!(jac_fortificationModeCheat) or !(jac_canUseWargameCheatsSetting)) then {_x removeItem _explosiveClass}; removeAllItemsWithMagazines _my_leader; sleep 0.1; _my_leader addItem _explosiveClass; break};
		}forEach (units _grp);
	};
	_my_leader disableAI 'MOVE';
	sleep 1; _index = ((magazines _my_leader) find _explosiveClass);
	if (_index != -1) then
	{
		_magazineDet = ((magazinesDetail _my_leader) # _index);
		_splitStr= _magazineDet splitString '([ ]/:)';
		_id= parseNumber (_splitStr # ((count _splitStr) - 2)); _owner= parseNumber (_splitStr # ((count _splitStr) - 1));
		CBA_logic action ['UseMagazine', _my_leader, _my_leader, _owner, _id];		
		[225, _my_leader, format ['placing %1', (getText (configFile >> 'CfgMagazines' >> _explosiveClass >> 'displayName'))]] remoteExecCall ['jac_fnc_zeusAddMSG3D', 0];
		_my_leader playAction 'PutDown';
		_my_leader setUnitLoadout _my_leaderLoadout; sleep 0.1; if ((_restoreMinusOne) && (!(jac_fortificationModeCheat) or !(jac_canUseWargameCheatsSetting)) ) then {_my_leader removeItem _explosiveClass};
	};
	
	_my_leader enableAI 'MOVE';
	sleep 0.25;
	_ammo= (getText (configFile >> 'CfgMagazines' >> _explosiveClass >> 'ammo'));
	if ( (getText (configFile >> 'CfgAmmo' >> _ammo >> 'mineTrigger')) == 'RemoteTrigger' ) then 
	{
		_mines= (nearestMines [(position _my_leader), [], 2.5]);
		if ((count _mines) > 0) then
		{
			_charge= (_mines select {!(_x getVariable ['jac_hasExplosiveHandler', false])} ) # 0;
			if !(isNull _charge) then
			{
				_colPos= []; _colObj= objNull;
				_objects = nearestObjects [(position _my_leader), ['LandVehicle', 'Air', 'Ship', 'House'], 8] select {( (alive _x) && !(isObjectHidden _x) )};

				if (count _objects > 0) then
				{
					_findIfId= _objects findIf {
						_collision_lineArray= lineIntersectsSurfaces [(eyePos _my_leader), (getPosASL _x) vectorAdd [0, 0, 0.5], _my_leader, vehicle _my_leader, true, 1, 'VIEW', 'NONE', true];
						if (count _collision_lineArray !=0) then 
						{					
							if (((_collision_lineArray # 0) # 2) isEqualTo _x) exitWith {_colPos= (_collision_lineArray # 0) # 0; _colObj= (_collision_lineArray # 0) # 2};
						};
						false;
					};
				};

				if !(isNull _colObj) then { sleep 0.1; _charge setPosASL _colPos; _charge setVectorUp surfaceNormal position _colObj; sleep 0.1; _charge attachTo [_colObj]};
				
				_charge setVariable ['jac_hasExplosiveHandler', true];
				_handlerObj= createagent ['jac_wargameLogic',[0,0,0],[],0,'CAN_COLLIDE']; sleep 0.1;
				_handlerObj setPosATL ((getPosATL  _charge) vectorAdd [0,0,3]);
				[_handlerObj, _charge, _my_leader] call jac_fnc_detonateRemoteExplosiveInit;
				_handlerObj attachTo [_charge];
			};
		};
	};
};
";

//// add menu option to activate an explosive
jac_fnc_detonateRemoteExplosiveInit=
{
	params ["_obj", "_charge", "_unit"];
	_optionData= _obj getVariable ['jac_interactiveMenuArray', []];	
	_optionData pushBack ["DETONATE", "jac_fnc_detonateRemoteExplosive", "Detonate the explosive, the group that placed the explosive must be within radio range.", 0, _charge];
	_obj setVariable ['jac_interactiveMenuArray', _optionData, true];
	_obj setVariable ['jac_detonateRemoteExplosiveParent', _unit, true];
	
	_zeusList= [_unit] call jac_fnc_objectCurators;
	[[_obj, _zeusList],
	{
		params ["_obj", "_zeusList"];
		{_x addCuratorEditableObjects [[_obj],true]}forEach _zeusList;
	}] remoteExecCall ['call', 2];
	
	//add icon to the logic object
	_obj setVariable ["jac_my3dIcon", "\a3\ui_f\data\igui\cfg\simpletasks\types\mine_ca.paa", true];
	jac_logicIconList pushBack _obj;
	
	//delete logic after explosive is used
	[_obj, _charge, _unit] spawn {
		params ["_obj", "_charge", "_unit"]; waitUntil { sleep 0.1; ([_obj, _charge, _unit] findIf {!(alive _x)} != -1) };
		deleteVehicle _obj;
	};//spawn
};

//// activate the explosive
jac_fnc_detonateRemoteExplosive=
{
	params ["_ctrlPos", "_ctrlOBJ", "_grpArray", "_charge"];
	if !(local _charge) exitWith {false};
	_unit= _ctrlOBJ getVariable ['jac_detonateRemoteExplosiveParent', objNull];
	
	if ((_unit distance _ctrlOBJ) > ((group _unit) getVariable ["jac_my_radio_range", 500])) exitWith {[225, _ctrlOBJ, "DETONATOR OUT OF RADIO RANGE!"] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0]};
	_charge setDamage 1;
};

/// repair vehicles waypoint
jac_fnc_addRepairWP=
{
	params ["_grp", "_targetPos", "_side"];
	if !(local _grp) exitWith {false};

	_units= (units _grp);
	_units doFollow (leader _grp);
	{if (!(_grp getVariable ['jac_isDugIn', false]) && (_x getVariable ['jac_isInCover', false]) ) then {_x enableAi 'PATH'}}forEach _units;	//enable path if in cover, but not dug in
	
	_wp = _grp addWaypoint [_targetPos, -1, (count (waypoints  _grp))+1, "repair"];
	_wp setWaypointType "SCRIPTED";
	_wp setWaypointStatements [wp_repairStatement, ""];
};

wp_repairStatement= "
	_my_grp=(group this);
	if !(local _my_grp) exitWith {false};

	_wp= (currentWaypoint _my_grp);
	_target_g= getWPPos [_my_grp,(currentWaypoint _my_grp)]; 

	if ( (this distance2D (_target_g) <= 12) && ( (waypointDescription [_my_grp, _wp]) != 'wp_repairStatementActivated') ) then 
	{
		[_my_grp, _wp] setWaypointDescription 'wp_repairStatementActivated';
		[this, _target_g, _my_grp, _wp] spawn jac_fnc_repairWPStatement;
	};
";

jac_fnc_repairWPStatement=
{
	params ['_unit', '_pos', '_grp', '_wpNumber'];
	[300, _unit, 'setting up repair station'] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0];
	
	// add icon to all friendly zeus
	_obj= createagent ['jac_wargameLogic',[0,0,0],[],0,'CAN_COLLIDE']; sleep 0.25;
	_obj setPosASL ((AGLToASL _pos) vectorAdd [0,0,15]);
	
	_side= (side _unit); 
	_sideId= "n"; _sideColor= [0.6,0.1,0.95,0.9];     //to set the correct icon
	switch (_side) do {
		case blufor: {_sideId= "b"; _sideColor= [0.5,0.8,0.95,0.9]};
		case opfor: {_sideId= "o"; _sideColor= [1,0,0,0.9]};
		case independent: {_sideColor= [0,1,0,0.9]};
	};
	
	_obj setVariable ["jac_my3dIconColor", _sideColor, true];
	_obj setVariable ["jac_my3dIconText", "Repair Station", true];
	_icon= format ["\A3\ui_f\data\map\markers\nato\%1_service.paa", _sideId];
	_obj setVariable ["jac_my3dIcon", _icon, true]; jac_logicIconList pushBack _obj;
	
	_zeusList1= allCurators select {side (getAssignedCuratorUnit _x) == _side};
	_zeusList2= [_unit] call jac_fnc_objectCurators;
	_zeusList= _zeusList1 arrayIntersect _zeusList2;
	[[_obj, _zeusList],
	{
		params ["_obj", "_zeusList"];
		{_x addCuratorEditableObjects [[_obj],true]}forEach _zeusList;
	}] remoteExecCall ['call', 2];
	
	/// start repair routine
	_PARAMS= _this; _PARAMS pushBack _obj;
	[{
		(_this # 0) params ['_unit', '_pos', '_grp', '_wpNumber', '_obj'];
		
		//check if friendly vehicle in need of repair is nearby
		//if has los to it move to vehicle and start repair action
		
		_colPos= []; _colObj= objNull;
		_objects = nearestObjects [_pos, ['LandVehicle', 'Air', 'Ship'], 40] select {
		_vehicle= _x;
		( (alive _x) && !(isObjectHidden _x) && !(_x getVariable ["jac_isRepairingAction", false]) && ( (damage _x > 0) or (jac_vehicleMainHitpoints findIf { ((_vehicle getHitPointDamage (_x # 0)) > 0) } != -1) ) && ( ([(side _unit), side _x] call BIS_fnc_sideIsFriendly) or ({alive _x} count (crew (vehicle _x))  == 0) ) )};
		
		if (count _objects > 0) then
		{
			_findIfId= _objects findIf {
				_collision_lineArray= lineIntersectsSurfaces [(eyePos _unit), (getPosASL _x) vectorAdd [0, 0, 0.5], _unit, vehicle _unit, true, 1, 'VIEW', 'NONE', true];
				if (count _collision_lineArray !=0) then 
				{
					if (((_collision_lineArray # 0) # 2) isEqualTo _x) exitWith {_colPos= (_collision_lineArray # 0) # 0; _colObj= (_collision_lineArray # 0) # 2};
				};
				false;
			};
		};

		//if found a vehicle to repair
		if ( !(isNull _colObj) && !(_unit getVariable ["jac_isRepairingAction", false]) ) then
		{
			_unit setVariable ["jac_isRepairingAction", true, true];
			_colObj setVariable ["jac_isRepairingAction", true, true];
			[_unit, _colObj, _colPos, _pos] spawn
			{
				params ['_unit', '_colObj', '_colPos', '_pos'];
				
				_msg= format ["repair that %1", (getText (configOf _colObj >> "displayName"))];
				[300, _unit, _msg] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0];
				_path= [[(ASLToAGL _colPos), false, '', 0], [(ASLToAGL _colPos), false, '', 0]];
			
				if ((getPosASL _unit) distance _colPos > 2.25) then {[_unit, _path, jac_scriptedMoveSpacing, jac_scriptedMoveTurnSpd, false] remoteExecCall ["jac_fnc_scriptedMove", 0]};
				
				waitUntil {sleep 0.01; ( !(alive _unit) or !(alive _colObj) or ((getPosASL _unit) distance _colPos <= 2.25) or (_grp getVariable ["jac_isCancelAction", false]) )};
				_doRepair= true;
				if ( (alive _unit) && (alive _colObj) ) then
				{
					_unit setVariable ['jac_endScriptedMove', true, true]; sleep 0.5; //end scripted move
					_unit setDir (_unit getDir _colObj); doStop _unit;
					_unit disableAI "PATH";
					
					_damageN= 0;
					{
						if ((_colObj getHitPointDamage (_x # 0)) > 0) then {_damageN= _damageN +1};
					} forEach jac_vehicleMainHitpoints;
					
					_timer= (_damageN * 15) max 60;

					for '_i' from _timer to 0 step -1 do {
						if (_i >= 5) then {_unit playAction 'MedicOther'};
						sleep 1; _msg= format ["repairing %1 sec.", (str _i)];
						[100, _unit, _msg] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0];						
						if ( !(alive _unit) or !(alive _colObj) or ((getPosASL _unit) distance _colPos > 2.5) or (_grp getVariable ["jac_isCancelAction", false]) ) exitWith {_doRepair= false};
					};
					
					if (_doRepair) then
					{
						_colObj setDammage 0;
						[300, _unit, 'repair finished!'] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0];
					}
					else
					{[300, _unit, 'repair cancelled'] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0]};

					_unit switchMove ''; //reset animation to prevent getting stuck (happens when prone at the start)
					_unit enableAI "PATH"; _unit doFollow _unit;
					sleep 0.5; _unit doMove _pos;
				};
				_unit setVariable ["jac_isRepairingAction", false, true];
				_colObj setVariable ["jac_isRepairingAction", false, true];
			};//spawn
		};
		
		//if unit dies or waypoint is cancelled/moved
		if ( !(alive _unit) or !((getWPPos [_grp, (currentWaypoint _grp)]) isEqualTo _pos) or (_grp getVariable ["jac_isCancelAction", false]) ) then
		{
			[300, _unit, 'repair station cancelled'] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0];
			if ((waypointName [_grp, (currentWaypoint _grp)]) == "repair") then {deleteWaypoint [_grp, (currentWaypoint _grp)]};
			deleteVehicle _obj;
			[(_this # 1)] call CBA_fnc_removePerFrameHandler;
		};	
	} , 5, _PARAMS] call CBA_fnc_addPerFrameHandler;
};


////digging foxhole/trench WP
// add new waypoint and set statements

jac_fnc_addDigInWP=
{
	params ["_grp", "_pos", "_type", "_buildArguments"];
	if !(local _grp) exitWith {false};
	
	_units= (units _grp);
	_units doFollow (leader _grp);
	{if (!(_grp getVariable ['jac_isDugIn', false]) && (_x getVariable ['jac_isInCover', false]) ) then {_x enableAi 'PATH'}}forEach (units _grp);	//enable path if in cover, but not dug in
	
	_wp = _grp addWaypoint [_pos, -1, (count (waypoints  _grp))+1, ""];
	_wp setWaypointName _type;	//type
	_wp setWaypointDescription str (_buildArguments);
	_wp setWaypointStatements ["true",jac_digInWPStatementString];
};

jac_digInWPStatementString=
"
_leader=this;
if !(local _leader) exitWith {false};

_grp= (group _leader);
_wp= [_grp, (currentWaypoint _grp)];

_array= parseSimpleArray (waypointDescription _wp);
[(getWPPos _wp), (_array # 0), (_array # 1), _leader] spawn jac_fnc_digInWPStatement;
";

jac_fnc_digInWPStatement=
{
params ["_WPPos", "_depth", "_dir", "_leader"];		//depth in ~meters

_leader doFollow _leader; _leader disableAI "MOVE";
[200, _leader, "digging in..."] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0];

_step= 0.1;
_pos= (getPosATL _leader);
_heightD =[(getTerrainHeightASL _pos), (getTerrainHeight _pos)];
_height = selectMin _heightD;
_WPPosH= ((AGLToASL _WPPos) # 2);

_grassCutter= createvehicle ["Land_ClutterCutter_large_F", _WPPos,[],0,"CAN_COLLIDE"];
_doDig= true;

for "_i" from 1 to ceil(_depth/_step) step +1 do
{
	if ( !(alive _leader) or ((getPosASL _leader) distance (AGLToASL _WPPos) > 5) or ((group _leader) getVariable ["jac_isCancelAction", false]) ) exitWith {_doDig= false; break};

	_heightMod= (_step*_i);
	_newHeight= (_height - _heightMod);
	if ( (_newHeight > 0) && (_newHeight >= (_WPPosH - _depth)) ) then 
	{
		if (!(jac_fortificationModeCheat) or !(jac_canUseWargameCheatsSetting)) then {_leader playActionNow 'MedicOther'; sleep 0.5} else {sleep 0.1};
		
		_pos set [2, _newHeight];
		[[_pos],
		{
			params ["_pos"];
			setTerrainHeight [[_pos], false];
		}] remoteExecCall ["call", 2];	//has to be server exec		
	};
};

[{
	params ['_grassCutter'];
	deleteVehicle _grassCutter;
}, [_grassCutter], 300] call CBA_fnc_waitAndExecute;

_leader switchAction '';
_leader enableAI "MOVE";
};


//"breach obstacle" waypoint
//find obstacle objects in the WP position, move close and delete them
jac_fnc_addBreachObstacleWP=
{
	params ["_grp", "_targetPos"];
	if !(local _grp) exitWith {false};

	_units= (units _grp);
	_units doFollow (leader _grp);
	{if (!(_grp getVariable ['jac_isDugIn', false]) && (_x getVariable ['jac_isInCover', false]) ) then {_x enableAi 'PATH'}}forEach _units;	//enable path if in cover, but not dug in
	
	_wp = _grp addWaypoint [_targetPos, -1, (count (waypoints  _grp))+1, "breach obstacle"];
	_wp setWaypointStatements ["true",wp_breachObstacleStatement];
};

wp_breachObstacleStatement="
_my_leader=this;
_my_grp=(group _my_leader);
if !(local _my_grp) exitWith {false};

_wp= (currentWaypoint _my_grp);
_target_g= getWPPos [_my_grp,(currentWaypoint _my_grp)]; 

if ((this distance2D (_target_g) <= 12) && (waypointDescription [_my_grp, _wp] != 'wp_fn_breachObstacleStatementcondActivated')) then {
[_my_grp, _wp] setWaypointDescription 'wp_fn_breachObstacleStatementcondActivated';

  [_my_leader, _target_g, _my_grp, _wp] spawn {
  params ['_my_leader', '_target_g', '_my_grp', '_wp'];

  deleteWaypoint [_my_grp,_wp];
  _my_grp addWaypoint [getPosASL _my_leader, -1, -1, 'tempWP'];
  sleep 0.1;
    
  deleteWaypoint [_my_grp, (count waypoints _my_grp)-1];  
  [_my_leader, _target_g] spawn wp_fn_breachObstacleStatement;
};
};
";


wp_fn_breachObstacleStatement=
{
params ["_unit", "_pos"];
_breachableObjs= jac_standardBreachableObjs + jac_customBreachableObjs + (_unit getVariable ['jac_breachableObjs', []]);
_objList= (nearestObjects [_pos, _breachableObjs, 12, false]);

	if ((count _objList) > 0) then
	{
	_objId= _objList findIf {alive _x};
	if (_objId != -1) then 
	{
		_obj= (_objList # _objId);
		_unitPos= getPosASL _unit;
		_pos= getPosASL _obj;
		_collisionLine= ( (lineIntersectsSurfaces [(_unitPos vectorAdd [0,0,0.5]), (_pos vectorAdd [0,0,0.5]), _unit, _obj, true, 1, 'VIEW', 'NONE', true]) );
		if ( ( ( _unitPos distance _pos) < 12 ) && (count _collisionLine == 0) ) then
		{
			[200, _unit, 'Breaching obstacle...'] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0];
			_path= [[(ASLToAGL _pos), false, '', 0], [(ASLToAGL _pos), false, '', 0]];
			
			if ((getPosASL _unit) distance _pos > 3) then {[_unit, _path, jac_scriptedMoveSpacing, jac_scriptedMoveTurnSpd, false] remoteExecCall ["jac_fnc_scriptedMove", 0]};
			
			waitUntil {sleep 0.001; ( !(alive _unit) or ((getPosASL _unit) distance _pos < 3) )};
			
			if (alive _unit) then
			{
				_unit setVariable ['jac_endScriptedMove', true, true]; sleep 0.5; //end scripted move
				_unit setDir (_unit getDir _obj);
				
				_confirmBreachEnd= true;
				_timer= 6; if (_obj isKindOf "BagBunker_base_F") then {_timer= 45};
				for '_i' from _timer to 0 step -1 do {
					if (_i >= 5) then {_unit playAction 'MedicOther'};
					sleep 1; _msg= format ["breaching %1 sec.", (str _i)];
					[100, _unit, _msg] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0];
					if ( !(alive _unit) or !(alive _obj) or ((getPosASL _unit) distance (getPosASL _obj) > 8) or ((group _unit) getVariable ["jac_isCancelAction", false]) ) exitWith {_confirmBreachEnd= false; false};
				};
				
				if ((_confirmBreachEnd)) then
				{
					if ( ((typeOf _obj) isEqualTo 'Land_Razorwire_F') or ((typeOf _obj) isEqualTo 'CamoNet_BLUFOR_big_F') ) then {_obj setDammage 1} else {deleteVehicle _obj};
					if (alive _unit) then {_unit switchMove ''}; //reset animation to prevent getting stuck (happens when prone at the start)
				};
			};
		};
	};
	};
};


/// returns the number of fortification points the units in the group have combined
//returns -1 if there are no fortification points
jac_fnc_getFortificationPointsGroup=
{	
	params ["_grp"];
	_total= -1;
	{
		_points= _x getVariable ["jac_isEngineer", -1];
		if (_points >= 0) then 
		{
			if (_total == -1) then {_total= 0};
			_total= (_total + _points)
		};
	}forEach (units _grp);
	if ( (_total != -1) && (jac_fortificationModeCheat) && (jac_canUseWargameCheatsSetting) ) then {_total= 999};	//if cheat mode is on, engineers will always have points
	_total;
};

// removes this amount of fortification points from the group, returns if the group had the requested amount of points
jac_fnc_useFortificationPoints=
{
	params ["_grp", "_cost"];
	_points= [_grp] call jac_fnc_getFortificationPointsGroup;	
	_result= (_points - _cost);
	if (_points >= _cost) then 	//if the group has the points needed - take points from each unit until the needed amount is reached
	{
		{
			_unitPoints= _x getVariable ["jac_isEngineer", -1];
			if (_unitPoints > 0) then 
			{				
				_x setVariable ["jac_isEngineer", ((_unitPoints - _cost) max 0), true];	//how many points were left for this unit
				_cost= (_cost - _unitPoints);
			};
			if (_cost <= 0) exitWith {true};	//if collected the amount of points necessary
		}forEach (units _grp);
	};
	_result;
};



//////////////// HELPER FUNCTIONS

//helper function to get info about a vehicle weapons 
//* credits to the wiki member DenVdmj *//
// "M1A2_US_TUSK_MG_EP1" call jac_funcGetTurretsWeapons; // (typeOf _veh)
jac_funcGetTurretsWeapons = {
	private ["_result", "_getAnyMagazines", "_findRecurse", "_class"];
	_result = [];
	_getAnyMagazines = {
		private ["_weapon", "_mags"];
		_weapon = configFile >> "CfgWeapons" >> _this;
		_mags = [];
		{
			_mags = _mags + getArray (
				(if (_x == "this") then { _weapon } else { _weapon >> _x }) >> "magazines"
			)
		} forEach getArray (_weapon >> "muzzles");
		_mags
	};
	_findRecurse = {
		private ["_root", "_class", "_path", "_currentPath"];
		_root = (_this select 0);
		_path = +(_this select 1);
		for "_i" from 0 to count _root -1 do {
			_class = _root select _i;
			if (isClass _class) then {
				_currentPath = _path + [_i];
				{
					_result set [count _result, [_x, _x call _getAnyMagazines, _currentPath, str _class]];
				} forEach getArray (_class >> "weapons");
				_class = _class >> "turrets";
				if (isClass _class) then {
					[_class, _currentPath] call _findRecurse;
				};
			};
		};
	};
	_class = (
		configFile >> "CfgVehicles" >> (
			switch (typeName _this) do {
				case "STRING" : {_this};
				case "OBJECT" : {typeOf _this};
				default {nil}
			}
		) >> "turrets"
	);
	[_class, []] call _findRecurse;
_result;
};

//////// get vehicle current ammo levels
jac_fnc_getVehicleAmmoDef =
{
	params ["_vehicle"];
	
	_totalCurAmmo = 0; _getVehicleAmmoDef = 0; _VehAmmoArray = [];
	
	// default ammo per magazine
	_VehAmmoArray =  getArray (configFile >> "CfgVehicles" >> typeOf _vehicle >> "Turrets" >> "MainTurret" >> "magazines");

	_defTotalAmmo = 0;
	{
		_defTotalAmmo = _defTotalAmmo + (getNumber (configFile >> "CfgMagazines" >> _x >> "count"));
	} forEach _VehAmmoArray;

	if ("Offroad_01_armed_F" in typeOf _vehicle) then {_defTotalAmmo = 400};   // cheat as it's not defined in CfgVehicles

	_totalCurAmmo = 0;
	{
		_totalCurAmmo = _totalCurAmmo + (_x select 1);
	} forEach (magazinesAmmo _vehicle);

	_getVehicleAmmoDef = 0;

	if (_defTotalAmmo != 0) then {_getVehicleAmmoDef = (_totalCurAmmo/_defTotalAmmo)}
	else
	{_getVehicleAmmoDef= -4};
	_getVehicleAmmoDef;
};


//// Function to find a point along a line between two 3D coordinates at a specified distance from point A
//_interpolatedPoint = [_pointA, _pointB, _distanceFromA] call jac_fnc_findInterpolatedPoint;
jac_fnc_findInterpolatedPoint = 
{
    params ["_pointA", "_pointB", "_distanceFromA"];

    _totalDistance = _pointA distance _pointB;
	if (_totalDistance <= 0) then {_totalDistance= 1};
    _t = _distanceFromA / _totalDistance;

    _interpolatedPoint = [
        (_pointA # 0) + _t * ( (_pointB # 0) - (_pointA # 0) ),
        (_pointA # 1) + _t * ( (_pointB # 1) - (_pointA # 1) ),
        (_pointA # 2) + _t * ( (_pointB # 2) - (_pointA # 2) )
    ];
_interpolatedPoint;
};


/// returns the position of the nearest door
jac_fnc_findNearestDoor=
{
	params ['_pos'];
	_doorArray= [];
	_selectedDoorPos= [-1];	_doorPosDis= -1;	//return -1 if not found
	_doorName= '';
	_obj= nearestBuilding  _pos;// nearObjects _radius;
	_closestDis= 99999;
		{
			if ( (_x find "door" >= 0) && (_x find "handle" < 0) ) then 
			{
				_doorPos= _obj modelToWorld (_obj selectionPosition [_x, "ViewGeometry", "AveragePoint"]);
				_doorPosDis = (_doorPos distance _pos);
				if (_doorPosDis < _closestDis) then {_selectedDoorPos= _doorPos; _closestDis= _doorPosDis; _doorName=_x};
			};	//array with all nearby doors
		}forEach(selectionNames _obj);
[_selectedDoorPos, _closestDis, _obj, _doorName];	//return array with door position, distance from point and buiding object
};

/// returns the nearest window
jac_fnc_findNearestWindow=
{
	params ['_pos'];
	_WindowArray= [];
	_selectedWindowPos= [-1];	_WindowPosDis= -1;	//return -1 if not found
	_WindowName= '';
	_building= nearestBuilding  _pos;
	_selections= (getAllHitPointsDamage _building);
	_closestDis= 99999;
	
	if ((count _selections) > 0) then
	{
		_selectionNames = _selections # 1;
		_hitPoints = _selections # 0;		
		{
			if ( "glass" in (toLower _x) or {"window" in (toLower _x)} ) then
			{
				_WindowPos= _building modelToWorld (_building selectionPosition [_x, "HitPoints", "AveragePoint"]);
				_WindowPosDis = (_WindowPos distance _pos);
				if (_WindowPosDis < _closestDis) then {_selectedWindowPos= _WindowPos; _closestDis= _WindowPosDis; _WindowName= (_hitPoints # _forEachIndex)};
			};
		}forEach _selectionNames;
	};
	
	[_selectedWindowPos, _closestDis, _building, _WindowName];	//return array with Window position, distance from point and buiding object
};


////for defensive grenade and for grenade throw enemy context menu option
//determine if anyone in the group has a damaging grenade of any kind
jac_getGroupGrenadeList=
{
	params ['_grp'];
	_throwList= [];
	{ _throwList append ((magazines _x) select {( (_x call BIS_fnc_isThrowable) && (getText (configFile >> "CfgAmmo" >> (getText (configFile >> "CfgMagazines" >> _x >> "ammo")) >> "simulation") != 'shotSmokeX') )}) }forEach units _grp;
	_throwList arrayIntersect _throwList;	//return
};

//list of smoke grenades
jac_getGroupSmokeGrenadeList=
{
	params ['_grp'];
	_throwList= [];
	{ _throwList append ((magazines _x) select {( (_x call BIS_fnc_isThrowable) && (getText (configFile >> "CfgAmmo" >> (getText (configFile >> "CfgMagazines" >> _x >> "ammo")) >> "simulation") == 'shotSmokeX') )}) }forEach units _grp;
	_throwList arrayIntersect _throwList;	//return
};

//get list of mines and explosives in the group
jac_getExplosiveList=
{
	params ['_grp'];
	_explosiveList= [];
	{
		_explosiveList append ((magazines _x) select {
		_placeType= (getText (configFile >> "CfgMagazines" >> _x >> "descriptionUse")); _nameSound= (getText (configFile >> "CfgMagazines" >> _x >> "nameSound"));
		( ("satchelcharge" in _nameSound) or ("mine" in _nameSound) or ("Place Mine" in _placeType) or ("Set Charge" in _placeType) ) }); 
	}forEach units _grp;
	_explosiveList arrayIntersect _explosiveList;	//return
};

//// check if there's a valid straight line path to the target position
jac_fnc_linePath=
{
	params ["_startPos", "_endPos"];
	_dis= _startPos distance _endPos; _dir= _startPos getDir _endPos; _checkPos1= _startPos; _haspath= true;
	for "_i" from 0 to (floor _dis) step +1 do {
		_checkPos2= AGLToASL (_checkPos1 getPos [1, _dir]);
		_checkPos2 set [2, (_checkPos1 # 2)];
		//check if the new position is valid, collision line, //the 'GEOM' > 'VIEW' order is to check for fences
		_LOD1= 'GEOM';	_LOD2= 'VIEW';
		_hasLineOfSight= true;	_hasDoor= false;
		_collisionLine= ( (lineIntersectsSurfaces [( (_checkPos1) vectorAdd [0,0,0.5]), ( (_checkPos2) vectorAdd [0,0,0.5]), objNull, objNull, true, 1, _LOD1, _LOD2, true]) );		
		_checkPos1= _checkPos2;
		
		//check if the collision should be ignored for some special cases
		if ( (count _collisionLine) > 0 ) then 
		{
			_hasLineOfSight= false; _collisionPoint= ASLToAGL(_collisionLine # 0 # 0);									
		
			//ignore collision with a door
			_isDoor= [_collisionPoint] call jac_fnc_findNearestDoor;
			if ( ((_isDoor # 0 # 0) != -1) && ((_isDoor # 1) < 1) ) then {_hasLineOfSight= true; _hasDoor=true};	//door close enough to the collision point
			
			//ignore collision with infantry
			if ( ((_collisionLine # 0) # 2) isKindOf 'CAManBase' ) then {_hasLineOfSight= true};
		};
		
		_hasGround= false; _lineCollisionGround= (lineIntersectsSurfaces [(_checkPos2 vectorAdd [0,0,1]), (_checkPos2 vectorAdd [0,0, -10]), objNull, objNull, true, 1, "GEOM", "FIRE", true]);
		if ((count _lineCollisionGround) > 0) then {_hasGround= true; _checkPos2 set [2, ((_lineCollisionGround # 0) # 0 # 2)]};
		if (!(_hasLineOfSight) or !(_hasGround)) exitWith {_haspath= false};	//stop if found an invalid position
	};
	_haspath;
};


//// get the area selected with the mouse on the 3D scene
// used for set Ambush and fire support area 
jac_getMouseRectangleAreaSelection=
{
	params ["_ambushKillzoneP1","_ambushKillzoneP3"];
	
	_dis= _ambushKillzoneP1 distance2D _ambushKillzoneP3;	
	if (_dis == 0) exitWith {[_ambushKillzoneP1 vectorAdd [0,0,5], _ambushKillzoneP1 vectorAdd [0,0,5], _ambushKillzoneP1 vectorAdd [0,0,5], _ambushKillzoneP1 vectorAdd [0,0,5], _ambushKillzoneP1 vectorAdd [0,0,5], 1, 1, 0]};	//prevent errors if area given is too small	
	
	//sides
	_hypotenuse = _dis *0.5;
	_sideA = abs ((_ambushKillzoneP1 # 0) - (_ambushKillzoneP3 # 0)) *0.5;
	_sideB = abs ((_ambushKillzoneP1 # 1) - (_ambushKillzoneP3 # 1)) *0.5;

	///center point of the target area
	_hypothenuseDir= _ambushKillzoneP1 getDir _ambushKillzoneP3;
	_center= [_ambushKillzoneP1, _ambushKillzoneP3, _hypotenuse] call jac_fnc_findInterpolatedPoint;
	_center set [2, 0];

	_ii= 0; _iii= 1; _pointsArr= []; _pointsArr2= [];		//store the 4 corners of the rectangle
	_orientationSize= [(_sideA), (_sideB)];
	_z= 2; _direction= 90;
	
	for "_i" from 0 to 3 do
	{
		_referencePoint= [_center, (_orientationSize # _ii), ( (_direction + (90 *_i)) mod 360 )] call BIS_fnc_relPos;	//from the center find the reference point
		_fromReferencePoint= [_referencePoint, (_orientationSize # _iii), ( ((_direction) + (90 *(_i +1)) ) mod 360 )] call BIS_fnc_relPos;	//from the reference point find the corner
		_fromReferencePoint set [2, 0];
		_fromReferencePoint deleteAt 2;
		_pointsArr pushBack _fromReferencePoint;
		_ii= (( _ii + 1 ) mod 2);	//0 or 1				
		_iii= (( _ii + 1 ) mod 2);				
	};
	
	_point1= (( (_pointsArr # 0)) vectorAdd [0,0,_z]); _point2= (( (_pointsArr # 1)) vectorAdd [0,0,_z]); 
	_point3= (( (_pointsArr # 2)) vectorAdd [0,0,_z]); _point4= (( (_pointsArr # 3)) vectorAdd [0,0,_z]);
	_pointCenter= (( _center) vectorAdd [0,0,_z]);
		
	[_point1, _point2, _point3, _point4, (AGLToASL _pointCenter), _sideA, _sideB, _direction];	//return positions AGL, center is position ASL 
};


////// get list of all curators that can control the given object
jac_fnc_objectCurators=
{
	params ['_object'];
	( (allCurators) select {( _object in (curatorEditableObjects _x) )} );
};


////function to call before you do createVehicle or other create* command during missing init scripts, which should activate missing addons without having to worry about re-adding previously added addons
//-arg= class of object to add _className call jac_fnc_prepareClassAddons;
//author: samatra - source: https://community.bistudio.com/wiki/activateAddons
jac_fnc_prepareClassAddons = 
{
	private _class = toLower _this;

	// To avoid double checks
	if(isNil"var_addonCheckedClasses") then {var_addonCheckedClasses = createHashMap;};
	if(_class in var_addonCheckedClasses) exitWith {};

	// Finding missing addons
	private _needed = unitAddons _class apply {toLower _x};
	private _active = activatedAddons;
	private _missing = _needed - (_needed arrayIntersect _active);
	if(count _missing > 0) then {
		// Adding everything again, engine will figure it out itself
		_active append _missing;
		activateAddons _active;
	};

	var_addonCheckedClasses set [_class, _needed];
};

/// change the combat mode quickly to instantly stop doSuppressiveFire action
jac_fnc_resetSuppressAction=
{
	params ["_unit", "_grp"];
	/// change combat mode to stop suppressing/engaging current target
	if (((_unit) getVariable ['my_og_combat_mode_do_change', true])) then
	{
		_unit setVariable ['my_og_combat_mode_do_change', false, true];		
		_unit setUnitCombatMode "BLUE";
		sleep 0.5;
		//return to previous combat mode
		_unit setUnitCombatMode (CombatMode _grp);
		_unit setVariable ['my_og_combat_mode_do_change',true, true];
	};
};


//// disable simulation for units placed in preparation phase and make enemies invisible in multiplayer on init
jac_fnc_preparationPhaseUnitDisableInit=
{
	params ["_unit"];
	sleep 0.25;
	//if there's a preparation phase disable simulation until combat phase starts
	if (jac_preparationPhaseTimer > 0) then
	{
		[_unit] spawn { params ["_unit"];
			while {(CBA_missionTime < (jac_preparationPhaseTimer +1))} do
			{
				if ((local _unit) && !(_unit getVariable ['jac_blackListFromValidUnits', false])) then 
				{_unit enableSimulationGlobal false; waitUntil {sleep 0.1; (CBA_missionTime > jac_preparationPhaseTimer)}; _unit enableSimulationGlobal true};
				sleep 0.1;
			};
		};	
	};
	
	if ( (isNull (objectParent _unit)) && (isMultiplayer) && (jac_doCallFogOfWarLocalVisibility) && (jac_isFowModeActive) && (jac_isWargameSpottingActive) && !(_unit getVariable ['jac_blackListFromValidUnits', false]) ) then
	{
		if ([(side player), (side _unit)] call BIS_fnc_sideIsEnemy) then
		{
			_waitTime= CBA_missionTime;
			waitUntil {sleep 1; ( (!(local _unit) && (jac_isWargameZeus)) or (CBA_missionTime > (30 + _waitTime )) )};
			
			if (CBA_missionTime > (30 + _waitTime )) exitWith {terminate _thisScript};
		
			if ( (!(local _unit) && (jac_isWargameZeus)) && (alive _unit) ) then 
			{
				if (((_unit getVariable ["jac_spottedMeList", []]) findif {side _x == (side player)}) == -1) then {_unit hideObject true; _unit setVariable ["jac_hiddenByFow", true]};
			};
		};
	};
};

///// DRAW CUSTOM 3D SHAPE
jac_fnc_drawRectangle3D=
{
	params ["_xSize", "_ySize", "_z", "_rgb", "_id", "_center", "_direction"];
	
	_ii= 0; _iii= 1; _pointsArr= []; _pointsArr2= [];		//store the 4 corners of the rectangle
	_orientationSize= [(_ySize), (_xSize)];
	
	for "_i" from 0 to 3 do
	{
		_referencePoint= [_center, (_orientationSize # _ii), ( (_direction + (90 *_i)) mod 360 )] call BIS_fnc_relPos;	//from the center find the reference point
		_fromReferencePoint= [_referencePoint, (_orientationSize # _iii), ( ((_direction) + (90 *(_i +1)) ) mod 360 )] call BIS_fnc_relPos;	//from the reference point find the corner
		_fromReferencePoint set [2, _z];
		_pointsArr pushBack _fromReferencePoint;
		_ii= (( _ii + 1 ) mod 2);	//0 or 1
		_iii= (( _ii + 1 ) mod 2);
	};
	_point1= (_pointsArr # 0); _point2= (_pointsArr # 1); _point3= (_pointsArr # 2); _point4= (_pointsArr # 3);
	
	jac_drawLineArray3D pushBack [_point1,_point2, _rgb, _id];
	jac_drawLineArray3D pushBack [_point2,_point3, _rgb, _id];
	jac_drawLineArray3D pushBack [_point3,_point4, _rgb, _id];
	jac_drawLineArray3D pushBack [_point4,_point1, _rgb, _id];
};

jac_fnc_delete3DShape=
{
	params ["_id"];
	for '_i' from (count jac_drawLineArray3D) - 1 to 0 step -1 do {
		if (((jac_drawLineArray3D # _i) # 3) isEqualTo _id) then {jac_drawLineArray3D deleteAt _i};
	};
};


// holster current weapon
jac_unitHolsterWeapon=
{
	params ["_unit"];
	_weapon= (primaryWeapon _unit); _magazine= primaryWeaponMagazine _unit; _items= primaryWeaponItems _unit;
	_weaponInfo= [_weapon, _magazine, _items];

	_unit setVariable ["jac_holsteredWeapon", _weaponInfo, true];
	_unit removeWeapon (primaryWeapon _unit);

	[{
		params ['_unit'];
		_unit action ['SwitchWeapon', _unit, _unit, 100];
	}, [_unit], 1] call CBA_fnc_waitAndExecute;	
};

// unholster a previously holstered weapon
jac_unitUnholsterWeapon=
{
	params ["_unit"];
	_weaponInfo= _unit getVariable ["jac_holsteredWeapon", []];
	if (count _weaponInfo >= 3) then
	{
		{_unit addMagazine _x}forEach (_weaponInfo # 1);
		_unit addWeapon (_weaponInfo # 0);
		{_unit addPrimaryWeaponItem _x}forEach (_weaponInfo # 2);
		
		[{
			params ['_unit'];
			_unit selectWeapon (primaryWeapon _unit);
			_unit setVariable ["jac_holsteredWeapon", [], true];
		}, [_unit], 0.5] call CBA_fnc_waitAndExecute;	
	}
	else
	{_unit setVariable ["jac_holsteredWeapon", [], true]};
};


/// remove unit from the population capacity list
jac_fnc_populationListRemoveUnit=
{
	params ["_unit", "_checkSide"];
	
	if ( (_checkSide) && ([(side player), (side _unit)] call BIS_fnc_sideIsFriendly) ) exitWith {false};	//don't remove if the unit entering the empty vehicle is friendly to the owner (prevent cheating)
	
	_findUnit= (jac_zeusUnitsAddedPopList findIf {_x isEqualTo _unit});
	if (_findUnit != -1) then 
	{
		_myTypeOf= (typeOf _unit);
		jac_zeusUnitsAddedPopList deleteAt _findUnit; 
		player setVariable ["jac_zeusUnitsAddedPopListVar", jac_zeusUnitsAddedPopList, true];
		
		_findType= jac_zeusUnitCostModuleCustomLoadout findIf {_x # 0 isEqualTo _myTypeOf};
		if (_findType != -1) then
		{
			_populationCost= (jac_zeusUnitCostModuleCustomLoadout # _findType # 5);
			jac_zeusUnitsAddedPopCost= (jac_zeusUnitsAddedPopCost - _populationCost) max 0;
			player setVariable ["jac_zeusUnitsAddedPopCostVar", jac_zeusUnitsAddedPopCost, true];
		};
	};
};


jac_fnc_unitCostModuleRewards=
{
	params ["_unit", "_killer"];
	
	_myTypeOf= (typeOf _unit);	
	_findType= jac_zeusUnitCostModuleCustomLoadout findIf {_x # 0 isEqualTo _myTypeOf};
	if (_findType != -1) then
	{
		_zeusList= [_killer] call jac_fnc_objectCurators;
		if (count _zeusList > 0) then
		{
			_unitCostModuleZeusReward= (jac_zeusUnitCostModuleCustomLoadout # _findType # 8);
			if (_unitCostModuleZeusReward > 0) then
			{
				_unitCostModuleZeusReward= (_unitCostModuleZeusReward *0.001);
				{[_x, _unitCostModuleZeusReward] remoteExec ["addCuratorPoints", 2]} forEach _zeusList;
			};
		};
		
		_unitCostModuleVPReward= (jac_zeusUnitCostModuleCustomLoadout # _findType # 9);
		if (_unitCostModuleVPReward > 0) then
		{
			_side= (side _killer);
			[_side, _unitCostModuleVPReward] remoteExecCall ["jac_fnc_setVictoryPoints", 0]
		};
	};
};


/// mark spotted mines on the map
jac_fnc_markSpottedMines=
{
	[{
		if (jac_markSpottedMines) then
		{
			_side= (side player);
			_minesDetected= allMines select {!(_x getVariable ["jac_spottedMine", false])};
			{
				if !(_x getVariable ["jac_spottedMine", false]) then
				{
					if (_x mineDetectedBy _side) then
					{
						_x setVariable ["jac_spottedMine", true];
						// draw mine spotted indicator until the mine is removed
						_targetPos= (position _x);
						_key= format ["%1%2", (str _x)];						
						_x setVariable ["jac_spottedMineKey", _key];
						jac_draw3dIconList pushBack [ _targetPos, '\a3\Ui_F_Curator\Data\CfgMarkers\minefieldAP_ca.paa', _key];
						
						//add event handlers to check if mine was removed						
						_x addEventHandler ["Deleted", {
							params ["_entity"];
							_key= _entity getVariable ["jac_spottedMineKey", (str _entity)];
							{if ((_x # 2) isEqualTo _key) then {jac_draw3dIconList deleteAt _forEachIndex}}forEach jac_draw3dIconList;
						}];
					};
				};
			}forEach allMines;
		};
	} , 8, []] call CBA_fnc_addPerFrameHandler;
};


/// set this entity to be affected by the fog of war system for players that are hostile to the given side
//used for empty vehicles
jac_fnc_setIsFogOfWarUnit=
{
	params ["_entity", "_side"];
	_entity setVariable ["jac_isFogOfWarUnit", false];
	if ( (alive _entity) && ([(side player), _side] call BIS_fnc_sideIsEnemy) ) then 
	{
		// if should be made invisible for this player
		if ( !(local _entity) && (jac_isWargameZeus) ) then 
		{
			if ( (isMultiplayer) && (jac_doCallFogOfWarLocalVisibility) && (jac_isFowModeActive) && (jac_isWargameSpottingActive) && !(_entity getVariable ['jac_blackListFromValidUnits', false]) ) then
			{
				if (((_entity getVariable ["jac_spottedMeList", []]) findif {side _x == (side player)}) == -1) then 
				{_entity hideObject true; _entity setVariable ["jac_hiddenByFow", true]};
			};
		};
	};
};


/// delete the context sensitive menu handler logic object
jac_fnc_deleteContextSensitiveMenuHandler=
{
	params ["_ctrlPos", "_ctrlOBJ", "_grpArray", "_optionalArgument"];
	deleteVehicle _ctrlOBJ;
};


/// call when unit is suppressed or hit to make it look towards the shooter
jac_AILookAtPotentialTargetLocation=
{
	params ["_unit", "_shooter"];
	//distance the enemy must be within for the improved awareness system to make the ai look at someone shooting at it
	_dis= jac_lookAtShooterDis; _isVeh= !((vehicle _unit) isKindOf "CAManBase");
	if (_isVeh) then {_dis= jac_lookAtShooterDisVeh};
	
	if ((_unit distance _shooter) < _dis) then
	{
		if (_unit checkAIFeature "MOVE") then 
		{
			if !(_unit getVariable ['jac_doSuppressAction', false]) then
			{
				if ( !((vehicle _shooter) isKindOf "CAManBase") or (_shooter weaponAccessories (currentWeapon _shooter) param [0, ""] == "") ) then  //no suppressor or shooter is vehicle
				{
					if ( !(alive (_unit getVariable ['jac_myCurrentTarget', objNull])) && !(alive (_unit getVariable ['jac_playerSetTarget', objNull])) ) then  //has no target currently
					{
						_unit reveal _shooter; _unit lookAt _shooter; _unit doWatch _shooter; _unit doTarget _shooter; //keep tracking target
					};
				};
			};
		};
	};
};

/// check if a target direction is within the given field of view
//[_dir, _targetDir, _FOV] call jac_targetDirInFOV;
jac_targetDirInFOV=
{
	params ["_dir", "_targetDir", "_FOV"];

	// Calculate the raw difference.
	_angleDiff = _targetDir - _dir;
	
	// Normalize the angle difference to the range -180 to 180
	if (_angleDiff > 180) then 
	{
		_angleDiff = _angleDiff - 360;
	} else 
	{
		if (_angleDiff < -180) then {_angleDiff = _angleDiff + 360};
	};
	
	// return whether the target is within the field of view or not
	((abs _angleDiff) < _FOV);
};


/// calls the pathfinder, the path found is saved on jac_pathAgent; "man"; "wheeled_APC"
jac_fnc_callPathfinder=
{
	params ["_positionStart", "_positionEnd", "_uniType"];
	
	[{!jac_isSearchingPath}, 
	{
		params ["_positionStart", "_positionEnd", "_uniType"];
		
		jac_pathAgent= []; jac_isSearchingPath= true;
		
		(calculatePath [_uniType, "SAFE", _positionStart, _positionEnd]) addEventHandler ["PathCalculated", {
			params ["_unit", "_path"];
		
			/*///debugging path
			{
				deleteMarker ("marker" + str _forEachIndex);
				_marker = createMarker ["marker" + str _forEachIndex, _x];
				_marker setMarkerType "mil_dot"; _marker setMarkerText str _forEachIndex;
			} forEach _path;
			/**/
			
			// For some unknown reason, the "PathCalculated" Event Handler is fired twice, first with the calculated path and second with an array consisting of 2 elements, which are identical and are equal to the end point
			if !(_unit getVariable ["jac_pathAgentSet", false]) then {jac_pathAgent= _path; _unit setVariable ["jac_pathAgentSet", true]; jac_isSearchingPath= false};
		}];	
	}, [_positionStart, _positionEnd, _uniType]] call CBA_fnc_waitUntilAndExecute;
};


/// AT quick shot - force AT soldier to quickly fire at target
jac_fnc_ForceFireATWeaponAtTarget=
{
	params ["_unit", "_weapon", "_target", "_delay"];
	
	if ( (_unit getVariable ["jac_isForceFiringAtTarget", false]) or !(_unit getVariable ['jac_endScriptedMove', true]) ) exitWith {false};	//not occuppied		
	if ( (isPlayer _unit) or !(isNull (remoteControlled _unit)) ) exitWith {false};	//not player controlled
	if (speed _target > 80) exitWith {false};	//is a proper target	
	if (_unit ammo _weapon == 0) exitWith {false}; //has ammo loaded
	
	_unit setVariable ["jac_isForceFiringAtTarget", true];
	_unit setVariable ["jac_ATLockTarget", _target];
	_unit switchMove "amovpercmstpsraswlnrdnon";
	_unit selectWeapon _weapon;
	_unit disableAI "ANIM";
	
	// event handler to guide rocket
	_handler= _unit addEventHandler ["FiredMan", {
		params ["_unit", "_weapon", "_muzzle", "_mode", "_ammo", "_magazine", "_projectile", "_vehicle"];

		if (isNull _projectile) then {_projectile = nearestObject [_unit, _ammo]};
		_projectile disableCollisionWith _unit;
		_ATLockTarget= (_unit getVariable ['jac_ATLockTarget', objNull]);
		
		if !(isNull _ATLockTarget) then
		{
			_speed= (((speed _projectile) *0.75) max 100);
			[_projectile, _ATLockTarget, _speed, jac_baseProjectileMinDistanceTarget, false] call jac_fnc_ScriptedMoveProjectileTarget;
		};

		_unit setVariable ["jac_ATLockTarget", objNull];
		_unit removeEventHandler [_thisEvent, _thisEventHandler];
	}];
	_unit setVariable ["jac_eventHandlerFiredMan", _handler];
	
	// waitUntil weapon is equipped
	_timer= CBA_missionTime +3;
	_PARAMS= [_unit, _weapon, _target, _delay, _timer];
	[
		{params ["_unit", "_weapon", "_target", "_delay", "_timer"];
		(currentWeapon _unit == _weapon) or !(alive _unit) or !(alive _target) or (_timer <= CBA_missionTime)},
	{
		params ["_unit", "_weapon", "_target", "_delay", "_timer"];
		_unit setDir ((getPosATL _unit) getDir (getPosATL _target));
		_unit setPos (getPosATL _unit);
		_unit reveal [_target, 4]; _unit lookAt _target; _unit doWatch _target; _unit doTarget _target;

		// waitUntil is aimed and shoot	
		_PARAMS= [_unit, _weapon, _target, _delay];
		[{
			params ["_unit", "_weapon", "_target", "_delay"];
			
			if ((alive _unit) && (alive _target) && (_unit getVariable ['jac_endScriptedMove', true])) then
			{
				_collisionLine= (lineIntersectsSurfaces [(eyePos _unit), (aimPos _target), (vehicle _unit), _target, true, 1, 'GEOM', 'VIEW', true]);
				if ((count _collisionLine) == 0) then 
				{
					_unit setDir ((getPosATL _unit) getDir (getPosATL _target));
					_unit forceWeaponFire [((_unit weaponsInfo [_weapon, true]) # 0 # 3 ), (getArray (configFile >> "CfgWeapons" >> _weapon >> "modes"))#0];
				};
			};
			
			[{
				params ["_unit"];
				_unit enableAI "ANIM";
				_unit setVariable ["jac_isForceFiringAtTarget", false];
				_unit removeEventHandler ["FiredMan", (_unit getVariable ["jac_eventHandlerFiredMan", -1])];
			}, _unit, 1.5] call CBA_fnc_waitAndExecute;
		}, _PARAMS, _delay] call CBA_fnc_waitAndExecute;
	}, _PARAMS] call CBA_fnc_waitUntilAndExecute;
};


/// Converts a 3D vector [x, y, z] to [yaw, pitch] in degrees.
//Yaw: Horizontal angle from North (0°), increasing clockwise (0° to 360°).
//Pitch: Vertical angle from the horizontal plane (-90° to 90°).
jac_fnc_vectorToDegrees=
{
	params ["_vector"];
	_xx = _vector # 0;
	_yy = _vector # 1;
	_z = _vector # 2;

	// Normalize the vector
	_mag = sqrt (_xx^2 + _yy^2 + _z^2);
	if (_mag == 0) exitWith { [0, 0] }; // Handle zero vector

	_x_norm = _xx / _mag;
	_y_norm = _yy / _mag;
	_z_norm = _z / _mag;

	// Calculate yaw (0° to 360°)
	_yaw = 0;
	if (_x_norm != 0 || _y_norm != 0) then 
	{
		_yaw = ( 90 - (_y_norm atan2 _x_norm) ) mod 360;
		
		// Ensure non-negative result
		if (_yaw < 0) then { _yaw = _yaw + 360 };
	};

	// Calculate pitch (-90° to 90°)
	_pitch = asin(_z_norm);
	[_yaw, _pitch];
};

/// finds the nearest building that has building positions
jac_fnc_nearestBuildingPositions=
{
	params ["_position"];
	_targetBuilding= objNull; _buildingPositions= []; _buildingPositionsCount= 0;
	
	_objArray= nearestObjects [_position, ["BagBunker_base_F", "House", "Building", "HouseBase"], 9];
	_objId= (_objArray findIf{ _buildingPositions= (_x  buildingPos -1 ); _buildingPositionsCount= count _buildingPositions; (_buildingPositionsCount > 0) });
	if (_objId != -1) then {_targetBuilding= _objArray # _objId};

	if (isNull _targetBuilding) then
	{
		_objArray= nearestTerrainObjects [_position, [], 9, true, true];
		_objId= (_objArray findIf{ _buildingPositions= (_x  buildingPos -1 ); _buildingPositionsCount= count _buildingPositions; (_buildingPositionsCount > 0) });
		if (_objId != -1) then {_targetBuilding= _objArray # _objId};
	};
	[_targetBuilding, _buildingPositions, _buildingPositionsCount];
};

/// returns a list of building positions near the given target position (pos AGL) sorted by their distance and whether they are occupied
jac_fnc_nearestBuildingPositionsSorted=
{
	params ["_targetPos"];

	_buildingPosArrayOrdered= []; _closest= 999999; _buildingPosArrayOrderedFreeCount= 0;

	_objArray= (nearestTerrainObjects [_targetPos, [], 25, false, false]);
	_objArray insert [-1, (nearestObjects [_targetPos, ["BagBunker_base_F", "House", "Building", "HouseBase"], 25]), true];

	{
		_building= _x;
		{
			_dis= (_x distance _targetPos);
		   _index= _buildingPosArrayOrdered findIf {(_dis <= (_closest)) };
			if (_index != -1) then {_closest= _dis};
			
			//check if position is occupied (look for units < 1.25 meters)
			_isOccupied= (nearestObjects [_x, ["CAManBase"], 1.25] findIf {((alive _x) && !(_x getVariable["jac_blackListFromValidUnits", false]))} != -1);
			_buildingPosArrayOrdered insert [_index, [ [_x, _isOccupied, _building] ]];
			
			if !(_isOccupied) then {_buildingPosArrayOrderedFreeCount= _buildingPosArrayOrderedFreeCount +1};
		}foreach ([_x] call CBA_fnc_buildingPositions);
	}foreach _objArray;
	
	[_buildingPosArrayOrdered, _buildingPosArrayOrderedFreeCount, _closest];
};

/// units will enter target building and occupy positions inside
jac_fnc_wp_occupyBuildingSorted=
{
	params ["_buildingGarrisonDrawArray", "_garrisonTeleport", "_units"];	//first 3 arg are obligatory

	_targetDis= if (_garrisonTeleport) then { 999999} else {2.5};
	_buildingPosArrayOrdered= (_buildingGarrisonDrawArray # 0 # 0);

	_buildingPosCount= (count _buildingPosArrayOrdered);
	_posIndex= 0; _position= [];
	{
		_unit= _x;
		_unit doFollow (leader _unit);
		if ((leader  _x) == _x) then 
		{
			(group _x) setVariable ['jac_doFormUp', false, true];
			[200, (leader _x), "OCCUPY THAT BUILDING!"] remoteExecCall ["jac_fnc_zeusAddMSG3D", 0];
		}; //cancel stacking behind vehicle
		_unit setVariable ['jac_doMoveCoverPos', [], true];

		if ( !((group _unit) getVariable ['jac_isDugIn', false]) && (_unit getVariable ['jac_isInCover', false]) ) then {_unit enableAi 'PATH'};	//enable path if in cover, but not dug in
		_moveToMarker= _unit getVariable ['jac_moveToMarker', objNull]; deleteVehicle _moveToMarker;		
		_unit setVariable ['jac_isMovingToCoverPos', false, true];
		_unit setVariable ['jac_pathCalculated', false, true];
		_unit setVariable ['jac_endMovingToCoverPos', true, true];
		_unit setVariable ['jac_endScriptedMove', true, true]; //end scripted move
		_unit setVariable ["jac_isClearingBuilding", false, true];

		//get pos to move to
		_posIndex= (_posIndex + 1);
		if (_posIndex < _buildingPosCount) then 
		{_position = (_buildingPosArrayOrdered # _posIndex # 0)}
		else
		{
			_posIndex= floor (random _buildingPosCount);
			_position = selectRandom _buildingPosArrayOrdered;
		};
		
		_isOccupied= (_buildingPosArrayOrdered # _posIndex # 1);
		_targetBuilding= (_buildingPosArrayOrdered # _posIndex # 2);
		_buildingSize= ((0 boundingBoxReal _targetBuilding) # 2);
		
		_unit setVariable ['jac_doMoveGarrisonPos', _position];
		[_unit, _position, _targetDis, _buildingSize, _targetBuilding] remoteExec ["jac_fnc_unitGarrison", 0];	//move unit to its garrison position
	}forEach _units;
};

/// move unit to its garrison position - used by jac_fnc_wp_occupyBuilding.sqf and jac_fnc_wp_occupyBuildingSorted
jac_fnc_unitGarrison=
{
	params ["_unit", "_position", "_targetDis", "_buildingSize", "_targetBuilding"];

	if !(local _unit) exitWith {false};

	_moveToMarker = createSimpleObject ["Sign_Arrow_Green_F", (AGLToASL _position), false];
	sleep 0.1;
	_moveToMarker hideObjectGlobal true;
	_unit doMove _position;

	//for all zeus that can control the group, unhide the marker
	[[_moveToMarker, _unit],
	{
		params ["_moveToMarker", "_unit"];
		_zeusList=[]; {_zeusList pushBack (owner _x)}forEach ( (allCurators) select {( _unit in (curatorEditableObjects _x) )} );		
		[[_moveToMarker],
		{
			params ["_moveToMarker"];
			if (!(isNull findDisplay 312) && (jac_doDraw3Dmarker)) then {_moveToMarker hideObject false};	//unhide marker for me
		}] remoteExecCall ['call', _zeusList];
	}] remoteExecCall ['call', 2];

	_garrisonUnitsDoTeleport= false;
	waitUntil 
	{
		sleep 0.001; _dis= ((getPosATL _unit) distance _position);
		if (leader _unit == _unit) then
		{
			if ( !(_garrisonUnitsDoTeleport) && (((getPosATL _unit) distance _targetBuilding) < _buildingSize) ) then
			{
				_garrisonUnitsDoTeleport= true;
				[{
					params ["_position", "_unit", "_buildingSize", "_targetBuilding"];
					_moveToPos= _unit getVariable ['jac_doMoveGarrisonPos', []];
					if ( (_position isEqualTo _moveToPos) && ((count _moveToPos) > 1) && (alive _unit) ) then
					{
						if ( (((getPosATL _unit) distance _moveToPos) > 2.5) && ( ((getPosATL _unit) distance _moveToPos) < 500 ) ) then
						{
							if ((_targetBuilding nearEntities ["CAManBase", (_buildingSize *0.5)]) findIf { ([(side _unit), (side _x)] call BIS_fnc_sideIsEnemy) } == -1) then
							{_unit setPosATL _moveToPos; doStop _unit};
						};
					};
					_unit setVariable ['jac_doMoveGarrisonPos', []];
				}, [_position, _unit, _buildingSize, _targetBuilding], 20] call CBA_fnc_waitAndExecute;
			};
		};
		((unitReady _unit) or (_dis < _targetDis));
	};

	deleteVehicle _moveToMarker; sleep 1;

	//take cover
	if (_unit distance _position < _targetDis) then 
	{
		doStop _unit; _unit setPosATL _position; sleep 0.25; doStop _unit;
		
		//damage nearest window
		[getPosATL _unit] call jac_fnc_findNearestWindow params ["_selectedWindowPos", "_closestDis", "_building", "_WindowName"];
		if (_closestDis < 4) then {_building setHitPointDamage [_WindowName, 1]};

		//after garrison check for cover around the unit
		_heading= [0,45,90,135,180,225,270,315]; //_heading is used to have control over the direction cover will be searched
		_coverPosDir= [(getPosASL _unit), 2.5, _heading] call jac_fnc_findCoverFromPos;
		if (count _coverPosDir > 0) then {[_unit, (getPosASL _unit), (_coverPosDir # 2)] call jac_scriptedMoveTakeCover};				
	
		// teleport stragglers to their garrison position
		{
			_moveToPosOG= _x getVariable ['jac_doMoveGarrisonPos', []];
			[{
				params ['_moveToPosOG', '_unit'];
				_moveToPos= _unit getVariable ['jac_doMoveGarrisonPos', []];
				if ( (_moveToPosOG isEqualTo _moveToPos) && ((count _moveToPos) > 1) && (alive _unit) ) then
				{
					if ( (((getPosATL _unit) distance (_moveToPos)) > 2.25) && (((getPosATL _unit) distance (_moveToPos)) < 500) ) then {_unit setPosATL (_moveToPos); doStop _unit};
					};
			}, [_moveToPosOG, _x], 8] call CBA_fnc_waitAndExecute;
		}forEach (units (group _unit));
	};
};


/// function to exit all special placement mode
jac_fnc_endSpecialPlacementModes=
{
	params [];
	
	/// cancel set ambush area mode
	if (jac_setAmbushArea) then	
	{
		jac_ambushKillzoneP1=[]; jac_ambushGroups=[]; jac_setAmbushArea=false;
		titleText ["EXITED SET AMBUSH MODE", "PLAIN"];
	};

	/// cancel set fire support target area
	if (jac_setFireSupportArea) then 
	{
		jac_setFireSupportArea=false;
		jac_mouse3DObject= "Sign_Arrow_Green_F"; 
		titleText ["CANCELED SET TARGET AREA", "PLAIN"];
	};

	/// cancel FORTIFICATION MODE
	if (jac_setBuildingArea) then 
	{
		jac_setBuildingAreaP1=[]; jac_setBuildingArea=false;
		titleText ["EXITED FORTIFICATION MODE", "PLAIN"];
	};

	/// cancel SUPPRESSION AREA MODE
	if (jac_setSuppressArea) then
	{
		jac_suppressAreaP1= []; jac_suppressAreaP2= []; jac_setSuppressArea=false;
		titleText ["CANCELED SET AREA SUPPRESSION", "PLAIN"];
	};
	
	/// cancel grenade wp and laser designator wp modes
	jac_setGrenadeWPMode= false;
	jac_setLaserDesignatorWPMode= false;
	
	/// cancel air support
	jac_setAirSupportArea= false;

	jac_mouse3DObject= "Sign_Arrow_Green_F";
	jac_customMouseIconActive= false;
	jac_customMouseIcon= "";
	hintSilent "";

	[{
		titleText ["", "PLAIN"]
	}, [], 3] call CBA_fnc_waitAndExecute;
};

//Convert seconds to H:MM:SS format
jac_fnc_formatSecondsTime=
{
	params ["_totalSeconds"];
	// Helper function to pad single-digit numbers with leading zero
	_twoDigits = 
	{
		private _num = _this;
		private _str = str _num;
		if (_num < 10) then { "0" + _str } else { _str };
	};

	// Format each component
	_remaining = _totalSeconds % 3600;
	_hoursStr = str (floor (_totalSeconds / 3600));
	_minutesStr = (floor (_remaining / 60)) call _twoDigits;
	_secondsStr = (round (_remaining % 60)) call _twoDigits;

	// Combine into final string
	format ["%1:%2:%3", _hoursStr, _minutesStr, _secondsStr];
};

/// returns a list of all turrets in the vehicle and its weapons 
//[_unit] call jac_fnc_returnAllTurretsWeapons params ["_allTurrets", "_allWeapons"];
jac_fnc_returnAllTurretsWeapons=
{
	params ["_unit"];
	
	_turretArray= []; _weaponsArray= [];
	
	for "_i" from -1 to 10 step +1 do	//main turrets
	{
		_weaponsTurret= (_unit weaponsTurret [_i]) select { (!("MASTERSAFE" in _x) && !("horn" in toLower _x)) };
		if (count _weaponsTurret > 0) then
		{
			_turretArray pushBack [_i];
			_weaponsArray pushBack _weaponsTurret;
		};
		
		for "_ii" from -1 to 10 step +1 do	//secondary turrets
		{
			_weaponsTurret= (_unit weaponsTurret [_i, _ii]) select { (!("MASTERSAFE" in _x) && !("horn" in toLower _x)) };
			if (count _weaponsTurret > 0) then
			{
				_turretArray pushBack [_i, _ii];
				_weaponsArray pushBack _weaponsTurret;
			};
		};
	};
	[_turretArray, _weaponsArray];	//returns list of turret paths that have weapons and the list of weapons for each turret path
};

/// show debugging variable for each unit
jac_varTest= scriptNull;
jac_fnc_debugVar=
{
	if !(jac_varTest isEqualTo scriptNull) then {removeMissionEventHandler ["Draw3D", jac_varTest]; jac_varTest= scriptNull}
	else
	{
		jac_varTest= addMissionEventHandler ["Draw3D",
		{
			{
				_unit = _x;
				_vehicle = vehicle _x;
				_var= ( _vehicle getVariable ['jac_spottedMeList', []]);

				drawIcon3D ["", [1,0,0,1], getPosATL (_vehicle), 0, 0, 0, str _var, 1, 0.05, jac_ctrlFont];
			}forEach allUnits;
		}];
	};
};











